using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using WillStott.MxAppToolbox;
using osImageManagerLib;
using osImageManagerApp.Properties;

namespace osImageManagerApp
{
    public partial class DlgDICOMCorrection : Form, IMxDlgForm
    {
        private DlgProcDICOMCorrection _dlgProc;
        private bool _surnameFirstQueryUKCTOCS;

        public DlgDICOMCorrection()
        {
            InitializeComponent();
            _surnameFirstQueryUKCTOCS = false;
        }

        public bool SurnameFirstQueryUKCTOCS
        {
            get { return _surnameFirstQueryUKCTOCS; }
            set { _surnameFirstQueryUKCTOCS = value; } 
        }

        public void Initialize(DlgProcDICOMCorrection proc)
        {
            _dlgProc = proc;
        }

        private void DlgDICOMCorrection_Load(object sender, EventArgs e)
        {
            dicomFileComboBox.SelectedIndex = -1;
            DicomFile[] files = _dlgProc.GetDicomFiles();
            foreach(DicomFile file in files)
                dicomFileComboBox.Items.Add(file);
            if (dicomFileComboBox.Items.Count > 0 )
                dicomFileComboBox.SelectedIndex = 0;
        }

        private void dicomFileComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (dicomFileComboBox.SelectedIndex > -1)
            {
                _dlgProc.SelectDicomFile(dicomFileComboBox.Items[dicomFileComboBox.SelectedIndex] as DicomFile);
                DicomRefLabel.Text = _dlgProc.DicomRefLabel;
                errorDetailsTextBox.Text = _dlgProc.ErrorText;
                patientNameTextBox.Text = _dlgProc.PatientNameLabel;
                volRefTextBox.Text = _dlgProc.VolunteerRefLabel;
                monthCalendar.SetDate(_dlgProc.AppointmentDateValue);
            }
        }

        private void viewImageButton_Click(object sender, EventArgs e)
        {
            DlgDICOMViewer dlg = new DlgDICOMViewer();
            dlg.AddDicomPathFilename(_dlgProc.ImagePathFilename);
            dlg.ShowDialog();
        }

        private void okButton_Click(object sender, EventArgs e)
        {
            _dlgProc.SaveRecordValues();    //DoOK updates various fields in the record, so save them in case user decides to cancel

            DialogResult = _dlgProc.DoOK(patientNameTextBox.Text, volRefTextBox.Text, monthCalendar.SelectionStart);

            if (DialogResult == DialogResult.OK)
            {
                if (_dlgProc.SetDetailsForAllFilesInExam(_dlgProc.SelectedDicomFile))
                    Close();
            }
            else
            {
                if (MessageBox.Show(_dlgProc.DoOKMessage, Resources.StrMsgBoxWarning, _dlgProc.DoOKMessageButtons) != DialogResult.Yes)
                    _dlgProc.RestoreRecordValues();     //if the user now presses cancel the record values are unaltered
                else
                {
                    if (_dlgProc.SetDetailsForAllFilesInExam(_dlgProc.SelectedDicomFile))
                    {
                        DialogResult = DialogResult.OK;
                        Close();
                    }
                }
            }
        }

        private void queryUKCTOCSButton_Click(object sender, EventArgs e)
        {
            DlgQueryUKCTOCS dlg = new DlgQueryUKCTOCS();

            dlg.Databases = _dlgProc.Databases;
            dlg.Surname = DlgProcDICOMCorrection.GetSurname(patientNameTextBox.Text, _surnameFirstQueryUKCTOCS);
            dlg.FirstNames = DlgProcDICOMCorrection.GetFirstNames(patientNameTextBox.Text, _surnameFirstQueryUKCTOCS);
            dlg.VolunteerRef = volRefTextBox.Text;

            dlg.ShowDialog();
        }


        public void HelpRequest(object sender, HelpEventArgs hlpevent)
        {
            DlgUtils.ShowHelp(this, "User_Manual_-_Dialog_-_DICOM_Correction.htm", hlpevent);
        }


    }
}