using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using WillStott.MxAppToolbox;
using osImageManagerLib;
using osImageManagerApp.Properties;

namespace osImageManagerApp
{
    public partial class DlgFileExport : Form, IMxDlgForm
    {
        DlgProcFileExport _dlgproc;
        int _delay;
        SelectionMode _selectionMode;

        public DlgFileExport()
        {
            _delay = 0;
            InitializeComponent();
            _dlgproc = new DlgProcFileExport();
            destinationTextBox.Text = osImageManagerProgram.Instance.LastFileExportDir;
            _selectionMode = SelectionMode.MultiExtended; 
        }

        public void Initialize(OsDatabases databases, ImageRecord[] recs)
        {
            _dlgproc.Initialize(databases, recs);
        }

        private void DlgFileExport_Load(object sender, EventArgs e)
        {
            _delay = Utils.GetAppSettingInt(ExecutionTime.Configuration.AppSettings, "ImportExportDelay");
            _dlgproc.LoadFileListbox(fileListBox);
           ProgressTextLabel.Text = GetProgressText();
           _selectionMode = fileListBox.SelectionMode;
           UpdateUI();
        }

        private void browseButton_Click(object sender, EventArgs e)
        {
            ProgressTextLabel.Text = "";
            FolderBrowserDialog dlg = new FolderBrowserDialog();
            dlg.SelectedPath = osImageManagerProgram.Instance.LastFileExportDir;
            dlg.ShowNewFolderButton = false;
            if (dlg.ShowDialog() == DialogResult.OK)
            {
                osImageManagerProgram.Instance.LastFileExportDir = dlg.SelectedPath;
                destinationTextBox.Text = dlg.SelectedPath;
            }
        }

        private void removeButton_Click(object sender, EventArgs e)
        {
            ProgressTextLabel.Text = "";
            int lastRemovedItem = DlgUtils.RemoveSelectedItemFromListBox(fileListBox, -1);
            if ((fileListBox.Items.Count > 0) && (lastRemovedItem != -1))
                fileListBox.SelectedIndex = (lastRemovedItem == fileListBox.Items.Count) ? lastRemovedItem - 1 : lastRemovedItem;
            ProgressTextLabel.Text = GetProgressText();
            UpdateUI();
        }

        private void exportButton_Click(object sender, EventArgs e)
        {
            bool rc = false;
            DisplayProgressStart(Resources.StrExportingImages, 0, fileListBox.Items.Count);

            DicomFile[] files = new DicomFile[fileListBox.Items.Count];
            fileListBox.Items.CopyTo(files, 0);

            try
            {

                for (int index = 0; index < files.Length; index++)
                {
                    DicomFile file = files[index];

                    DisplayProgressStep();

                    _dlgproc.Export(file, restoreMaskCheckBox.Checked, destinationTextBox.Text + "\\");

                    fileListBox.Items.Remove(file);
                }
                rc = true;
            }
            catch (Exception)
            {
                DisplayProgressEnd(files.Length);
                throw;
            }
            if (rc == true)
            {
                DisplayProgressEnd(files.Length);
                DialogResult = DialogResult.OK;
                Close();
            }
        }

        private void DisplayProgressStart(string label, int startValue, int maxValue)
        {
            workProgressBar.Minimum = 0;
            workProgressBar.Step = 1;
            workProgressBar.Maximum = maxValue;
            workProgressBar.Value = startValue;
            ProgressTextLabel.Text = label;
            Refresh();
            _selectionMode = fileListBox.SelectionMode;
            fileListBox.SelectionMode = SelectionMode.None;
            if (_delay == -1)
                fileListBox.BeginUpdate();
        }

        private void DisplayProgressEnd(int itemsProcessed)
        {
            ProgressTextLabel.Text = itemsProcessed.ToString() + " " + ((itemsProcessed == 1) ? Resources.StrImageExported : Resources.StrImagesExported);
            workProgressBar.Value = 0;
            fileListBox.SelectionMode = _selectionMode;
            if (_delay == -1)
                fileListBox.EndUpdate();
            Refresh();
        }

        private void DisplayProgressStep()
        {
            workProgressBar.PerformStep();
            if (_delay != -1)
            {
                ProgressTextLabel.Text = Resources.StrExporting + workProgressBar.Value.ToString() + Resources.StrOf + workProgressBar.Maximum.ToString();
                Refresh();
            }
            if (_delay > 0)
                System.Threading.Thread.Sleep(_delay);
        }

     
        private string GetProgressText()
        {
            string rc = Resources.StrNoImagesAvailableForExport;
            if (fileListBox.Items.Count > 0)
                rc = fileListBox.Items.Count.ToString() + ((fileListBox.Items.Count == 1) ? Resources.StrFileReadyForExport : Resources.StrFilesReadyForExport);
            return rc;
        }

        private void UpdateUI()
        {
            if (fileListBox.Items.Count > 0)
            {
                exportButton.Enabled = true;
                removeButton.Enabled = true;
                restoreMaskCheckBox.Enabled = (_dlgproc.IsURAAdminLoggedOn()) ? true : false;
            }
            else
            {
                exportButton.Enabled = false;
                removeButton.Enabled = false;
                restoreMaskCheckBox.Enabled = false;
            }
        }

        public void HelpRequest(object sender, HelpEventArgs hlpevent)
        {
            DlgUtils.ShowHelp(this, "User_Manual_-_Dialog_-_Export_Images.htm", hlpevent);
        }
    }
}