using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using WillStott.MxAppToolbox;
using osImageManagerLib;
using osImageManagerApp.Properties;


namespace osImageManagerApp
{
    public partial class DlgFileImport : Form, IMxDlgForm
    {
        private OsDatabases _databases;
        private string _diskRef;
        private bool _autoReadAfterBrowse;
        private bool _surnameFirstQueryUKCTOCS;
        private DlgProcFileImport _dlgproc;

        public DlgFileImport()
        {
            InitializeComponent();

            _diskRef = "";
            _databases = null;
            _autoReadAfterBrowse = false;
            _surnameFirstQueryUKCTOCS = false;
            _dlgproc = null;

        }

        public bool clearImageTree { get { return clearImageTreeCheckBox.Checked; } }
        public string DiskRef { get { return _diskRef; } }

        public bool AutoReadAfterBrowse 
        { 
            get { return _autoReadAfterBrowse; }
            set {_autoReadAfterBrowse = value; }
        }
        public bool SurnameFirstQueryUKCTOCS 
        { 
            get { return _surnameFirstQueryUKCTOCS; } 
            set { _surnameFirstQueryUKCTOCS = value; } 
        }

        private void DlgFileImport_Load(object sender, EventArgs e)
        {
            int delay = Utils.GetAppSettingInt(ExecutionTime.Configuration.AppSettings, "ImportExportDelay");

            if ((_dlgproc = DlgProcFileImport.CreateDlgProc(osImageManagerProgram.Instance.ImageImportMode, _databases, imageListBox, CorrectionsListBox, workProgressBar, ProgressTextLabel, delay, osImageManagerProgram.Instance)) != null)
                Text = Resources.StrImportDlgTitle + _dlgproc.DialogTitle;

            diskRefTextBox.MaxLength = ImageRecord.MaxDiskLabelLength;
            fileLocationLabel.Text = osImageManagerProgram.Instance.LastFileImportDir;

            UpdateUI();
            ProgressTextLabel.Text = GetProgressText();
        }

        public OsDatabases Databases
        {
            set { _databases = value; }
            get { return _databases; }
        }

        private void viewImportImageButton_Click(object sender, EventArgs e)
        {
            ViewFileSelectedInListBox(imageListBox);
        }

        private void viewBadImageButton_Click(object sender, EventArgs e)
        {
            ViewFileSelectedInListBox(CorrectionsListBox);
        }

        private void removeButton_Click(object sender, EventArgs e)
        {
            int selected = imageListBox.SelectedIndex;
            if (selected != -1)
            {
                ListBox.SelectedObjectCollection selections = imageListBox.SelectedItems;
                foreach (object item in selections)
                    CorrectionsListBox.Items.Add(item);
                   
                int lastRemovedItem = DlgUtils.RemoveSelectedItemFromListBox(imageListBox, -1);
                if ((imageListBox.Items.Count > 0) && (lastRemovedItem != -1))
                    imageListBox.SelectedIndex = (lastRemovedItem == imageListBox.Items.Count) ? lastRemovedItem - 1 : lastRemovedItem;
                ProgressTextLabel.Text = GetProgressText();
            }
            UpdateUI();
        }

        private void acceptButton_Click(object sender, EventArgs e)
        {
            if (_dlgproc == null)
                return;

            if (_dlgproc.acceptClick(CorrectionsListBox, imageListBox))
                ProgressTextLabel.Text = GetProgressText();
            UpdateUI();
        }

        private void correctButton_Click(object sender, EventArgs e)
        {
            if (CorrectionsListBox.SelectedIndex != -1)
            {
                DlgDICOMCorrection dlg = new DlgDICOMCorrection();
                dlg.Initialize(new DlgProcDICOMCorrection(CorrectionsListBox.Items[CorrectionsListBox.SelectedIndex], _databases, osImageManagerProgram.Instance));
                dlg.SurnameFirstQueryUKCTOCS = _surnameFirstQueryUKCTOCS;
                if (dlg.ShowDialog() == DialogResult.OK)
                    acceptButton_Click(null, null);
            }
        }

        private void browseButton_Click(object sender, EventArgs e)
        {
            if (_dlgproc == null)
                return;

            String dir = _dlgproc.OpenBrowseDlg (osImageManagerProgram.Instance.LastFileImportDir, fileLocationLabel);
            if (dir.Length > 0 )
            {
                osImageManagerProgram.Instance.LastFileImportDir = dir;
                if (_autoReadAfterBrowse)
                    readDiskButton_Click(this, new EventArgs());
            }
        }

        private void readDiskButton_Click(object sender, EventArgs e)
        {
            if (_dlgproc == null)
                return;

            _dlgproc.ReadFiles(fileLocationLabel.Text, osImageManagerProgram.Instance.ExamDirectoryNamePattern);

            UpdateUI();
        }

        private void importButton_Click(object sender, EventArgs e)
        {
            if (_dlgproc == null)
                return;

            if (_dlgproc.DiskRefIsValid(diskRefTextBox.Text) == false)
                MessageBox.Show(Resources.StrMsgBoxInvalidDiskRef, Resources.StrMsgBoxWarning, MessageBoxButtons.OK);
            else
            {
                _dlgproc.ImportFiles(diskRefTextBox.Text, DateTime.Now);
                _diskRef = diskRefTextBox.Text;
                DialogResult = DialogResult.OK;
            }
        }

        private void ViewFileSelectedInListBox(ListBox box)
        {
            if (_dlgproc == null)
                return;

            if (box.SelectedIndex != -1)
            {
                String[] pathFilenames = null;
                if ((pathFilenames = _dlgproc.GetFilenamesForSelectedItem(box.Items[box.SelectedIndex])) != null)
                {
                    DlgDICOMViewer dlg = new DlgDICOMViewer();
                    foreach (String pathFilename in pathFilenames)
                        dlg.AddDicomPathFilename(pathFilename);
                    dlg.ShowDialog();
                }
            }
        }

        private void CorrectionsListBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            UpdateUI();
        }

        private string GetProgressText()
        {
            string rc = Resources.StrNoImagesAvailableForImport;
            if (imageListBox.Items.Count > 0)
                rc = imageListBox.Items.Count.ToString() + ((imageListBox.Items.Count == 1) ? Resources.StrFileReadyForImport : Resources.StrFilesReadyForImport);
            return rc;
        }

        private void UpdateUI()
        {
            viewImportImageButtonSet();
            UpdateUIImageBox();
            UpdateUICorrectionBox();
        }

        private void UpdateUIImageBox()
        {
            if (imageListBox.Items.Count > 0)
            {
                removeButton.Enabled = true;
                if (_databases.IsUserLoggedOn())
                {
                    importButton.Enabled = (_databases.IsGuestLoggedOn() || _databases.IsSaLoggedOn()) ? false : true;
                    diskRefTextBox.Enabled = true;
                    clearImageTreeCheckBox.Enabled = true;
                    clearImageTreeCheckBox.Checked = true;
                }
                else
                {
                    importButton.Enabled = false;
                    diskRefTextBox.Enabled = false;
                    clearImageTreeCheckBox.Enabled = false;
                    clearImageTreeCheckBox.Checked = false;
                }
            }
            else
            {
                removeButton.Enabled = false;
                importButton.Enabled = false;
                diskRefTextBox.Enabled = false;
                clearImageTreeCheckBox.Enabled = false;
                clearImageTreeCheckBox.Checked = false;
            }
        }

        private void UpdateUICorrectionBox()
        {
            if (_dlgproc == null)
                return;

            if ((CorrectionsListBox.Items.Count > 0) && (CorrectionsListBox.SelectedIndex != -1) && (CorrectionsListBox.Items[CorrectionsListBox.SelectedIndex] != null))
            {
                correctButton.Enabled = ((_databases.IsUserLoggedOn()) && (_databases.IsGuestLoggedOn() == false)) ? true : false;
                viewBadImageButton.Enabled = true;
                acceptButton.Enabled = _dlgproc.GetAcceptButtonStateForSelection(CorrectionsListBox.Items[CorrectionsListBox.SelectedIndex]);
            }
            else
            {
                acceptButton.Enabled = false;
                correctButton.Enabled = false;
                viewBadImageButton.Enabled = false;
            }
        }

        public void HelpRequest(object sender, HelpEventArgs hlpevent)
        {
            DlgUtils.ShowHelp(this, "User_Manual_-_Dialog_-_Import_Images.htm", hlpevent);
        }

        private void imageListBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            viewImportImageButtonSet();
        }

        private void viewImportImageButtonSet()
        {
            int selectCnt = 0;
            for (int x = 0; x < imageListBox.Items.Count; x++)
            {
                if (imageListBox.GetSelected(x) == true)
                    selectCnt++;
            }
            viewImportImageButton.Enabled = ((selectCnt > 1) || (imageListBox.Items.Count == 0)) ? false : true;
        }
    }
}