using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using WillStott.MxAppToolbox;
using osImageManagerLib;
using osImageManagerApp.Properties;

namespace osImageManagerApp
{
    public partial class DlgQueryUKCTOCS : Form, IMxDlgForm
    {
        private string _surname;
        private string _firstNames;
        private string _volunteerRef;
        private OsDatabases _databases;
        private bool _radioSelectionInProgress;

        public DlgQueryUKCTOCS()
        {
            InitializeComponent();
            _radioSelectionInProgress = false;
        }

        public string Surname 
        {
            get { return _surname; }
            set { _surname = value; } 
        }
        public string FirstNames 
        {
            get { return _firstNames; }
            set { _firstNames = value; } 
        }
        public string VolunteerRef 
        { 
            get { return _volunteerRef; }
            set { _volunteerRef = value; } 
        }
        public OsDatabases Databases
        {
            get { return _databases; }
            set {_databases = value; }
        }

        private void DlgQueryUKCTOCS_Load(object sender, EventArgs e)
        {
            if (_databases == null)
                throw new UniException("OSA-038", UniException.Reset.AbortOp, UniException.Category.Resource, "Dialog not initialized: _BadFile or _Databases are null");

            QueryTypeRadioButton_CheckedChanged(this, null);
        }

        private void runQueryButton_Click(object sender, EventArgs e)
        {
            string storedProcName = OsDatabases.ListVolunteerRefForNamesSP;
            string Param1 = _surname;
            string Param2 = _firstNames;
            if (listVolunteerRefsForNameRadioButton.Checked == false)
            {
                Param1 = _volunteerRef;
                Param2 = "";
                storedProcName = (lookupNamesForVolunteerRefRadioButton.Checked == true) ? OsDatabases.ListNameForVolunteerRefSP : OsDatabases.ListScanDatesForVolunteerRefSP;
            }

            if ( (Param1 == "%") && (Param2 == "%"))
                MessageBox.Show(Resources.StrWarnWildcardSearch, Resources.StrMsgBoxWarning, MessageBoxButtons.OK);
            else
                queryResultsTextBox.Text = _databases.GetUKCTOCSQueryResults(storedProcName, Param1, Param2);
        }

        private void QueryTypeRadioButton_CheckedChanged(object sender, EventArgs e)
        {
            _radioSelectionInProgress = true;
            if ( listVolunteerRefsForNameRadioButton.Checked )
            {
                QuerySurnameTextBox.Text = _surname;
                QuerySurnameTextBox.ReadOnly = false;
                QuerySurnameTextBox.TabStop = true;
                QueryFirstNamesTextBox.Text = _firstNames;
                QueryFirstNamesTextBox.ReadOnly = false;
                QueryFirstNamesTextBox.TabStop = true;
                QueryVolunteerRefTextBox.Text = "";
                QueryVolunteerRefTextBox.ReadOnly = true;
                QueryVolunteerRefTextBox.TabStop = false;
            }
            else
            {
                QuerySurnameTextBox.Text = "";
                QuerySurnameTextBox.ReadOnly = true;
                QuerySurnameTextBox.TabStop = false;
                QueryFirstNamesTextBox.Text = "";
                QueryFirstNamesTextBox.ReadOnly = true;
                QueryFirstNamesTextBox.TabStop = false;
                QueryVolunteerRefTextBox.Text = _volunteerRef;
                QueryVolunteerRefTextBox.ReadOnly = false;
                QueryVolunteerRefTextBox.TabStop = true;
            }
            _radioSelectionInProgress = false;
        }

        private void QueryVolunteerRefTextBox_TextChanged(object sender, EventArgs e)
        {
            if ( _radioSelectionInProgress == false)
                _volunteerRef = QueryVolunteerRefTextBox.Text;
        }
        private void QuerySurnameTextBox_TextChanged(object sender, EventArgs e)
        {
            if (_radioSelectionInProgress == false)
                _surname = QuerySurnameTextBox.Text;
        }
        private void QueryFirstNamesTextBox_TextChanged(object sender, EventArgs e)
        {
            if (_radioSelectionInProgress == false)
                _firstNames = QueryFirstNamesTextBox.Text;
        }

        public void HelpRequest(object sender, HelpEventArgs hlpevent)
        {
            DlgUtils.ShowHelp(this, "User_Manual_-_Dialog_-_Query_UKCTOCS.htm", hlpevent);
        }
    }
}