using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Diagnostics;
using System.Configuration;

using osImageManagerApp.Properties;
using osImageManagerLib;
using WillStott.MxAppToolbox;

namespace osImageManagerApp
{
    public partial class DlgToolsDbAdmin : Form, IMxDlgForm
    {
        private SqlScripts _scripts;
        private OsDatabases _databases;
        private string _UATDocFilename;
        private string _UATDocResultFilename;
        private Process _process;
        private string _processErrMsg = "";

        public DlgToolsDbAdmin()
        {
            InitializeComponent();
            surnameTextBox.Text = "Stott";
            firstNamesTextBox.Text = "William Philip Quentin";
            volunteerRefTextBox.Text = "99000002";
            _UATDocFilename = "";
            _UATDocResultFilename = ""; 
            _process = null;
            _processErrMsg = "";
        }

        public void RunButtonEnable(bool state)
        {
            runButton.Enabled = state;
        }

        public OsDatabases Databases
        {
            get { return _databases; }
            set { _databases = value; }
        }

        private void runButton_Click(object sender, EventArgs e)
        {
            if ((scriptListBox.SelectedIndex >= 0) && (scriptListBox.SelectedIndex < _scripts.GetList().Length))
            {
                string result = "";
                SqlScriptElement script = scriptListBox.SelectedItem as SqlScriptElement;
                if (script != null)
                {
                    string dir = ExecutionTime.Instance.ScriptsDir;
                    if (script.DoConfirm() == false)
                        result = _databases.RunScript(script.DbName, script.CmdType, script.GetScript(dir), script.ConnName);
                    else
                    {
                        if (MessageBox.Show(script.ConfirmMsg(), Resources.StrMsgBoxConfirm, MessageBoxButtons.YesNo) == DialogResult.Yes)
                            result = _databases.RunScript(script.DbName, script.CmdType, script.GetScript(dir), script.ConnName);
                    }
                    if (String.Compare(script.ShowResults, "YES", true) == 0)
                        resultsTextBox.Text = result;
                }
            }
        }

        private void DlgToolsDbAdmin_Load(object sender, EventArgs e)
        {
            try
            {
                Text += " " + "ImageDB: " + _databases.ImageDbSource + "[" + _databases.ImageDbName + "] PatientDB: " + _databases.PatientDbSource + "[" + _databases.PatientDbName + "]";
                if (_databases.IsSaLoggedOn())
                {
                    createUATFilesButton.Enabled = false;
                    testPatientDbButton.Enabled = false;
                    editUATDocButton.Enabled = false;
                    runUATButton.Enabled = false;
                }

                _scripts = new SqlScripts();
                scriptListBox.BeginUpdate();
                foreach (SqlScriptElement script in _scripts.GetList())
                {
                    script.DbName = script.DbName.Trim();
                    if (_databases.IsSaLoggedOn())
                    {
                        if (script.RunBy == "SysAdmin")
                            scriptListBox.Items.Add(script);
                    }
                    else
                    {
                        if (script.RunBy == "URA-Admin")
                            scriptListBox.Items.Add(script);
                    }
                }
                scriptListBox.EndUpdate();

                if (_databases.IsSaLoggedOn() == false)
                {
                    uatFilesDirlabel.Text = ExecutionTime.Instance.ImageDir;
                    SetUATDocPathFilename();
                }
                UATResultDocFilename.Text = "";
            }
            catch (Exception ex)
            {
                closeButton.PerformClick();
                throw new UniException("OSA-004", UniException.Reset.AbortOp, UniException.Category.Program, ex, "DlgToolsDbAdmin load is bad");
            }
        }

        private void createUATFilesButton_Click(object sender, EventArgs e)
        {
            resultsTextBox.Text = "";
            Refresh();
            if (TestSuiteSetup.CreateTestImageDirectories())
            {
                if (TestSuiteSetup.CreateTestImageFiles() == false)
                    resultsTextBox.Text = Resources.StrMsgBoxActionFailed + Utils.StrLFCR;
                else
                    resultsTextBox.Text = TestSuiteSetup.ListTestImageFiles() + Utils.StrLFCR; 
            }
        }

        private void SetUATDocPathFilename()
        {
            editUATDocButton.Enabled = false;
            runUATButton.Enabled = false;

            String documentName = Utils.GetAppSettingString (ExecutionTime.Configuration.AppSettings, "UATDocFilename");
            if ( documentName != null )
            {
                _UATDocFilename = documentName;
                UATDocFilenamelabel.Text = ExecutionTime.Instance.FITDir + _UATDocFilename;
                if (File.Exists(ExecutionTime.Instance.FITDir + _UATDocFilename))
                {
                    _UATDocResultFilename = "UATResults.htm";
                    editUATDocButton.Enabled = true;
                    runUATButton.Enabled = true;
                }
            }
            return;
        }


        private void testPatientDbButton_Click(object sender, EventArgs e)
        {
            string param1 = "";
            string param2 = "";

            resultsTextBox.Text = StartTest();

            if (surnameTextBox.Text.Length > 0)
            {
                resultsTextBox.Text += OsDatabases.ListVolunteerRefForNamesSP + "() = ";
                param1 = _databases.GetUKCTOCSQueryResults(OsDatabases.ListVolunteerRefForNamesSP, surnameTextBox.Text, firstNamesTextBox.Text);
                resultsTextBox.Text += param1 + Utils.StrLFCR + Utils.StrLFCR;
            }
            if (volunteerRefTextBox.Text.Length > 0)
            {
                param1 = volunteerRefTextBox.Text;
                resultsTextBox.Text += OsDatabases.ListNameForVolunteerRefSP + "() = ";
                resultsTextBox.Text += _databases.GetUKCTOCSQueryResults(OsDatabases.ListNameForVolunteerRefSP, param1, param2) + Utils.StrLFCR + Utils.StrLFCR;

                param1 = volunteerRefTextBox.Text;
                resultsTextBox.Text += OsDatabases.ListScanDatesForVolunteerRefSP + "() = ";
                resultsTextBox.Text += _databases.GetUKCTOCSQueryResults(OsDatabases.ListScanDatesForVolunteerRefSP, param1, param2);
            }
            resultsTextBox.Text += EndTest();
        }

        private string StartTest()
        {
            editUATDocButton.Enabled = false;
            runUATButton.Enabled = false;
            testPatientDbButton.Enabled = false;
            runButton.Enabled = false;

            return Resources.StrTestStartsAt + DateTime.Now.ToString("hh:mm:ss.fffff") + Utils.StrLFCR + Resources.StrHR + Utils.StrLFCR;
        }

        private string EndTest()
        {
            editUATDocButton.Enabled = true;
            runUATButton.Enabled = true;
            testPatientDbButton.Enabled = true;
            runButton.Enabled = true;

            return Utils.StrLFCR + Resources.StrHR + Utils.StrLFCR + Resources.StrTestEndsAt + DateTime.Now.ToString("hh:mm:ss.fffff") + Utils.StrLFCR;
        }


        private void runAUTButton_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show(Resources.StrConfirmRunUAT, Resources.StrMsgBoxWarning, MessageBoxButtons.YesNo) == DialogResult.Yes)
            {
                UATResultDocFilename.Text = ExecutionTime.Instance.FITDir + _UATDocResultFilename;
                resultsTextBox.Text = StartTest();
                resultsTextBox.Text += Resources.StrRunningUATPleaseWait;

                if (File.Exists(ExecutionTime.Instance.FITDir + _UATDocResultFilename))
                    File.Delete(ExecutionTime.Instance.FITDir + _UATDocResultFilename);

                Refresh();

                _processErrMsg = "";

                String documentName = Utils.GetAppSettingString(ExecutionTime.Configuration.AppSettings, "UATDocFilename");

                if ((_process = ProcessOps.StartCmdLineApp(ExecutionTime.Instance.FITDir + "runFile.exe", ExecutionTime.Instance.FITDir, _UATDocFilename + " " + _UATDocResultFilename + " .")) != null) 
                {
                    resultsTextBox.Text += Utils.StrLFCR + Resources.StrWorkingDir + ExecutionTime.Instance.FITDir + Utils.StrLFCR;
                    resultsTextBox.Text += _process.StandardError.ReadToEnd();
                    _processErrMsg = _process.StandardOutput.ReadToEnd();
                    Refresh();

                    uatTimer.Interval = 1000;
                    uatTimer.Start();
                }
            }
        }

        private void uatTimer_Tick(object sender, EventArgs e)
        {
            if (_process != null)
            {
                Refresh();
                if (_process.HasExited)
                {
                    _process = null;
                    uatTimer.Stop();

                    if (_processErrMsg.Length > 0)
                        resultsTextBox.Text += Utils.StrLFCR + Resources.StrUATErrors + _processErrMsg + Utils.StrLFCR;

                    if (File.Exists(ExecutionTime.Instance.FITDir + _UATDocResultFilename) == false)
                        resultsTextBox.Text += Utils.StrLFCR + Resources.StrUATResultsNotFound + Utils.StrLFCR;
                    else
                        ProcessOps.StartApp(ExecutionTime.Instance.FITDir + _UATDocResultFilename, "", "", true);

                    resultsTextBox.Text += EndTest();
                    Refresh();
                }
            }
        }

        private void editUATDocButton_Click(object sender, EventArgs e)
        {
            ProcessOps.StartApp(ExecutionTime.Instance.FITDir + _UATDocFilename, "", "edit", true);
        }

        private void closeButton_Click(object sender, EventArgs e)
        {
            if (_process != null)
            {
                _process.Close();
                _process.Kill();
                _process = null;
            }
            Close();
        }

        public void HelpRequest(object sender, HelpEventArgs hlpevent)
        {
            DlgUtils.ShowHelp(this, "User_Manual_-_Dialog_-_Tools_-_DbAdmin.htm", hlpevent);
        }


    }
}