using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Reflection;
using System.Configuration;

using WillStott.MxAppToolbox;
using osImageManagerApp.Properties;
using osImageManagerLib;


namespace osImageManagerApp
{
    public partial class MainForm : Form, IMainForm
    {
        private bool _logon;
        private OsDatabases _databases;
        private Favorites _favorites;
        static public readonly string uriLabel = "db://";
        static public readonly string fileLabel = "file://";
        static public readonly string supportWebSite = "http://www.ospacs.org";

        public MainForm()
        {
            InitializeComponent();

            _favorites = new Favorites();
            _databases = new OsDatabases();
            _logon = false;
            LogonToggleUI();

            InitToolbar();

            UpdateFormUI();
        }


        #region IMainForm Members

        public void UpdateFormUI()
        {
            SetMainFormTitle();

            ToolbarOps.UpdateButtonState(toolStripMain);

            clearImageTreeToolStripMenuItem.Enabled = (imageRecordTreeView.TotalImagesInTree > 0) ? true : false;
            showHeaderImageTreeToolStripMenuItem.Enabled = (imageRecordTreeView.TotalImagesInTree > 0) ? true : false;
            refreshImageTreeToolStripMenuItem.Enabled = (imageRecordTreeView.TotalImagesInTree > 0) ? true : false;
            editFindToolStripMenuItem.Enabled = (imageRecordTreeView.TotalImagesInTree > 0) ? true : false;
        }

        public void InitToolbar()
        {
            fileLogonToolStripButton.Tag = fileLogonToolStripMenuItem;
            fileImportToolStripButton.Tag = fileImportToolStripMenuItem;
            viewImageRecordsByVolunteerRefToolStripButton.Tag = viewImageRecordsByVolunteerRefToolStripMenuItem;
            toolsOptionsToolStripButton.Tag = toolsOptionsToolStripMenuItem;
            helpSupportWebsiteToolStripButton.Tag = helpSupportWebsiteToolStripMenuItem;
            helpAboutToolStripButton.Tag = helpAboutToolStripMenuItem;
            helpContentsToolStripButton.Tag = helpContentsToolStripMenuItem;

            ToolbarOps.SetMenuItemImages(toolStripMain);
        }

        public void toolStripMain_ItemClicked(object sender, ToolStripItemClickedEventArgs e)
        {
            ToolbarOps.ButtonClick(e);
        }

        public void MainForm_Closing(object sender, FormClosingEventArgs e)
        {
            if (( e != null ) && (osImageManagerProgram.Instance.CloseAllDocuments(true) == false))
                e.Cancel = true;
            else
            {
                Logoff();
            }
        }

        public void MainForm_Load(object sender, EventArgs e)
        {
            DicomMask.InitializeMaskSettings();
            imageRecordTreeView.Init(axezDICOMX, _databases, new RegionalCentres(), osImageManagerProgram.Instance.TempDicomPathFile, statusLabelToolStrip, editDeleteToolStripMenuItem, osImageManagerProgram.Instance);
            osImageManagerProgram.Instance.MainFormClosing = false;
        }

        #endregion

        private void LogonToggleUI()
        {
            if (_logon == false)
            {
                fileLogonToolStripMenuItem.Text = Resources.StrMenuFileLogon;
                fileLogonToolStripButton.ToolTipText = Resources.StrToolbarFileLogon;

                viewImageRecordsByVolunteerRefToolStripMenuItem.Enabled = false;
                viewImageRecordsByDiskRefToolStripMenuItem.Enabled = false;
                fileExportToolStripMenuItem.Enabled = false;
                toolsDbAdminToolStripMenuItem.Visible = false;
                toolsDbAdminToolStripMenuItem.Enabled = false;
                toolsDbAdminToolStripSeparator.Visible = false;
                editDeleteToolStripMenuItem.Visible = false;
            }
            else
            {
                fileLogonToolStripMenuItem.Text = Resources.StrMenuFileLogoff;
                fileLogonToolStripButton.ToolTipText = Resources.StrToolbarFileLogoff;

                if (_databases.IsSaLoggedOn())
                {
                    toolsDbAdminToolStripMenuItem.Visible = true;
                    toolsDbAdminToolStripMenuItem.Enabled = true;
                    toolsDbAdminToolStripSeparator.Visible = true;
                }
                else
                {
                    viewImageRecordsByVolunteerRefToolStripMenuItem.Enabled = true;
                    viewImageRecordsByDiskRefToolStripMenuItem.Enabled = true;
                    fileExportToolStripMenuItem.Enabled = true;

                    if (_databases.IsURAAdminLoggedOn())
                    {
                        toolsDbAdminToolStripMenuItem.Visible = true;
                        toolsDbAdminToolStripMenuItem.Enabled = true;
                        toolsDbAdminToolStripSeparator.Visible = true;
                        editDeleteToolStripMenuItem.Visible = true;
                        editDeleteToolStripMenuItem.Enabled = true;
                    }
                }

            }
            UpdateFormUI();
        }

        private void SetMainFormTitle()
        {
            Text = Resources.StrMainFormTitle + " v" + Assembly.GetExecutingAssembly().GetName().Version.ToString() + " " + Resources.StrNotForClinicalUse;
            if (_logon == true)
            {
                string username = osImageManagerProgram.Instance.UserName;
                if (_databases.IsURAAdminLoggedOn())
                    username += Resources.StrURAAdminIndicator;
                if (_databases.IsSaLoggedOn())
                    username += Resources.StrSysAdminIndicator;
                Text = Text.Insert(0, string.Format("[{0}] {1}:{2}, {3}:{4} - ", username, _databases.ImageDbSource, _databases.ImageDbName, _databases.PatientDbSource, _databases.PatientDbName));
            }
        }

        private void fileLogonToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (_logon == true)
                Logoff();
            else
            {
                DlgFileLogon dlg = new DlgFileLogon();
                dlg.UserName = osImageManagerProgram.Instance.UserName;
                if (dlg.ShowDialog() == DialogResult.OK)
                {
                    if (_logon = _databases.Connect(dlg.UserName, dlg.Password)) 
                    {
                        osImageManagerProgram.Instance.UserName = dlg.UserName;
                        LogonToggleUI();
                    }
                }
            }
        }

        private void fileImportToolStripMenuItem_Click(object sender, EventArgs e)
        {
            DlgFileImport dlg = new DlgFileImport();
            dlg.Databases = _databases;
            dlg.AutoReadAfterBrowse = osImageManagerProgram.Instance.AutoReadAfterBrowseFileImport;
            dlg.SurnameFirstQueryUKCTOCS = osImageManagerProgram.Instance.SurnameFirstQueryUKCTOCS;
            if (dlg.ShowDialog() == DialogResult.OK)
            {
                if ( dlg.clearImageTree)
                    _favorites.Clear();
                imageRecordTreeView.PopulateTree(-1, dlg.DiskRef, dlg.clearImageTree);
                refreshToolStripMenuItem_Click(this, null);
            }
            UpdateFormUI();
        }


        private void fileExportToolStripMenuItem_Click(object sender, EventArgs e)
        {
            DlgFileExport dlg = new DlgFileExport();
            dlg.Initialize(_databases, imageRecordTreeView.GetDicomRecords());
            dlg.ShowDialog();
            UpdateFormUI();
        }

        private void viewImageRecordsForVolunteerToolStripMenuItem_Click(object sender, EventArgs e)
        {
            DlgSelectItemsForImageTree dlg = new DlgSelectItemsForImageTree();
            dlg.ForVolunteers = true;
            if (dlg.ShowDialog() == DialogResult.OK)
            {
                if ( dlg.clearImageTree )
                _favorites.Clear();
                imageRecordTreeView.PopulateTree(dlg.VolunteerRef, null, dlg.clearImageTree);
            }
            UpdateFormUI();
        }

        private void viewImageRecordsForDiskRefToolStripMenuItem_Click(object sender, EventArgs e)
        {
            DlgSelectItemsForImageTree dlg = new DlgSelectItemsForImageTree();
            dlg.ForDiskRef = true;
            if (dlg.ShowDialog() == DialogResult.OK)
            {
                if (dlg.clearImageTree)
                    _favorites.Clear();
                imageRecordTreeView.PopulateTree(-1, dlg.DiskRef, dlg.clearImageTree);
            }
            UpdateFormUI();
        }

        private void showHeaderToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (axezDICOMX.DCMshowHeader == true)
            {
                axezDICOMX.DCMshowHeader = false;
                showHeaderImageTreeToolStripMenuItem.Checked = false;
            }
            else
            {
                axezDICOMX.DCMshowHeader = true;
                showHeaderImageTreeToolStripMenuItem.Checked = true;
            }
        }

        private void clearImageTreeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            _favorites.Clear();
            imageRecordTreeView.ClearAllNodes();
            UpdateFormUI();
        }

        private void refreshToolStripMenuItem_Click(object sender, EventArgs e)
        {
            imageRecordTreeView.RefreshAllNodes();
            if ( imageRecordTreeView.TotalImagesInTree == 0 )
                UpdateFormUI();        //in case refresh throws an error
        }

        private void toolsDbAdminToolStripMenuItem_Click(object sender, EventArgs e)
        {
            DlgToolsDbAdmin dlg = new DlgToolsDbAdmin();
            dlg.RunButtonEnable(_logon);
            dlg.Databases = _databases;
            dlg.ShowDialog();
            UpdateFormUI();
        }

         
        private void toolsOptionsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            DlgToolsOptions dlg = new DlgToolsOptions();
            dlg.AutoReadAfterBrowseFileImport = osImageManagerProgram.Instance.AutoReadAfterBrowseFileImport;
            dlg.SurnameFirstQueryUKCTOCS = osImageManagerProgram.Instance.SurnameFirstQueryUKCTOCS;
            dlg.ImageImportMode = osImageManagerProgram.Instance.ImageImportMode;
            dlg.ExamDirName = osImageManagerProgram.Instance.ExamDirectoryNamePattern;

            if (dlg.ShowDialog() == DialogResult.OK)
            {
                osImageManagerProgram.Instance.AutoReadAfterBrowseFileImport = dlg.AutoReadAfterBrowseFileImport;
                osImageManagerProgram.Instance.SurnameFirstQueryUKCTOCS = dlg.SurnameFirstQueryUKCTOCS;
                osImageManagerProgram.Instance.ImageImportMode = dlg.ImageImportMode;
                osImageManagerProgram.Instance.ExamDirectoryNamePattern = dlg.ExamDirName;
            }
            UpdateFormUI();
        }

        private void editFindToolStripMenuItem_Click(object sender, EventArgs e)
        {
            DlgEditFind dlg = new DlgEditFind();
            if (dlg.ShowDialog() == DialogResult.OK)
                imageRecordTreeView.SelectTreeItem(null, null, dlg.VolunteerRef, null);
        }

        private void editDeleteToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ImageRecord rec = imageRecordTreeView.SelectedImageItem;
            string msg = Resources.StrMsgBoxConfirmDeleteRecord + Utils.StrLFCR + Utils.StrLFCR + rec.ToString();
            if (MessageBox.Show(msg, Resources.StrMsgBoxWarning, MessageBoxButtons.YesNo) == DialogResult.Yes)
            {
                _favorites.Clear();
                _databases.DeleteImageRecord(rec.ImageID);
                imageRecordTreeView.RemoveSelectedItem();
            }
        }

        private void fileExitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            osImageManagerProgram.Instance.TerminateApp(false);
        }

        private void viewTooolbarToolStripMenuItem_Click(object sender, EventArgs e)
        {
            MenuOps.ToggleShow(toolStripMain, viewToolbarToolStripMenuItem);
        }

        private void viewStatusBarToolStripMenuItem_Click(object sender, EventArgs e)
        {
            MenuOps.ToggleShow(statusStripMain, viewStatusBarToolStripMenuItem);
        }

        private void helpContentsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Help.ShowHelp(this, osImageManagerProgram.Instance.HelpFilePathName, HelpNavigator.TableOfContents);
        }

        private void helpIndexToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Help.ShowHelp(this, osImageManagerProgram.Instance.HelpFilePathName, HelpNavigator.Index);
        }

        private void helpSearchToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Help.ShowHelp(this, osImageManagerProgram.Instance.HelpFilePathName, HelpNavigator.Find, "");
        }

        private void helpReleaseNotesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ProcessOps.StartApp(fileLabel + Utils.GetPath(osImageManagerProgram.Instance.HelpFilePathName) + @"Release_Notes.htm", null, true);
        }

        private void helpLicenceViewToolStripMenuItem_Click(object sender, EventArgs e)
        {
            DlgFileViewer dlg = new DlgFileViewer();
            dlg.SetFile("View Licence", Utils.GetPath(osImageManagerProgram.Instance.HelpFilePathName) + @"Licence.rtf");
            dlg.ShowDialog();
        }

        private void helpSupportWebsiteToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ProcessOps.StartApp(supportWebSite, null, true);
        }

        private void helpAboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            DlgHelpAbout dlg = new DlgHelpAbout();
            dlg.ShowDialog();
        }

        public void Logoff()
        {
            _databases.Disconnect();
            _logon = false;
            imageRecordTreeView.ClearAllNodes();
            _favorites.Clear();
            LogonToggleUI();
        }

        private void favoritesToolStripMenuItem_DropDownOpened(object sender, EventArgs e)
        {
            PopulateFavoritesMenu();
        }

        private void favoritesClearToolStripMenuItem_Click(object sender, EventArgs e)
        {
            _favorites.Clear();
        }

        private void favoritesAddToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ImageRecord rec = imageRecordTreeView.SelectedImageItem;
            if ( rec != null)
                _favorites.Add(new Favorite(rec.GetShortDicomRef(), new Uri(uriLabel + rec.ImageID)));
        }

        private void favoritesToolStripMenuItem_DisplayImage(object sender, EventArgs e)
        {
            ToolStripMenuItem item = sender as ToolStripMenuItem;
            if ( item != null)
            {
                ImageRecord rec = new ImageRecord();
                rec.ImageID = _favorites[item.Text].Host.ToUpper();
                imageRecordTreeView.SelectTreeItem(rec, null, null, null);
            }
        }

        private void PopulateFavoritesMenu()
        {
            favoritesToolStripMenuItem.DropDownItems.Clear();

            int adminMenuItems = (_favorites.Count > 0) ? 3 : 1;
            ToolStripItem[] FavItems = new ToolStripItem[_favorites.Count + adminMenuItems];

            ToolStripMenuItem addMenu = new ToolStripMenuItem(Resources.StrMenuAddFavorite, null, new EventHandler(favoritesAddToolStripMenuItem_Click));
            addMenu.AccessibleDescription = Resources.StrMenuAddFavoriteAccessDesc;
            addMenu.AccessibleName = Resources.StrMenuAddFavoriteAccessName;
            addMenu.ShortcutKeys = ((System.Windows.Forms.Keys)((System.Windows.Forms.Keys.Control | System.Windows.Forms.Keys.A)));
            addMenu.Enabled = imageRecordTreeView.IsImageSelected();
            FavItems[0] = addMenu;

            if (adminMenuItems == 3)
            {
                ToolStripMenuItem clearMenu = new ToolStripMenuItem(Resources.StrMenuClearFavorite, null, new EventHandler(favoritesClearToolStripMenuItem_Click));
                addMenu.AccessibleDescription = Resources.StrMenuClearFavoriteAccessDesc;
                addMenu.AccessibleName = Resources.StrMenuClearFavoriteAccessName;

                FavItems[1] = clearMenu;
                FavItems[2] = new ToolStripSeparator();

                int cnt = adminMenuItems;
                IEnumerator<Favorite> myEnum = _favorites.GetEnumerator();
                while (myEnum.MoveNext())
                {
                    ToolStripMenuItem item = new ToolStripMenuItem(myEnum.Current.Label);
                    item.Click += new System.EventHandler(favoritesToolStripMenuItem_DisplayImage);
                    FavItems[cnt++] = item;
                }
            }
            favoritesToolStripMenuItem.DropDownItems.AddRange(FavItems);
        }
    }
}
