using System;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;
using System.IO;
using WillStott.MxAppToolbox;

namespace osImageManagerLib
{
    public class DicomFileSet
    {
        public DicomFile[] MakeRecordsFromDirectory(string filePath)
        {
            List<DicomFile> list = new List<DicomFile>();
            RecurseDirectoryTree(list, filePath);           
            DicomFile[] arr = new DicomFile[list.Count];
            list.CopyTo(arr);
            return arr;
        }

        public static DicomFile MakeRecordFromFile(string pathFilename)
        {
            DicomFile rc = null;
            if (File.Exists(pathFilename) == false)
                throw new UniException("OSA-072", UniException.Reset.AbortOp, UniException.Category.User, "File not found: {0}", pathFilename);
            else
                rc = new DicomFile(pathFilename);
            return rc;
        }

        private void RecurseDirectoryTree(List<DicomFile> list, string filePath)
        {
            FileSystemInfo info = null;
            try
            {
                DirectoryInfo dir = new DirectoryInfo(filePath);
                FileSystemInfo[] items = dir.GetFileSystemInfos();
                foreach (FileSystemInfo item in items)
                {
                    info = item;
                    if (info is DirectoryInfo)
                        RecurseDirectoryTree(list, info.FullName);

                    if (info is FileInfo)
                    {
                        if (DicomFile.IsValidFilename(info))
                        {
                            list.Add(new DicomFile(info.FullName));
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                string filename = (info == null ) ? "[null]" : info.FullName;
                throw new UniException("OSA-010", UniException.Reset.AbortOp, UniException.Category.Resource, ex, "File IO problem: {0}: {1} - msg[{2}]", filePath, filename, ex.Message);
            }
        }


    }
}
