using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;

using WillStott.MxAppToolbox;

namespace osImageManagerLib
{
    public class DlgProcFileImportExam : DlgProcFileImport
    {
        public DlgProcFileImportExam( OsDatabases databases, ListBox imageListBox, ListBox correctionsListBox, ProgressBar workProgressBar, Label progressTextLabel, int delay, Program errorHandler) : base (databases, imageListBox, correctionsListBox, workProgressBar, progressTextLabel, delay, errorHandler)
        {
            dlgBrowser = new DlgBrowserDir();
        }

        public override string DialogTitle
        {
            get { return Resources.StrImportModeExam; } 
        }

        public override int ReadFiles(string Location, string regex)
        {
            int rc = 0;

            imageLB.Items.Clear();
            correctionsLB.Items.Clear();

            DisplayProgressStart(Resources.StrLookingForFiles, 1, 10);

            ExaminationSet exams = new ExaminationSet(Location, regex);

            DisplayProgressEnd(exams.FileCount, (exams.FileCount == 1) ? Resources.StrFileFound : Resources.StrFilesFound);

            if (exams.FileCount > 0)
            {
                DisplayProgressStart(Resources.StrReadingFiles, 0, exams.FileCount);
                foreach (Examination exam in exams.GetExamList())
                {
                    bool allGood = true;
                    foreach (DicomFile file in exam.GetList())
                    {
                        DisplayProgressStep(Resources.StrReadingHeader);
                        if (file.ReadFromDisk(db, errorHandle) == false)
                            allGood = false;
                        rc++;
                    }
                    if (allGood)
                        imageLB.Items.Add(exam);
                    else
                        correctionsLB.Items.Add(exam);
                }
                DisplayProgressEnd(exams.FileCount, (exams.FileCount == 1) ? Resources.StrFileRead : Resources.StrFilesRead);
            }
            if (imageLB.Items.Count > 0)
                imageLB.SelectedIndex = 0;
            if (correctionsLB.Items.Count > 0)
                correctionsLB.SelectedIndex = 0;
            return rc;
        }

  
        public override bool acceptClick(ListBox correctionsListbox, ListBox imageListbox)
        {
            bool rc = false;

            if ((correctionsListbox != null) && (imageListbox != null))
            {
                int selected = correctionsListbox.SelectedIndex;
                if (selected != -1)
                {
                    Examination correctedExam = correctionsListbox.Items[selected] as Examination;
                    if (correctedExam != null)
                    {
                        bool allGood = true;
                        DicomFile[] fileList = correctedExam.GetList();
                        if (fileList != null)
                        {
                            foreach (DicomFile file in fileList)
                            {
                                if ((file.Record.NameMismatch) || (file.Record.UnspecifiedError) || (file.Record.IsValidDicomRef() == false))
                                    allGood = false;
                            }
                        }
                        if (allGood == true)
                        {
                            int index = imageListbox.Items.Add(correctedExam);
                            imageListbox.SelectedIndex = index;

                            int lastRemovedItem = DlgUtils.RemoveSelectedItemFromListBox(correctionsListbox, -1);
                            if ((correctionsListbox.Items.Count > 0) && (lastRemovedItem != -1))
                                correctionsListbox.SelectedIndex = (lastRemovedItem == correctionsListbox.Items.Count) ? lastRemovedItem - 1 : lastRemovedItem;
                            rc = true;
                        }
                    }
                }
            }
            return rc;
        }

        public override bool GetAcceptButtonStateForSelection(object selectedItem)
        {
            bool rc = false;
            Examination exam = selectedItem as Examination;
            if (exam != null)
            {
                rc = true;
                foreach (DicomFile file in exam.GetList())
                {
                    if ((file.Record.NameMismatch == true) || (file.Record.UnspecifiedError == true))
                    {
                        rc = false;
                        break;
                    }
                }
            }
            return rc;
        }

        public override String[] GetFilenamesForSelectedItem(object selectedItem)
        {
            String[] rc = null;
            Examination exam = selectedItem as Examination;
            if (exam != null)
            {
                DicomFile[] files = exam.GetList();
                rc = new String[files.Length];
                for (int x = 0; x < files.Length; x++ )
                    rc[x] = files[x].Record.DicomPathFilename;
            }
            return rc;
        }

        public override int ImportFiles(string diskRef, DateTime captureDate)
        {
            int rc = 0;

            DisplayProgressStart(Resources.StrImportingImages, 0, imageLB.Items.Count);

            string errorText = Utils.StrLFCR;
            int errorCnt = 0;

            Examination[] exams = new Examination[imageLB.Items.Count];
            imageLB.Items.CopyTo(exams, 0);
            for (int index = 0; index < exams.Length; index++)
            {
                bool allGood = true;
                DisplayProgressStep(Resources.StrImportingExam);
                foreach (DicomFile file in exams[index].GetList())
                {
                    file.Record.DiskLabel = diskRef;
                    file.Record.ArchiveDate = captureDate;
                    if (db.StoreImageRecord(file, errorHandle) == true)
                        rc++;
                    else
                    {
                        errorText += file.ToString() + " : " + file.Record.UnspecifiedErrorDetails + Utils.StrLFCR;
                        errorCnt++;
                        allGood = false;
                    }
                }
                if (allGood == true)
                    imageLB.Items.Remove(exams[index]);
            }
            DisplayProgressEnd(rc, (rc == 1) ? Resources.StrFileImported : Resources.StrFilesImported);

            if (errorCnt > 0)
                throw new UniException("OSA-073", UniException.Reset.AbortOp, UniException.Category.System, "failed to import {0} files: {1}", errorCnt, errorText);

            return rc;
        }

    }
}
