using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;

using WillStott.MxAppToolbox;

namespace osImageManagerLib
{
    public class DlgProcFileImportFile : DlgProcFileImport
    {
        public DlgProcFileImportFile(OsDatabases databases, ListBox imageListBox, ListBox correctionsListBox, ProgressBar workProgressBar, Label progressTextLabel, int delay, Program errorHandler) : base (databases, imageListBox, correctionsListBox, workProgressBar, progressTextLabel, delay, errorHandler)
        {
            dlgBrowser = new DlgBrowser();
        }

        public override string DialogTitle
        {
            get { return Resources.StrImportModeFile; }
        }

        public override int ReadFiles(string Location, string regex)
        {
            int rc = 0;
            imageLB.Items.Clear();
            correctionsLB.Items.Clear();

            DisplayProgressStart(Resources.StrLookingForFiles, 1, 10);

            DicomFile file = DicomFileSet.MakeRecordFromFile(Location);
            if (file != null)
            {
                if (file.ReadFromDisk(db, errorHandle))
                {
                    imageLB.Items.Add(file);
                    rc = 1;
                }
                else
                {
                    if (file.IsDicomHeader())        //if the file hasn't a dicom header ignore it
                        correctionsLB.Items.Add(file);
                    rc = 1;
                }
            }
            DisplayProgressEnd(rc, Resources.StrFileRead);

            if (imageLB.Items.Count > 0)
                imageLB.SelectedIndex = 0;
            if (correctionsLB.Items.Count > 0)
                correctionsLB.SelectedIndex = 0;

            return rc;
        }


        public override String OpenBrowseDlg(String LastImportDir, Control fileLocation)
        {
            String rc = "";

            if (dlgBrowser != null)
            {
                dlgBrowser.SelectedDir = LastImportDir;
                if (dlgBrowser.ShowDialog() == DialogResult.OK)
                {
                    rc = dlgBrowser.SelectedDir;
                    if (fileLocation != null)
                        fileLocation.Text = dlgBrowser.SelectedPathFilename;
                }
            }
            return rc;
        }
    }
}
