using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;

using WillStott.MxAppToolbox;

namespace osImageManagerLib
{
    public class DlgProcFileImportTree : DlgProcFileImport
    {
        public DlgProcFileImportTree(OsDatabases databases, ListBox imageListBox, ListBox correctionsListBox, ProgressBar workProgressBar, Label progressTextLabel, int delay, Program errorHandler) : base (databases, imageListBox, correctionsListBox, workProgressBar, progressTextLabel, delay, errorHandler)
        {
            dlgBrowser = new DlgBrowserDir();
        }

        public override string DialogTitle 
        {
            get { return Resources.StrImportModeTree; }
        }

        public override int ReadFiles(string Location, string regex)
        {
            int rc = 0;
            imageLB.Items.Clear();
            correctionsLB.Items.Clear();

            DisplayProgressStart(Resources.StrLookingForFiles, 1, 10);
            DicomFileSet disk = new DicomFileSet();
            DicomFile[] fileList = disk.MakeRecordsFromDirectory(Location);
            DisplayProgressEnd(fileList.Length, (fileList.Length == 1) ? Resources.StrFileFound : Resources.StrFilesFound);

            DisplayProgressStart(Resources.StrReadingFiles, 0, fileList.Length);
            foreach (DicomFile file in fileList)
            {
                DisplayProgressStep(Resources.StrReadingHeader);

                if (file.ReadFromDisk(db, errorHandle))
                    imageLB.Items.Add(file);
                else
                {
                    if (file.IsDicomHeader())        //if the file hasn't a dicom header ignore it
                        correctionsLB.Items.Add(file);
                }
                rc++;
            }
            DisplayProgressEnd(fileList.Length, (fileList.Length == 1) ? Resources.StrFileRead : Resources.StrFilesRead);

            if (imageLB.Items.Count > 0)
                imageLB.SelectedIndex = 0;
            if (correctionsLB.Items.Count > 0)
                correctionsLB.SelectedIndex = 0;

            return rc;
        }
    }
}
