using System;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;
using System.IO;
using WillStott.MxAppToolbox;

namespace osImageManagerLib
{
    public class ExaminationSet
    {
        public static readonly string DirRegexPattern = "\\bE\\w*\\b";
        private List<Examination> _examList;

        public ExaminationSet()
        {
            _examList = new List<Examination>();
        }

        public ExaminationSet(string filePath, string regex)
        {
            try
            {
                _examList = new List<Examination>();

                List<string> dirs = new List<string>();

                RecurseExamDirectoryTree(dirs, filePath, regex);

                foreach (string examDirectory in dirs)
                {
                    DicomFileSet disk = new DicomFileSet();
                    DicomFile[] fileList = disk.MakeRecordsFromDirectory(examDirectory);
                    if (fileList.Length > 0)
                    {
                        Examination exam = new Examination(examDirectory);
                        foreach (DicomFile file in fileList)
                            exam.Add(file);
                        _examList.Add(exam);
                    }
                }
            }
            catch (Exception ex)
            {
                throw new UniException("OSA-069", UniException.Reset.AbortOp, UniException.Category.Resource, ex, "File IO problem: regex[{0}], file[{1}] - msg[{2}]", (regex == null) ? "[null]" : regex, (filePath == null) ? "[null]" : filePath, ex.Message);
            }
        }

        public Examination[] GetExamList()
        {
            Examination[] arr = null;
            if (_examList != null)
            {
                arr = new Examination[_examList.Count];
                _examList.CopyTo(arr);
            }
            return arr;
        }

        public Examination GetExaminationForFile(DicomFile file)
        {
            Examination rc = new Examination(Resources.StrFileNotFoundInExams);
            if (_examList != null)
            {
                foreach (Examination exam in _examList)
                {
                    if (exam.Contains(file))
                    {
                        rc = exam;
                        break;
                    }
                }
            }
            return rc;
        }

        public int FileCount
        {
            get
            {
                int rc = 0;
                if (_examList != null)
                {
                    foreach (Examination exam in _examList)
                        rc += exam.Count;
                }
                return rc;
            }
        }

        public DicomFile[] GetFileList()
        {
            DicomFile[] arr = null;
            if (FileCount > 0)
            {
                arr = new DicomFile[FileCount];
                int cnt = 0;
                foreach (Examination exam in _examList)
                {
                    exam.GetList().CopyTo(arr, cnt);
                    cnt += exam.Count;
                }
            }
            return arr;
        }

        private void RecurseExamDirectoryTree(List<string> examDirectories, string filePath, string regex)
        {
            FileSystemInfo info = null;
            try
            {
                Regex expression = new Regex(regex);
                DirectoryInfo dir = new DirectoryInfo(filePath);
                FileSystemInfo[] items = dir.GetFileSystemInfos();
                foreach (FileSystemInfo item in items)
                {
                    info = item;
                    if (info is DirectoryInfo)
                    {
                        if (expression.Match(info.Name).Success)
                            examDirectories.Add(info.FullName + "\\");
                        RecurseExamDirectoryTree(examDirectories, info.FullName, regex);
                    }
                }
            }
            catch (Exception)
            {
                throw;
            }
        }

    }
}
