using System;
using System.Collections.Generic;
using System.Reflection;
using System.Configuration;
using System.Text;
using WillStott.MxAppToolbox;

namespace osImageManagerLib
{
    public class ExecutionTime
    {
        public string ScriptsDir { get { return InstallDir + "DbScripts\\"; } }
        public string OutputDir { get { return InstallDir + "Output\\"; } }
        public string DataDir { get { return InstallDir + "Data\\"; } }
        public string ImageDir { get { return DataDir + "IMAGE\\"; } }
        public string FITDir { get { return DataDir + "FIT\\"; } }
        public string DataDirMonth0 { get { return DataDir + "Y2007M07\\"; } }
        public string DataDirDay0 { get { return DataDirMonth0 + "D21\\"; } }
        public string DataDirExam0 { get { return DataDirDay0 + "E000002\\"; } }
        public string DataDirMonth1 { get { return DataDir + "Y2007M03\\"; } }
        public string DataDirDay1 { get { return DataDirMonth1 + "D10\\"; } }
        public string DataDirExam1 { get { return DataDirDay1 + "E000003\\"; } }
        public string DataDirMonth2 { get { return DataDir + "Y2007M06\\"; } }
        public string DataDirDay2 { get { return DataDirMonth2 + "D14\\"; } }
        public string DataDirExam2 { get { return DataDirDay2 + "E000001\\"; } }

        public string MonochromeSampleDicomFile { get { return DataDir + "MonochromeSample.dicom"; } }
        public string ColourSampleDicomFile { get { return DataDir + "ColourSample.dicom"; } }
        public string InvalidSampleDicomFile { get { return DataDir + "InvalidSample.dicom"; } }

        public string ImageDir1DicomFileA { get { return DataDirExam1 + "MonochromeSample.dicom"; } }
        public string ImageDir2DicomFileA { get { return DataDirExam2 + "MonochromeSample.dicom"; } }
        public string ImageDir2DicomFileB { get { return DataDirExam2 + "ColourSample.dicom"; } }

        private static readonly string configFileName = "osImageManagerApp.exe.config";
        private static ExecutionTime _instance;
        private string _installDir;

        private ExecutionTime() { }
        static ExecutionTime()
        {
            _instance = new ExecutionTime();
            _instance._installDir = "";
        }
        static public ExecutionTime Instance
        {
            get { return _instance; }
        }

        public static Configuration Configuration
        {
            get
            {
                Configuration config = null;

                ExeConfigurationFileMap file = new ExeConfigurationFileMap();
                file.ExeConfigFilename = ExecutionTime.Instance.InstallDir + configFileName;
                file.LocalUserConfigFilename = ExecutionTime.Instance.InstallDir + configFileName;
                config = ConfigurationManager.OpenMappedExeConfiguration(file, ConfigurationUserLevel.None);

                return config;
            }
        }

        public string InstallDir 
        { 
            get 
            {
                if (_installDir.Length == 0)
                {
                    Assembly resAsm = Program.GetResourceAssembly(this);
                    _installDir = Utils.GetPath(resAsm.Location);
                }
                return _installDir; 
            } 
        }
    }
}
