using System;
using System.Collections.Generic;
using System.Text;

namespace osImageManagerLib
{
    public class Favorites : ICollection<Favorite>
    {
        private Dictionary<string, Favorite> favoriteDictionary = new Dictionary<string, Favorite>();

        public int Count { get { return favoriteDictionary.Count; } }
        public bool IsReadOnly  { get { return false; } }

        public void Add(Favorite favorite) 
        {
            if (favorite.Label == null)
                throw new ArgumentNullException("label cannot be null");
            if (favorite.Uri == null)
                throw new ArgumentNullException("uri cannot be null");
            if (Contains(favorite))
                throw new ArgumentException("duplicate label");

            favoriteDictionary.Add(favorite.Label, favorite); 
        }
        public bool Remove(Favorite favorite) 
        { 
            return favoriteDictionary.Remove(favorite.Label); 
        }
        public Favorite Find(string label)
        {
            return favoriteDictionary.ContainsKey(label) ? favoriteDictionary[label] : null;
        }
        public void Clear() 
        { 
            favoriteDictionary.Clear(); 
        }

        public Uri this[String label]
        {
            get
            {
                return (favoriteDictionary.ContainsKey(label)) ? favoriteDictionary[label].Uri : null;
            }
        }

        #region ICollection<Favorite> Members


        public bool Contains(Favorite favorite)
        {
            return favoriteDictionary.ContainsKey(favorite.Label); 
        }

        public void CopyTo(Favorite[] array, int arrayIndex)
        {
            favoriteDictionary.Values.CopyTo(array, arrayIndex);
            return;
        }



        #endregion

        #region IEnumerable<Favorite> Members

        public IEnumerator<Favorite> GetEnumerator()
        {
            return favoriteDictionary.Values.GetEnumerator();
        }

        #region IEnumerable Members

        System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
        {
            return favoriteDictionary.GetEnumerator();
        }

        #endregion

        #endregion
    }
}
