using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Configuration;
using System.Reflection;
using System.Text;
using System.IO;
using WillStott.MxAppToolbox;

namespace osImageManagerLib
{
    public sealed class RegionalCentres
    {
        RegionalCentresCollection _regionalCentresCollection;

        public RegionalCentres()
        {
            Load();
        }

        public bool Load()
        {
            bool rc = false;
            try
            {
                RegionalCentresSection centreSection = ExecutionTime.Configuration.Sections["RegionalCentres"] as RegionalCentresSection;
                _regionalCentresCollection = centreSection.RegionalCentres;
            }
            catch (Exception ex)
            {
                throw new UniException("OSA-040", UniException.Reset.AbortOp, UniException.Category.Resource, ex, "config file is bad");
            }
            return rc;
        }


        public RegionalCentreElement[] GetList()
        {
            RegionalCentreElement[] arr = new RegionalCentreElement[_regionalCentresCollection.Count];
            _regionalCentresCollection.CopyTo(arr, 0);
            return arr;
        }
    }

    public sealed class RegionalCentresSection : ConfigurationSection
    {
        public RegionalCentresSection()
        {
        }

        [ConfigurationProperty("", IsDefaultCollection = true)]
        public RegionalCentresCollection RegionalCentres
        {
            get { return (RegionalCentresCollection)base[""]; }
        }
    }

    public sealed class RegionalCentresCollection : ConfigurationElementCollection
    {
        protected override ConfigurationElement CreateNewElement()
        {
            return new RegionalCentreElement();
        }
        protected override object GetElementKey(ConfigurationElement element)
        {
            return ((RegionalCentreElement)element).Name;
        }
        public override ConfigurationElementCollectionType CollectionType
        {
            get { return ConfigurationElementCollectionType.BasicMap; }
        }
        protected override string ElementName
        {
            get { return "RegionalCentre"; }
        }
    }

    public sealed class RegionalCentreElement : ConfigurationElement
    {
        [ConfigurationProperty("name", IsKey = true, IsRequired = true)]
        public string Name
        {
            get { return (string)base["name"]; }
            set { base["name"] = value; }
        }
        [ConfigurationProperty("number", IsRequired = true)]
        public string Number
        {
            get { return (string)base["number"]; }
            set { base["number"] = value; }
        }

        public override string ToString()
        {
            string output = "";
            output += string.Format("{0}", Name).PadRight(20, ' ');
            output += string.Format("{0}", Number);

            return output;
        }
    }
}

 

 



