using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Configuration;
using System.Reflection;
using System.Text;
using System.IO;
using WillStott.MxAppToolbox;


namespace osImageManagerLib
{
    public sealed class SqlScripts
    {
        private string _path;
        SqlScriptsCollection _scriptsCollection;


        public SqlScripts()
        {
            Load();
        }

        public bool Load()
        {
            bool rc = false;
            try
            {
                SqlScriptsSection scriptSection = ExecutionTime.Configuration.Sections["SqlScripts"] as SqlScriptsSection;
                _path = scriptSection.FilePath;
                _scriptsCollection = scriptSection.SqlScripts;
            }
            catch (Exception ex)
            {
                throw new UniException("OSA-001", UniException.Reset.AbortOp, UniException.Category.Resource, ex, "config file is bad");
            }
            return rc;
        }

        public SqlScriptElement[] GetList()
        {
            SqlScriptElement[] arr = new SqlScriptElement[_scriptsCollection.Count];
            _scriptsCollection.CopyTo(arr, 0);
            return arr;
        }

        public string Directory
        {
            get { return _path; }
        }
    }

    public sealed class SqlScriptsSection : ConfigurationSection
    {
        public SqlScriptsSection()
        {
        }

        [ConfigurationProperty( "", IsDefaultCollection = true )]
        public SqlScriptsCollection SqlScripts
        {
            get{ return (SqlScriptsCollection)base[""]; }
        }
        [ConfigurationProperty("filePath", IsRequired = true)]
        public string FilePath
        {
            get { return (string)base["filePath"]; }
            set { base["filePath"] = value; }
        }
    }

    public sealed class SqlScriptsCollection : ConfigurationElementCollection
    {
        protected override ConfigurationElement CreateNewElement()
        {
            return new SqlScriptElement();
        }
        protected override object GetElementKey( ConfigurationElement element )
        {
            return ((SqlScriptElement)element).Name;
        }
        public override ConfigurationElementCollectionType CollectionType
        {
            get { return ConfigurationElementCollectionType.BasicMap;}
        }
        protected override string ElementName
        {
            get { return "SqlScript"; }
        }
    }

    public sealed class SqlScriptElement : ConfigurationElement
    {
        [ConfigurationProperty("name", IsKey = true, IsRequired = true)]
        public string Name
        {
            get { return (string)base["name"]; }
            set { base["name"] = value; }
        }
        [ConfigurationProperty("fileName", IsRequired = true)]
        public string FileName
        {
            get { return (string)base["fileName"]; }
            set { base["fileName"] = value; }
        }
        [ConfigurationProperty("scriptType", IsRequired = true)]
        public string ScriptType
        {
            get { return (string)base["scriptType"]; }
            set { base["scriptType"] = value; }
        }
        [ConfigurationProperty("runBy", IsRequired = true)]
        public string RunBy
        {
            get { return (string)base["runBy"]; }
            set { base["runBy"] = value; }
        }
        [ConfigurationProperty("dbName", IsRequired = true)]
        public string DbName
        {
            get { return (string)base["dbName"]; }
            set { base["dbName"] = value; }
        }
        [ConfigurationProperty("connName", IsRequired = true)]
        public string ConnName
        {
            get { return (string)base["connName"]; }
            set { base["connName"] = value; }
        }
        [ConfigurationProperty("cmdType", IsRequired = true)]
        public string CmdType
        {
            get { return (string)base["cmdType"]; }
            set { base["cmdType"] = value; }
        }
        [ConfigurationProperty("showResults", IsRequired = true)]
        public string ShowResults
        {
            get { return (string)base["showResults"]; }
            set { base["showResults"] = value; }
        }
        [ConfigurationProperty("comment", IsRequired = true)]
        public string Comment
        {
            get { return (string)base["comment"]; }
            set { base["comment"] = value; }
        }

        public override string ToString()
        {
            string output = "";
            output += string.Format("{0}\t", Name).PadRight(20, ' ');
            output += string.Format("{0}\t", DbName).PadRight(20, ' ');
            output += string.Format("{0}\t", ScriptType).PadRight(10, ' ');
            output += string.Format("{0}", FileName);

            return output;
        }

        public bool DoConfirm()
        {
            return (Comment.Length > 0) ? true : false;
        }

        public string ConfirmMsg()
        {
            return Resources.StrMsgScript + " " + Name + "\n\n" + Comment + "\n\n" + Resources.StrMsgRunThisScript;
        }

        public string GetScript(string filePath)
        {
            string rc = "";

            try
            {
                using ( FileStream fs = new FileStream(filePath + "\\" + FileName, FileMode.Open))
                {
                    using (StreamReader reader = new StreamReader(fs, Encoding.ASCII))
                    {
                        StringBuilder result = new StringBuilder();
                        string line = null;
                        while ((line = reader.ReadLine()) != null)
                            result.Append(line + "\n");
                        rc = result.ToString();
                    }
                }
            }
            catch (Exception ex)
            {
                throw new UniException("OSA-005", UniException.Reset.AbortOp, UniException.Category.Resource, ex, "script file is bad: {0}", filePath + FileName);
            }
            return rc;
        }
    }
}

 

 


