using System;
using System.Collections.Generic;
using System.Text;
using System.Globalization;


using WillStott.MxAppToolbox;
using System.Data.SqlClient;
using System.Data;

namespace osImageManagerLib
{
    sealed public class SqlUtils
    {
        private SqlUtils()
        {

        }

        public static string getConnectionElement(string Connection, string tag)
        {
            string rc = "";
            if ((Connection != null) && (tag != null))
            {
                int start = Connection.IndexOf(tag);
                if (start != -1)
                {
                    start += tag.Length;
                    int end = Connection.IndexOf(';', start);
                    if (end != -1)
                        rc = Connection.Substring(start, end - start);
                }
            }
            return rc;
        }
        public static SqlParameter CreateVarCharParam(SqlCommand cmd, string ParamName, int ParamSize, bool Output, string Value)
        {
            SqlParameter param = null;
            if ((cmd != null) && (ParamName != null))
            {
                param = cmd.CreateParameter();
                param.ParameterName = ParamName;
                param.SqlDbType = SqlDbType.VarChar;
                param.Size = ParamSize;
                if (Output == true)
                    param.Direction = ParameterDirection.Output;
                else
                {
                    param.Direction = ParameterDirection.Input;
                    param.Value = Value;
                }
                cmd.Parameters.Add(param);
            }
            return param;
        }

        public static SqlParameter CreateIntParam(SqlCommand cmd, string ParamName, bool Output, int Value)
        {
            SqlParameter param = null;
            if ((cmd != null) && (ParamName != null))
            {
                param = cmd.CreateParameter();
                param.ParameterName = ParamName;
                param.SqlDbType = SqlDbType.Int;
                if (Output == true)
                    param.Direction = ParameterDirection.Output;
                else
                {
                    param.Direction = ParameterDirection.Input;
                    param.Value = Value;
                }
                cmd.Parameters.Add(param);
            }
            return param;
        }

        public static SqlParameter CreateDateTimeParam(SqlCommand cmd, string ParamName, bool Output, DateTime Value)
        {
            SqlParameter param = null;
            if ((cmd != null) && (ParamName != null))
            {
                param = cmd.CreateParameter();
                param.ParameterName = ParamName;
                param.SqlDbType = SqlDbType.DateTime;
                if (Output == true)
                    param.Direction = ParameterDirection.Output;
                else
                {
                    param.Direction = ParameterDirection.Input;
                    param.Value = Value;
                }
                cmd.Parameters.Add(param);
            }
            return param;
        }

        public static SqlParameter CreateUniqueIDParam(SqlCommand cmd, string ParamName, bool Output, Guid Value)
        {
            SqlParameter param = null;
            if ((cmd != null) && (ParamName != null))
            {
                param = cmd.CreateParameter();
                param.ParameterName = ParamName;
                param.SqlDbType = SqlDbType.UniqueIdentifier;
                if (Output == true)
                    param.Direction = ParameterDirection.Output;
                else
                {
                    param.Direction = ParameterDirection.Input;
                    param.Value = Value;
                }
                cmd.Parameters.Add(param);
            }
            return param;
        }

        public static SqlParameter CreateVarBinaryParam(SqlCommand cmd, string ParamName, int size, bool Output, byte[] Value)
        {
            SqlParameter param = null;
            if ((cmd != null) && (ParamName != null))
            {
                param = cmd.CreateParameter();
                param.ParameterName = ParamName;
                param.SqlDbType = SqlDbType.VarBinary;
                param.Size = size;
                if (Output == true)
                    param.Direction = ParameterDirection.Output;
                else
                {
                    param.Direction = ParameterDirection.Input;
                    param.Value = Value;
                }
                cmd.Parameters.Add(param);
            }
            return param;
        }

        public static SqlParameter CreateImageParam(SqlCommand cmd, string ParamName, int size, bool Output, byte[] Value)
        {
            SqlParameter param = null;
            if ((cmd != null) && (ParamName != null))
            {
                param = cmd.CreateParameter();
                param.ParameterName = ParamName;
                param.SqlDbType = SqlDbType.Image;
                param.Size = size;
                if (Output == true)
                    param.Direction = ParameterDirection.Output;
                else
                {
                    param.Direction = ParameterDirection.Input;
                    param.Value = Value;
                }
                cmd.Parameters.Add(param);
            }
            return param;
        }

        public static string ExecuteScript(SqlConnection con, string CmdType, string script)
        {
            string rc = "";
            if ((con != null) && (CmdType != null) && (script != null))
            {
                SqlCommand cmd = con.CreateCommand();
                cmd.CommandType = CommandType.Text;
                cmd.CommandText = script;

                con.Open();

                if (String.Compare(CmdType, "READER", true) == 0)
                    rc = SqlUtils.GetResultsFromCommandExecution(cmd);
                else if (String.Compare(CmdType, "SCALAR", true) == 0)
                    rc = cmd.ExecuteScalar().ToString();
                else if (String.Compare(CmdType, "NONQUERY", true) == 0)
                    rc = cmd.ExecuteNonQuery().ToString();
                else
                    throw new InvalidOperationException("invalid command type");
            }
            return rc;
        }

        public static string GetResultsFromCommandExecution(SqlCommand cmd)
        {
            string rc = "";

            if (cmd != null)
            {
                SqlDataReader reader = cmd.ExecuteReader();
                DataTable pub = new DataTable();
                pub.Locale = CultureInfo.InvariantCulture;
                pub.Load(reader, LoadOption.Upsert);

                StringBuilder buffer = new StringBuilder();
                for (int x = 0; x < pub.Columns.Count; x++)
                {
                    DataColumn dc = pub.Columns[x];
                    buffer.Append(string.Format("{0,15}\t", dc.ColumnName));
                }
                buffer.Append(Utils.StrLFCR);

                foreach (DataRow row in pub.Rows)
                {
                    for (int x = 0; x < pub.Columns.Count; x++)     // foreach (DataColumn col in pub.Columns) gives coverage leak!
                    {
                        DataColumn col = pub.Columns[x];
                        buffer.Append(string.Format("{0,15}\t", row[col]));
                    }
                    buffer.Append(Utils.StrLFCR);
                }
                rc = buffer.ToString();
            }
            return rc;
        }
    }
}
