using System;
using System.IO;
using System.Globalization;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using WillStott.MxAppToolbox;

namespace osImageManagerLib
{
    sealed public class TestSuiteSetup
    {
        static public readonly string ProjectRootFromUnitTestRuntime = "..\\..\\..\\";

        static public readonly string ConfigDbScripts0Name = "a. TestImageDb        ";
        static public readonly string ConfigDbScripts0Filename = "TestImageDbSQL9.sql";
        static public readonly string ConfigDbScripts0dbName = "osImage";
        static public readonly string ConfigDbScripts0scriptType = "UAT-Test-SQL9    ";
        static public readonly string ConfigDbScripts0cmdType = "Reader";
        static public readonly string ConfigDbScripts0ShowResults = "yes";
        static public readonly string ConfigDbScripts0Comment = "Display records in Image table for DiskRef=UAT";

        //database setup
        static public readonly string ImageDb = "osImage";
        static public readonly string PatientDb = "ukctocs";
        static public readonly string ImageDataSource = ".\\SQLEXPRESS";
        static public readonly string PatientDataSource = ".\\SQLEXPRESS";
        static public readonly string SamplePatientDbSQL = "SELECT * FROM [" + PatientDb + "].[dbo].[UltrascanResults] GO";
        static public readonly string SampleImageDbSQL = "SELECT * FROM [" + ImageDb + "].[dbo].[Image] GO";
        static public readonly string BadImageDbSQL = "SELECT * FROM [" + ImageDb + "].[dbo].[Image]";

        static public readonly string AdminUser = "URA-Admin";
        static public readonly string AdminUserPassword = "password";
        static public readonly string OpUser = "URA-Operator";
        static public readonly string OpUserPassword = "password";
        static public readonly string GuestUser = "URA-Guest";
        static public readonly string GuestUserPassword = "password";

        static public readonly string connectImageStr = "Persist Security Info=False;Data Source=" + TestSuiteSetup.ImageDataSource + ";Database=" + TestSuiteSetup.ImageDb + ";Integrated Security=false;User ID=" + AdminUser + ";Password=" + AdminUserPassword;
        static public readonly string connectPatientStr = "Persist Security Info=False;Data Source=" + TestSuiteSetup.PatientDataSource + ";Database=" + TestSuiteSetup.PatientDb + ";Integrated Security=false;User ID=" + AdminUser + ";Password=" + AdminUserPassword;

        static public readonly string SampleDbScript = "AddOperatorImageSQL9.sql";
        static public readonly string SampleDbScriptHeader = "/****** AddOperatorImageSQL9.sql - run by 'sa' to Create a User and Logon Permissions on a database engine (UAT/Production)                    ******/";

                //these must be added to ImageDb during setup - see OsDatabasesTests.CreateTestImageRecord()
        static public readonly string TestImageSonoViewDicomRef = "1.2." + "410.200001" + ".1.9900.1.20050531132655297.1.1";
        static public readonly string TestImageDicomRef = "1.2." + ImageRecord.CranfieldCountryOrgCode + ".1.9900.1.20050531132655297.1.1" + ".0";

                //these must be added to PatientDb during setup - set UATSetup.sql
        static public readonly string ValidPatientName = "Stott^William^Philip^Quentin";
        static public readonly string ValidPatientSurname = "Stott";
        static public readonly string ValidPatientFirstNames = "William Philip Quentin";
        static public readonly string ValidVolunteerRef = "99000002";
        static public readonly DateTime ValidAppointmentDateForVolunteerRef = new DateTime(2005, 1, 21, 23, 59, 59);

                //DicomMask - Default values initialized in class
        static public readonly bool DicomMaskDefaultMaskData = false;           //true if a mask is to be applied - a rectangle in image
        static public readonly int DicomMaskDefaultLeft = 39;                   //value of mask first left column  
        static public readonly int DicomMaskDefaultWidth = 201;                 //number of columns in the mask
        static public readonly int DicomMaskDefaultStartrow = 15;               //value of mask first row
        static public readonly int DicomMaskDefaultEndrow = 37;                 //value of mask last row
        static public readonly string DicomMaskDefaultMaskDir = "MaskFragments";  //directory for holding mask data removed from image - filename is DicomRef (if length == 0, then don't save mask)
        static public readonly byte DicomMaskDefaultMaskImageValue = 0xF0;      //mask is almost white
        static public readonly byte DicomMaskDefaultMaskPatientNameChar = 0x79; //ASCII 'Y'
        static public readonly ushort DicomMaskDefaultMaskPatientNameCnt = 4;   //PatientName=YYYY

                //DicomMask - runTimeConfigFileConfiguration file values
        static public readonly bool DicomMaskConfigMaskData = true;           //true if a mask is to be applied - a rectangle in image
        static public readonly int DicomMaskConfigLeft = 40;                   //value of mask first left column  
        static public readonly int DicomMaskConfigWidth = 200;                 //number of columns in the mask
        static public readonly int DicomMaskConfigStartrow = 16;               //value of mask first row
        static public readonly int DicomMaskConfigEndrow = 36;                 //value of mask last row
        static public readonly byte DicomMaskConfigMaskImageValue = 0xFF;      //mask is white
        static public readonly byte DicomMaskConfigMaskPatientNameChar = 0x78; //ASCII 'X'
        static public readonly ushort DicomMaskConfigMaskPatientNameCnt = 3;   //PatientName=XXX

        static public readonly string CranfieldImportRev = ".0";               //sample files start with rev 0 and are incremented when duplicates are found in the database

        static public readonly string Y2004M01Dir = "\\Y2004M01";
        static public readonly string D03Dir0 = "\\D03";
        static public readonly string D04Dir0 = "\\D04";

        static public readonly string Y2004M02Dir = "\\Y2004M02";
        static public readonly string D03Dir = "\\D03";
        static public readonly string D04Dir = "\\D04";
        static public readonly string E0000001Dir = "\\E0000001";
        static public readonly string D05Dir = "\\D05";
        static public readonly string E0000001Dir2 = "\\E0000001";
        static public readonly string E0000002Dir2 = "\\E0000002";
        static public readonly string D06Dir = "\\D06";
        static public readonly string E0000001Dir3 = "\\E0000001";
        static public readonly string E0000002Dir3 = "\\E0000002";
        static public readonly string E0000003Dir3 = "\\E0000003";

        static public readonly string Y2005M04Dir = "\\Y2005M04";
        static public readonly string D02Dir = "\\D02";
        static public readonly string Y2005M05Dir = "\\Y2005M05";

        static public readonly string TestFile999Filename = "Test999";
        static public readonly string TestFile999DicomRef = "1.2.410.200001.1.9900.1.20050531130000999.1.1";

        static public readonly string TestFile000PatientName = "Will^Stott";
        static public readonly string TestFile000PatientID = "99000002";
        static public readonly string TestFile000StudyDate = "20050121";
        static public readonly string TestFile000CranfieldDicomRef = "1.2.826.999999.1.9900.1.20050531130000000.1.1.0";

        static public readonly string TestFile000Filename = "Test000";
        static public readonly string TestFile000DicomRef = "1.2.410.200001.1.9900.1.20050531130000000.1.1";
        static public readonly string TestFile000 = Y2004M02Dir + D04Dir + E0000001Dir + "\\" + TestFile000Filename;
        static public readonly string TestFile001DicomRef = "1.2.410.200001.1.9900.1.20050531130000001.1.1";
        static public readonly string TestFile001 = Y2004M02Dir + D04Dir + E0000001Dir + "\\Test001.dicom";
        static public readonly string TestFile002DicomRef = "1.2.410.200001.1.9900.1.20050531130000002.1.1";
        static public readonly string TestFile002 = Y2004M02Dir + D04Dir + E0000001Dir + "\\Test002";
        static public readonly string TestFile003DicomRef = "1.2.410.200001.1.9900.1.20050531130000003.1.1";
        static public readonly string TestFile003 = Y2004M02Dir + D04Dir + E0000001Dir + "\\Test003.dicom";
        static public readonly string TestFile004DicomRef = "1.2.410.200001.1.9900.1.20050531130000004.1.1";
        static public readonly string TestFile004 = Y2004M02Dir + D04Dir + E0000001Dir + "\\Test004";
        static public readonly string TestFile005DicomRef = "1.2.410.200001.1.9900.1.20050531130000005.1.1";
        static public readonly string TestFile005 = Y2004M02Dir + D04Dir + E0000001Dir + "\\Test005.dicom";
        static public readonly string TestFile006DicomRef = "1.2.410.200001.1.9900.1.20050531130000006.1.1";
        static public readonly string TestFile006 = Y2004M02Dir + D04Dir + E0000001Dir + "\\Test006";
        static public readonly string TestFile007DicomRef = "1.2.410.200001.1.9900.1.20050531130000007.1.1";
        static public readonly string TestFile007 = Y2004M02Dir + D04Dir + E0000001Dir + "\\Test007.dicom";
        static public readonly string TestFile008DicomRef = "1.2.410.200001.1.9900.1.20050531130000008.1.1";
        static public readonly string TestFile008 = Y2004M02Dir + D04Dir + E0000001Dir + "\\Test008";
        static public readonly string TestFile009DicomRef = "1.2.410.200001.1.9900.1.20050531130000009.1.1";
        static public readonly string TestFile009 = Y2004M02Dir + D04Dir + E0000001Dir + "\\Test009.dicom";
        static public readonly string TestFile010DicomRef = "1.2.410.200001.1.9900.1.20050531130000010.1.1";
        static public readonly string TestFile010 = Y2004M02Dir + D04Dir + E0000001Dir + "\\Test010.dicom";

        static public readonly string TestFile011DicomRef = "1.2.410.200001.1.9900.1.20050531130000011.1.1";
        static public readonly string TestFile011 = Y2004M02Dir + D05Dir + E0000001Dir2 + "\\Test011.dicom";
        static public readonly string TestFile012DicomRef = "1.2.410.200001.1.9900.1.20050531130000012.1.1";
        static public readonly string TestFile012 = Y2004M02Dir + D05Dir + E0000001Dir2 + "\\Test012.dicom";
        static public readonly string TestFile013DicomRef = "1.2.410.200001.1.9900.1.20050531130000013.1.1";
        static public readonly string TestFile013 = Y2004M02Dir + D05Dir + E0000001Dir2 + "\\Test013.dicom";
        static public readonly string TestFile014DicomRef = "1.2.410.200001.1.9900.1.20050531130000014.1.1";
        static public readonly string TestFile014 = Y2004M02Dir + D05Dir + E0000001Dir2 + "\\Test014.dicom";
        static public readonly string TestFile015DicomRef = "1.2.410.200001.1.9900.1.20050531130000015.1.1";
        static public readonly string TestFile015 = Y2004M02Dir + D05Dir + E0000001Dir2 + "\\Test015.dicom";
        static public readonly string TestFile016DicomRef = "1.2.410.200001.1.9900.1.20050531130000016.1.1";
        static public readonly string TestFile016 = Y2004M02Dir + D05Dir + E0000001Dir2 + "\\Test016.dicom";
        static public readonly string TestFile017DicomRef = "1.2.410.200001.1.9900.1.20050531130000017.1.1";
        static public readonly string TestFile017 = Y2004M02Dir + D05Dir + E0000001Dir2 + "\\Test017.dicom";
        static public readonly string TestFile018DicomRef = "1.2.410.200001.1.9900.1.20050531130000018.1.1";
        static public readonly string TestFile018 = Y2004M02Dir + D05Dir + E0000001Dir2 + "\\Test018.dicom";
        static public readonly string TestFile019DicomRef = TestFile000DicomRef;
        static public readonly string TestFile019 = Y2004M02Dir + D05Dir + E0000001Dir2 + "\\Test019.dicom";
        static public readonly string TestFile020DicomRef = "1.2.410.200001.1.9900.1.20050531130000020.1.1";
        static public readonly string TestFile020 = Y2004M02Dir + D05Dir + E0000001Dir2 + "\\Test020.dicom";
        static public readonly string TestFile021DicomRef = "1.2.410.200001.1.9900.1.20050531130000021.1.1";
        static public readonly string TestFile021 = Y2004M02Dir + D05Dir + E0000001Dir2 + "\\Test021.dicom";
        static public readonly string TestFile022DicomRef = "1.2.410.200001.1.9900.1.20050531130000022.1.1";
        static public readonly string TestFile022 = Y2004M02Dir + D05Dir + E0000001Dir2 + "\\Test022.dicom";

        static public readonly string TestFile023DicomRef = "1.2.410.200001.1.9900.1.20050531130000023.1.1";
        static public readonly string TestFile023 = Y2004M02Dir + D04Dir + E0000001Dir + "\\Test023.dico";

        static public readonly string TestFile024 = Y2004M02Dir + D05Dir + E0000002Dir2 + "\\Test024.dicom";

        static private bool _FilesCreated = false;

        private TestSuiteSetup()
        {

        }

        public static bool IsTestImageDirectoryPresent()
        {
            return Directory.Exists(ExecutionTime.Instance.ImageDir);
        }

        public static bool CreateTestImageDirectories()
        {
            bool rc = false;
            try
            {
                string dir = ExecutionTime.Instance.ImageDir;
                if (Directory.Exists(dir))
                    Directory.Delete(dir, true);

                Directory.CreateDirectory(dir);

                Directory.CreateDirectory(dir + Y2004M01Dir);
                Directory.CreateDirectory(dir + Y2004M01Dir + D03Dir0);
                Directory.CreateDirectory(dir + Y2004M01Dir + D04Dir0);
                Directory.CreateDirectory(dir + Y2004M02Dir);
                Directory.CreateDirectory(dir + Y2004M02Dir + D03Dir);
                Directory.CreateDirectory(dir + Y2004M02Dir + D04Dir);
                Directory.CreateDirectory(dir + Y2004M02Dir + D04Dir + E0000001Dir);
                Directory.CreateDirectory(dir + Y2004M02Dir + D05Dir);
                Directory.CreateDirectory(dir + Y2004M02Dir + D05Dir + E0000001Dir2);
                Directory.CreateDirectory(dir + Y2004M02Dir + D05Dir + E0000002Dir2);
                Directory.CreateDirectory(dir + Y2004M02Dir + D06Dir);
                Directory.CreateDirectory(dir + Y2004M02Dir + D06Dir + E0000001Dir3);
                Directory.CreateDirectory(dir + Y2004M02Dir + D06Dir + E0000002Dir3);
                Directory.CreateDirectory(dir + Y2004M02Dir + D06Dir + E0000003Dir3);
                Directory.CreateDirectory(dir + Y2005M04Dir);
                Directory.CreateDirectory(dir + Y2005M04Dir + D02Dir);
                Directory.CreateDirectory(dir + Y2005M05Dir);

                rc = true;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error!");
            }
            return rc;
        }

        public static void CreateTestImageFile000()
        {
            DicomFile file = new DicomFile();
            file.TestReadFromDisk(ExecutionTime.Instance.MonochromeSampleDicomFile);

            file.Record.PatientName = TestFile000PatientName;
            file.Record.VolunteerRef = TestFile000PatientID;
            file.Record.AppointmentDate = new DateTime(int.Parse(TestFile000StudyDate.Substring(0, 4)), int.Parse(TestFile000StudyDate.Substring(4, 2)), int.Parse(TestFile000StudyDate.Substring(6, 2)));

            file.Record.DicomRef = TestFile000DicomRef;
            file.TestUpdateCacheFromRecord(ExecutionTime.Instance.DataDir + TestFile000Filename);     
        }

        public static bool CheckTestImageFile000()
        {
            return File.Exists(ExecutionTime.Instance.DataDir + TestFile000Filename);
        }

        public static void DeleteTestImageFile000()
        {
            if (File.Exists(ExecutionTime.Instance.DataDir + TestFile000Filename))
                File.Delete(ExecutionTime.Instance.DataDir + TestFile000Filename);
        }


        public static void CreateTestImageFile999()
        {               //used for SOPInstanceUIDValidation
            DicomFile file = new DicomFile();
            file.TestReadFromDisk(ExecutionTime.Instance.MonochromeSampleDicomFile);

            file.Record.PatientName = TestFile000PatientName;
            file.Record.VolunteerRef = TestFile000PatientID;
            file.Record.AppointmentDate = new DateTime(int.Parse(TestFile000StudyDate.Substring(0, 4)), int.Parse(TestFile000StudyDate.Substring(4, 2)), int.Parse(TestFile000StudyDate.Substring(6, 2)));

            file.Record.DicomRef = TestFile999DicomRef;
            file.TestUpdateCacheFromRecord(ExecutionTime.Instance.DataDir + TestFile999Filename);
        }

        public static bool CheckTestImageFile999()
        {
            return File.Exists(ExecutionTime.Instance.DataDir + TestFile999Filename);
        }

        public static void DeleteTestImageFile999()
        {
            if (File.Exists(ExecutionTime.Instance.DataDir + TestFile999Filename))
                File.Delete(ExecutionTime.Instance.DataDir + TestFile999Filename);
        }

        public static bool CreateTestImageFiles()
        {
            bool rc = false;

            DicomFile file = new DicomFile();
            try
            {

                //changes to DicomFile
                //1. Add TestUpdateCacheFromRecord()
                //  public void TestUpdateCacheFromRecord(string filename)
                //  {
                //      ProcessCache(ProcessCacheAction.UpdateCacheFromRecordHeaderFields, null);
                //      WriteToDisk(null, false, filename, null);
                //  }
                //2. Alter UpdateCacheFromRecordHeaderFields()
                //  if ((Group == 0x0010) && (Element == 0x0010))       //PatientName
                //  {
                //      value = new byte[encode.GetByteCount(_record.PatientName)];
                //      value = encode.GetBytes(_record.PatientName);
                //!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
                //      //          value = new byte[DicomMask.MaskPatientNameCnt];
                //      //          for (int y = 0; y < DicomMask.MaskPatientNameCnt; y++)
                //      //               value[y] = DicomMask.MaskPatientNameChar;
                //!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
                //  }
                //  if ((Group == 0x7FE0) && (Element == 0x0010))       //image data - for mask, 
                //  {
                //      //!!!!   _mask.ApplyMask(value, _rowsImage, _colsImage);
                //  }
                //3. Alter GetDataElementLength()
                //  if ((Group == 0x0010) && (Element == 0x0010))       //PatientName
                //  {
                //      dataLength = (Uint16) ? ReadUint16((ushort)_record.PatientName.Length, binreader, binwriter) : (int)ReadUint32((uint)_record.PatientName.Length, binreader, binwriter);
                //!!!!!!!!!!!!!!!!!!!!!!!
                //     //!!!!!  dataLength = (Uint16) ? ReadUint16(DicomMask.MaskPatientNameCnt, binreader, binwriter) : (int)ReadUint32(DicomMask.MaskPatientNameCnt, binreader, binwriter);
                //!!!!!!!!!!!!!!!!!!!!!!!!
                //  }

                CreateTestImageFile000(); //writes to datadir so UAT can separately read this file and import it before importing the rest - gives test019 as duplicate
                CreateTestImageFile999(); //create file for SOPInstanceUIDValidation, 

                file.TestReadFromDisk(ExecutionTime.Instance.MonochromeSampleDicomFile);
                file.Record.PatientName = "William^StOTT";
                file.Record.VolunteerRef = "99000002U";
                file.Record.AppointmentDate = new DateTime(2005, 01, 21);
                file.Record.DicomRef = TestFile001DicomRef;
                file.TestUpdateCacheFromRecord(ExecutionTime.Instance.ImageDir + TestFile001);
                file.Record.PatientName = "William^Philip^Quentin^Stott";
                file.Record.VolunteerRef = "99000002UU";
                file.Record.AppointmentDate = new DateTime(2005, 01, 21);
                file.Record.DicomRef = TestFile002DicomRef;
                file.TestUpdateCacheFromRecord(ExecutionTime.Instance.ImageDir + TestFile002);
                file.Record.PatientName = "Stott^W^^^";
                file.Record.VolunteerRef = "99000002UUU";
                file.Record.AppointmentDate = new DateTime(2005, 01, 21);
                file.Record.DicomRef = TestFile003DicomRef;
                file.TestUpdateCacheFromRecord(ExecutionTime.Instance.ImageDir + TestFile003);

                file.TestReadFromDisk(ExecutionTime.Instance.ColourSampleDicomFile);
                file.Record.PatientName = "^^^Will^Stott";
                file.Record.VolunteerRef = "99000002UUUU";
                file.Record.AppointmentDate = new DateTime(2005, 01, 22);
                file.Record.DicomRef = TestFile004DicomRef;
                file.TestUpdateCacheFromRecord(ExecutionTime.Instance.ImageDir + TestFile004);

                file.TestReadFromDisk(ExecutionTime.Instance.MonochromeSampleDicomFile);
                file.Record.PatientName = "William^StOTT";
                file.Record.VolunteerRef = "99000002UUUUU";
                file.Record.AppointmentDate = new DateTime(2005, 01, 22);
                file.Record.DicomRef = TestFile005DicomRef;
                file.TestUpdateCacheFromRecord(ExecutionTime.Instance.ImageDir + TestFile005);
                file.Record.PatientName = "Will^Stott";
                file.Record.VolunteerRef = "99000002";
                file.Record.AppointmentDate = new DateTime(2005, 01, 22);
                file.Record.DicomRef = TestFile006DicomRef;
                file.TestUpdateCacheFromRecord(ExecutionTime.Instance.ImageDir + TestFile006);
                file.Record.PatientName = "Will^Stott";
                file.Record.VolunteerRef = "99000002";
                file.Record.AppointmentDate = new DateTime(2006, 07, 01);
                file.Record.DicomRef = TestFile007DicomRef;
                file.TestUpdateCacheFromRecord(ExecutionTime.Instance.ImageDir + TestFile007);
                file.Record.PatientName = "Wxxxx^StOTT";
                file.Record.VolunteerRef = "99000002";
                file.Record.AppointmentDate = new DateTime(2006, 07, 01);
                file.Record.DicomRef = TestFile008DicomRef;
                file.TestUpdateCacheFromRecord(ExecutionTime.Instance.ImageDir + TestFile008);
                file.Record.PatientName = "Will^Stott";
                file.Record.VolunteerRef = "99000002";
                file.Record.AppointmentDate = new DateTime(2005, 11, 01);
                file.Record.DicomRef = TestFile009DicomRef;
                file.TestUpdateCacheFromRecord(ExecutionTime.Instance.ImageDir + TestFile009);
                file.Record.PatientName = "Charlie^Petzold";
                file.Record.VolunteerRef = "9811111";
                file.Record.AppointmentDate = new DateTime(2003, 02, 23);
                file.Record.DicomRef = TestFile010DicomRef;
                file.TestUpdateCacheFromRecord(ExecutionTime.Instance.ImageDir + TestFile010);
                file.Record.PatientName = "PETZOLD^C";
                file.Record.VolunteerRef = "9811111";
                file.Record.AppointmentDate = new DateTime(2003, 02, 23);
                file.Record.DicomRef = TestFile011DicomRef;
                file.TestUpdateCacheFromRecord(ExecutionTime.Instance.ImageDir + TestFile011);
                file.Record.PatientName = "Dr^C^PETZOLD";
                file.Record.VolunteerRef = "98011111";
                file.Record.AppointmentDate = new DateTime(2006, 01, 23);
                file.Record.DicomRef = TestFile012DicomRef;
                file.TestUpdateCacheFromRecord(ExecutionTime.Instance.ImageDir + TestFile012);
                file.Record.PatientName = "Herr^Oosterwijk";
                file.Record.VolunteerRef = "98555552";
                file.Record.AppointmentDate = new DateTime(2006, 01, 23);
                file.Record.DicomRef = TestFile013DicomRef;
                file.TestUpdateCacheFromRecord(ExecutionTime.Instance.ImageDir + TestFile013);
                file.Record.PatientName = "Jimmy^Denning";
                file.Record.VolunteerRef = "98000003";
                file.Record.AppointmentDate = new DateTime(2006, 01, 23);
                file.Record.DicomRef = TestFile014DicomRef;
                file.TestUpdateCacheFromRecord(ExecutionTime.Instance.ImageDir + TestFile014);
                file.Record.PatientName = "B^Stott";                         //this name doesnt exist in the database
                file.Record.VolunteerRef = "99000002";
                file.Record.AppointmentDate = new DateTime(2006, 07, 21);
                file.Record.DicomRef = TestFile015DicomRef;
                file.TestUpdateCacheFromRecord(ExecutionTime.Instance.ImageDir + TestFile015);
                file.Record.PatientName = "W^Stott";                        //name and vol ref mismatch
                file.Record.VolunteerRef = "99000001";                      //name and vol ref mismatch
                file.Record.AppointmentDate = new DateTime(2006, 07, 22);
                file.Record.DicomRef = TestFile016DicomRef;
                file.TestUpdateCacheFromRecord(ExecutionTime.Instance.ImageDir + TestFile016);
                file.Record.PatientName = "W^Stott";
                file.Record.VolunteerRef = "99000000";                      //this vol ref doesnt exist in database
                file.Record.AppointmentDate = new DateTime(2006, 07, 23);
                file.Record.DicomRef = TestFile017DicomRef;
                file.TestUpdateCacheFromRecord(ExecutionTime.Instance.ImageDir + TestFile017);
                file.Record.PatientName = "W^Stott";
                file.Record.VolunteerRef = "99000002";
                file.Record.AppointmentDate = new DateTime(2006, 07, 24);   //this date doesnt exist in database
                file.Record.DicomRef = TestFile018DicomRef;
                file.TestUpdateCacheFromRecord(ExecutionTime.Instance.ImageDir + TestFile018);

                file.TestReadFromDisk(ExecutionTime.Instance.ColourSampleDicomFile);
                file.Record.PatientName = "W^Stott";
                file.Record.VolunteerRef = "99000002";
                file.Record.AppointmentDate = new DateTime(2006, 07, 25);
                file.Record.DicomRef = TestFile000DicomRef;   //duplicate - this record should be created in the database prior to UAT
                file.TestUpdateCacheFromRecord(ExecutionTime.Instance.ImageDir + TestFile019);
                file.Record.PatientName = "W^Stott";
                file.Record.VolunteerRef = "ABC";                       //vol ref is numeric
                file.Record.AppointmentDate = new DateTime(2006, 07, 26);
                file.Record.DicomRef = TestFile020DicomRef;
                file.TestUpdateCacheFromRecord(ExecutionTime.Instance.ImageDir + TestFile020);
                file.Record.PatientName = "W^Stott";
                file.Record.VolunteerRef = "99000002";
                file.Record.AppointmentDate = new DateTime(2006, 09, 30);   //date doesn't exist in database
                file.Record.DicomRef = TestFile021DicomRef;
                file.TestUpdateCacheFromRecord(ExecutionTime.Instance.ImageDir + TestFile021);

                file.TestReadFromDisk(ExecutionTime.Instance.MonochromeSampleDicomFile);
                file.Record.PatientName = "B^Stott";                        //name not in database
                file.Record.VolunteerRef = "ABC";                           //vol ref not numeric
                file.Record.AppointmentDate = new DateTime(2006, 09, 30);   //
                file.Record.DicomRef = TestFile022DicomRef;
                file.TestUpdateCacheFromRecord(ExecutionTime.Instance.ImageDir + TestFile022);
                file.Record.PatientName = "Will^Stott";
                file.Record.VolunteerRef = "99000002";
                file.Record.AppointmentDate = new DateTime(2005, 01, 21);
                file.Record.DicomRef = TestFile023DicomRef;       //invalid file extn
                file.TestUpdateCacheFromRecord(ExecutionTime.Instance.ImageDir + TestFile023);

                File.Copy(ExecutionTime.Instance.InvalidSampleDicomFile, ExecutionTime.Instance.ImageDir + TestFile024);  //not a dicom file

                rc = true;
            }
            catch (Exception ex)
            {
                string msg = ex.Message;
                if (file != null)
                {
                    if ((file.Record != null) && (file.Record.UnspecifiedError == true))
                        msg += string.Format("[{0}]", file.Record.UnspecifiedErrorDetails);
                }
                MessageBox.Show(msg, "Error!");
            }
            return rc;
        }


        public static int CheckTestImageFiles()
        {
            int checkcnt = CheckTestImageFilesCorrect();

            checkcnt += CheckTestImageFilesIncorrect();

            return checkcnt;
        }

        private static int CheckTestImageFilesCorrect()
        {
            int checkcnt = 0;
            if (CheckTestImageFile000() == false)
                checkcnt++;
            if (CheckTestImageFile999() == false)
                checkcnt++;
            if (File.Exists(ExecutionTime.Instance.ImageDir + TestFile001) == false)
                checkcnt++;
            if (File.Exists(ExecutionTime.Instance.ImageDir + TestFile002) == false)
                checkcnt++;
            if (File.Exists(ExecutionTime.Instance.ImageDir + TestFile003) == false)
                checkcnt++;
            if (File.Exists(ExecutionTime.Instance.ImageDir + TestFile004) == false)
                checkcnt++;
            if (File.Exists(ExecutionTime.Instance.ImageDir + TestFile005) == false)
                checkcnt++;
            if (File.Exists(ExecutionTime.Instance.ImageDir + TestFile006) == false)
                checkcnt++;
            if (File.Exists(ExecutionTime.Instance.ImageDir + TestFile007) == false)
                checkcnt++;
            if (File.Exists(ExecutionTime.Instance.ImageDir + TestFile008) == false)
                checkcnt++;
            if (File.Exists(ExecutionTime.Instance.ImageDir + TestFile009) == false)
                checkcnt++;
            if (File.Exists(ExecutionTime.Instance.ImageDir + TestFile010) == false)
                checkcnt++;
            if (File.Exists(ExecutionTime.Instance.ImageDir + TestFile011) == false)
                checkcnt++;
            return checkcnt;
        }

        private static int CheckTestImageFilesIncorrect()
        {
            int checkcnt = 0;
            if (File.Exists(ExecutionTime.Instance.ImageDir + TestFile012) == false)
                checkcnt++;
            if (File.Exists(ExecutionTime.Instance.ImageDir + TestFile013) == false)
                checkcnt++;
            if (File.Exists(ExecutionTime.Instance.ImageDir + TestFile014) == false)
                checkcnt++;
            if (File.Exists(ExecutionTime.Instance.ImageDir + TestFile015) == false)
                checkcnt++;
            if (File.Exists(ExecutionTime.Instance.ImageDir + TestFile016) == false)
                checkcnt++;
            if (File.Exists(ExecutionTime.Instance.ImageDir + TestFile017) == false)
                checkcnt++;
            if (File.Exists(ExecutionTime.Instance.ImageDir + TestFile018) == false)
                checkcnt++;
            if (File.Exists(ExecutionTime.Instance.ImageDir + TestFile019) == false)
                checkcnt++;
            if (File.Exists(ExecutionTime.Instance.ImageDir + TestFile020) == false)
                checkcnt++;
            if (File.Exists(ExecutionTime.Instance.ImageDir + TestFile021) == false)
                checkcnt++;
            if (File.Exists(ExecutionTime.Instance.ImageDir + TestFile022) == false)
                checkcnt++;
            if (File.Exists(ExecutionTime.Instance.ImageDir + TestFile023) == false)
                checkcnt++;
            if (File.Exists(ExecutionTime.Instance.ImageDir + TestFile024) == false)
                checkcnt++;
            return checkcnt;
        }

        public static void DeleteTestImageFiles()
        {
            DeleteTestImageFile000();
            DeleteTestImageFile999();
            if (File.Exists(ExecutionTime.Instance.ImageDir + TestFile001))
                File.Delete(ExecutionTime.Instance.ImageDir + TestFile001);
            if (File.Exists(ExecutionTime.Instance.ImageDir + TestFile002))
                File.Delete(ExecutionTime.Instance.ImageDir + TestFile002);
            if (File.Exists(ExecutionTime.Instance.ImageDir + TestFile003))
                File.Delete(ExecutionTime.Instance.ImageDir + TestFile003);
            if (File.Exists(ExecutionTime.Instance.ImageDir + TestFile004))
                File.Delete(ExecutionTime.Instance.ImageDir + TestFile004);
            if (File.Exists(ExecutionTime.Instance.ImageDir + TestFile005))
                File.Delete(ExecutionTime.Instance.ImageDir + TestFile005);
            if (File.Exists(ExecutionTime.Instance.ImageDir + TestFile006))
                File.Delete(ExecutionTime.Instance.ImageDir + TestFile006);
            if (File.Exists(ExecutionTime.Instance.ImageDir + TestFile007))
                File.Delete(ExecutionTime.Instance.ImageDir + TestFile007);
            if (File.Exists(ExecutionTime.Instance.ImageDir + TestFile008))
                File.Delete(ExecutionTime.Instance.ImageDir + TestFile008);
            if (File.Exists(ExecutionTime.Instance.ImageDir + TestFile009))
                File.Delete(ExecutionTime.Instance.ImageDir + TestFile009);
            if (File.Exists(ExecutionTime.Instance.ImageDir + TestFile010))
                File.Delete(ExecutionTime.Instance.ImageDir + TestFile010);
            if (File.Exists(ExecutionTime.Instance.ImageDir + TestFile011))
                File.Delete(ExecutionTime.Instance.ImageDir + TestFile011);
            if (File.Exists(ExecutionTime.Instance.ImageDir + TestFile012))
                File.Delete(ExecutionTime.Instance.ImageDir + TestFile012);
            if (File.Exists(ExecutionTime.Instance.ImageDir + TestFile013))
                File.Delete(ExecutionTime.Instance.ImageDir + TestFile013);
            if (File.Exists(ExecutionTime.Instance.ImageDir + TestFile014))
                File.Delete(ExecutionTime.Instance.ImageDir + TestFile014);
            if (File.Exists(ExecutionTime.Instance.ImageDir + TestFile015))
                File.Delete(ExecutionTime.Instance.ImageDir + TestFile015);
            if (File.Exists(ExecutionTime.Instance.ImageDir + TestFile016))
                File.Delete(ExecutionTime.Instance.ImageDir + TestFile016);
            if (File.Exists(ExecutionTime.Instance.ImageDir + TestFile017))
                File.Delete(ExecutionTime.Instance.ImageDir + TestFile017);
            if (File.Exists(ExecutionTime.Instance.ImageDir + TestFile018))
                File.Delete(ExecutionTime.Instance.ImageDir + TestFile018);
            if (File.Exists(ExecutionTime.Instance.ImageDir + TestFile019))
                File.Delete(ExecutionTime.Instance.ImageDir + TestFile019);
            if (File.Exists(ExecutionTime.Instance.ImageDir + TestFile020))
                File.Delete(ExecutionTime.Instance.ImageDir + TestFile020);
            if (File.Exists(ExecutionTime.Instance.ImageDir + TestFile021))
                File.Delete(ExecutionTime.Instance.ImageDir + TestFile021);
            if (File.Exists(ExecutionTime.Instance.ImageDir + TestFile022))
                File.Delete(ExecutionTime.Instance.ImageDir + TestFile022);
            if (File.Exists(ExecutionTime.Instance.ImageDir + TestFile023))
                File.Delete(ExecutionTime.Instance.ImageDir + TestFile023);
            if (File.Exists(ExecutionTime.Instance.ImageDir + TestFile024))
                File.Delete(ExecutionTime.Instance.ImageDir + TestFile024);

            return;
        }

        public static void DeleteMaskFragmentFiles(string MaskDirectory)
        {
            if ( Directory.Exists(MaskDirectory) && DicomMask.IsMaskDirRelative())
            {
                MaskDirectory += "\\";  //v1.0.0.18 added

                File.Delete(MaskDirectory + ImageRecord.GetShortDicomRef(TestFile999DicomRef) + CranfieldImportRev + DicomMask.StdExt);
                File.Delete(MaskDirectory + ImageRecord.GetShortDicomRef(TestFile000DicomRef) + CranfieldImportRev + DicomMask.StdExt);
                File.Delete(MaskDirectory + ImageRecord.GetShortDicomRef(TestFile001DicomRef) + CranfieldImportRev + DicomMask.StdExt);
                File.Delete(MaskDirectory + ImageRecord.GetShortDicomRef(TestFile002DicomRef) + CranfieldImportRev + DicomMask.StdExt);
                File.Delete(MaskDirectory + ImageRecord.GetShortDicomRef(TestFile003DicomRef) + CranfieldImportRev + DicomMask.StdExt);
                File.Delete(MaskDirectory + ImageRecord.GetShortDicomRef(TestFile004DicomRef) + CranfieldImportRev + DicomMask.StdExt);
                File.Delete(MaskDirectory + ImageRecord.GetShortDicomRef(TestFile005DicomRef) + CranfieldImportRev + DicomMask.StdExt);
                File.Delete(MaskDirectory + ImageRecord.GetShortDicomRef(TestFile006DicomRef) + CranfieldImportRev + DicomMask.StdExt);
                File.Delete(MaskDirectory + ImageRecord.GetShortDicomRef(TestFile007DicomRef) + CranfieldImportRev + DicomMask.StdExt);
                File.Delete(MaskDirectory + ImageRecord.GetShortDicomRef(TestFile008DicomRef) + CranfieldImportRev + DicomMask.StdExt);
                File.Delete(MaskDirectory + ImageRecord.GetShortDicomRef(TestFile009DicomRef) + CranfieldImportRev + DicomMask.StdExt);
                File.Delete(MaskDirectory + ImageRecord.GetShortDicomRef(TestFile010DicomRef) + CranfieldImportRev + DicomMask.StdExt);
                File.Delete(MaskDirectory + ImageRecord.GetShortDicomRef(TestFile011DicomRef) + CranfieldImportRev + DicomMask.StdExt);
                File.Delete(MaskDirectory + ImageRecord.GetShortDicomRef(TestFile012DicomRef) + CranfieldImportRev + DicomMask.StdExt);
                File.Delete(MaskDirectory + ImageRecord.GetShortDicomRef(TestFile013DicomRef) + CranfieldImportRev + DicomMask.StdExt);
                File.Delete(MaskDirectory + ImageRecord.GetShortDicomRef(TestFile014DicomRef) + CranfieldImportRev + DicomMask.StdExt);
                File.Delete(MaskDirectory + ImageRecord.GetShortDicomRef(TestFile015DicomRef) + CranfieldImportRev + DicomMask.StdExt);
                File.Delete(MaskDirectory + ImageRecord.GetShortDicomRef(TestFile016DicomRef) + CranfieldImportRev + DicomMask.StdExt);
                File.Delete(MaskDirectory + ImageRecord.GetShortDicomRef(TestFile017DicomRef) + CranfieldImportRev + DicomMask.StdExt);
                File.Delete(MaskDirectory + ImageRecord.GetShortDicomRef(TestFile018DicomRef) + CranfieldImportRev + DicomMask.StdExt);
                File.Delete(MaskDirectory + ImageRecord.GetShortDicomRef(TestFile019DicomRef) + ".1" + DicomMask.StdExt);
                File.Delete(MaskDirectory + ImageRecord.GetShortDicomRef(TestFile020DicomRef) + CranfieldImportRev + DicomMask.StdExt);
                File.Delete(MaskDirectory + ImageRecord.GetShortDicomRef(TestFile021DicomRef) + CranfieldImportRev + DicomMask.StdExt);
                File.Delete(MaskDirectory + ImageRecord.GetShortDicomRef(TestFile022DicomRef) + CranfieldImportRev + DicomMask.StdExt);
            }
        }

        public static string ListTestImageFiles()
        {
            string rc = Resources.StrImageFilesCreated + Utils.StrLFCR;

            FileInfo[] files = DirOps.GetFilesInDirectoryTree(ExecutionTime.Instance.ImageDir);
            StringBuilder result = new StringBuilder();
            foreach (FileInfo file in files)
                result.Append(file.FullName.Replace(ExecutionTime.Instance.ImageDir, "") + Utils.StrLFCR);
            rc += result.ToString() + Resources.StrEndofList;

            return rc;
        }

        public static bool DeleteMonochromeImageRecord(Program progErrorHandler)
        {
            OsDatabases db = new OsDatabases();
            db.Connect(TestSuiteSetup.AdminUser, TestSuiteSetup.AdminUserPassword);
            ImageRecord rec = new ImageRecord();
            rec.DicomRef = TestSuiteSetup.TestImageDicomRef;
            rec.ImageID = db.GetImageIDForDicomRef(rec, progErrorHandler);

            if (rec.ImageID.Length > 0)
                db.DeleteImageRecord(rec.ImageID);
            rec.ImageID = db.GetImageIDForDicomRef(rec, progErrorHandler);
            return (rec.ImageID.Length > 0) ? false : true;
        }

        public static bool CreateMonochromeImageRecord(Program progErrorHandler)
        {
            bool rc = false;

            OsDatabases db = new OsDatabases();
            db.Connect(TestSuiteSetup.AdminUser, TestSuiteSetup.AdminUserPassword);
            ImageRecord rec = new ImageRecord();
            rec.DicomRef = TestSuiteSetup.TestImageDicomRef;
            rec.ImageID = db.GetImageIDForDicomRef(rec, progErrorHandler);

            if (rec.ImageID.Length > 0)
                rc = true;
            else
            {
                string maskdir = DicomMask.MaskDir;
                DicomMask.MaskDir = "";

                DicomFile file = DicomFileSet.MakeRecordFromFile(ExecutionTime.Instance.MonochromeSampleDicomFile);
                if (file != null)
                {
                    file.Record.ArchiveDate = DateTime.Now;
                    file.Record.DiskLabel = "UAT";
                    file.ReadFromDisk(db, progErrorHandler);

                    if ((file.Record.UnspecifiedError == false))
                    {
                        if (db.StoreImageRecord(file, progErrorHandler) == true)
                            rc = ImageRecord.MatchDicomRefAnyRevision(file.Record.DicomRef, TestSuiteSetup.TestImageDicomRef);
                    }
                }
                DicomMask.MaskDir = maskdir;
            }
            return rc;
        }

        public static CultureInfo SetGetResourceCulture(string culture)
        {
            Resources.Culture = new System.Globalization.CultureInfo(culture);
            return Resources.Culture;
        }

        public static bool ReadResourceIcons()
        {
            bool rc = true;

            if (Resources.IconCalendar == null)
                rc = false;
            if (Resources.IconCentre == null)
                rc = false;
            if (Resources.IconPatient == null)
                rc = false;
            if (Resources.IconScan == null)
                rc = false;
            if (Resources.IconURA == null)
                rc = false;
            return rc;
        }

        public static String ReadResourceStrings()
        {
            StringBuilder result = new StringBuilder();

            result.Append(Resources.StrErrorCorruptDicomHeader);
            result.Append(Resources.StrErrorInvalidDicomRef);
            result.Append(Resources.StrErrorInvalidScanDate);
            result.Append(Resources.StrErrorInvalidVolunteerRef);
            result.Append(Resources.StrFileFound);
            result.Append(Resources.StrFileImported);
            result.Append(Resources.StrFileRead);
            result.Append(Resources.StrFilesFound);
            result.Append(Resources.StrFilesImported);
            result.Append(Resources.StrFilesRead);
            result.Append(Resources.StrFrom);
            result.Append(Resources.StrImageTreeRootNode);
            result.Append(Resources.StrImagesInTree);
            result.Append(Resources.StrImportingFile);
            result.Append(Resources.StrImportingImages);
            result.Append(Resources.StrInvalidDicomRefTooLong);
            result.Append(Resources.StrLookingForFiles);
            result.Append(Resources.StrMsgBoxUnspecError);
            result.Append(Resources.StrNoImagesInTree);
            result.Append(Resources.StrOf);
            result.Append(Resources.StrOneImageInTree);
            result.Append(Resources.StrReadingFiles);
            result.Append(Resources.StrReadingHeader);
            result.Append(Resources.StrReservedDiskLabel);
            result.Append(Resources.StrSelectedImageItem);

            return result.ToString();
        }

        public static bool CreateStdDirAndSampleFiles(string ProjectRoot, Program progErrorHandler)
        {
            if (_FilesCreated == false)
            {
                if (Directory.Exists(ExecutionTime.Instance.OutputDir) == false)
                    Directory.CreateDirectory(ExecutionTime.Instance.OutputDir);
                if (Directory.Exists(ExecutionTime.Instance.DataDir) == false)
                    Directory.CreateDirectory(ExecutionTime.Instance.DataDir);

                if (Directory.Exists(ExecutionTime.Instance.DataDirMonth0) == false)
                    Directory.CreateDirectory(ExecutionTime.Instance.DataDirMonth0);
                if (Directory.Exists(ExecutionTime.Instance.DataDirDay0) == false)
                    Directory.CreateDirectory(ExecutionTime.Instance.DataDirDay0);
                if (Directory.Exists(ExecutionTime.Instance.DataDirExam0) == false)
                    Directory.CreateDirectory(ExecutionTime.Instance.DataDirExam0);
                if (Directory.Exists(ExecutionTime.Instance.DataDirMonth1) == false)
                    Directory.CreateDirectory(ExecutionTime.Instance.DataDirMonth1);
                if (Directory.Exists(ExecutionTime.Instance.DataDirDay1) == false)
                    Directory.CreateDirectory(ExecutionTime.Instance.DataDirDay1);
                if (Directory.Exists(ExecutionTime.Instance.DataDirExam1) == false)
                    Directory.CreateDirectory(ExecutionTime.Instance.DataDirExam1);
                if (Directory.Exists(ExecutionTime.Instance.DataDirMonth2) == false)
                    Directory.CreateDirectory(ExecutionTime.Instance.DataDirMonth2);
                if (Directory.Exists(ExecutionTime.Instance.DataDirDay2) == false)
                    Directory.CreateDirectory(ExecutionTime.Instance.DataDirDay2);
                if (Directory.Exists(ExecutionTime.Instance.DataDirExam2) == false)
                    Directory.CreateDirectory(ExecutionTime.Instance.DataDirExam2);

                if (Directory.Exists(ExecutionTime.Instance.ImageDir) == false)
                    Directory.CreateDirectory(ExecutionTime.Instance.ImageDir);
                if (Directory.Exists(ExecutionTime.Instance.ScriptsDir) == false)
                    Directory.CreateDirectory(ExecutionTime.Instance.ScriptsDir);
                if (Directory.Exists(ExecutionTime.Instance.FITDir) == false)
                    Directory.CreateDirectory(ExecutionTime.Instance.FITDir);

                File.Copy(ProjectRoot + "Data\\Release\\Data\\MonochromeSample.dicom", ExecutionTime.Instance.MonochromeSampleDicomFile);
                File.Copy(ProjectRoot + "Data\\Release\\Data\\ColourSample.dicom", ExecutionTime.Instance.ColourSampleDicomFile);
                File.Copy(ProjectRoot + "Data\\Release\\Data\\InvalidSample.dicom", ExecutionTime.Instance.InvalidSampleDicomFile);
                File.Copy(ProjectRoot + "db\\" + SampleDbScript, ExecutionTime.Instance.ScriptsDir + SampleDbScript);

                File.Copy(ProjectRoot + "Data\\Release\\Data\\MonochromeSample.dicom", ExecutionTime.Instance.ImageDir1DicomFileA);
                File.Copy(ProjectRoot + "Data\\Release\\Data\\MonochromeSample.dicom", ExecutionTime.Instance.ImageDir2DicomFileA);
                File.Copy(ProjectRoot + "Data\\Release\\Data\\ColourSample.dicom", ExecutionTime.Instance.ImageDir2DicomFileB);


                TestSuiteSetup.CreateTestImageFile000();
                CreateMonochromeImageRecord(progErrorHandler);

                _FilesCreated = true;
            }
            return _FilesCreated;
        }
    }
}
