using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Text;
using System.Configuration;
using System.Data.SqlClient;
using System.Data.SqlTypes;
using System.Data;
using System.IO;

using WillStott.MxAppToolbox;


namespace osImageManagerLib
{
    public class OsDatabases
    {                               //name of connections to databases
        public static readonly string ImageConn = "ImageDb";
        public static readonly string PatientConn = "PatientDb";
                                    //list of stored procedures and their parameters
        public static readonly string DeleteImageSP = "DeleteImage";
        public static readonly string DeleteDiskRefImagesSP = "DeleteDiskRefImages";
        public static readonly string InsertImageSP = "InsertImage";
        public static readonly string UpdateImageDb = "UpdateImage";
        public static readonly string GetImageSP = "GetImage";
        public static readonly string GetImageDetailsDb = "GetImageDetails";
        public static readonly string GetImageIDFromDicomRefSP = "GetImageIDFromDicomRef";
        public static readonly string GetAllImageDetailsForDiskRefSP = "GetAllImageDetailsForDiskRef";
        public static readonly string GetAllImageDetailsForVolunteerSP = "GetAllImageDetailsForVolunteer";

        public static readonly string ListVolunteerRefForNamesSP = "ListVolunteerRefForNames";
        public static readonly string ListNameForVolunteerRefSP = "ListNameForVolunteerRef";
        public static readonly string ListScanDatesForVolunteerRefSP = "ListScanDatesForVolunteerRef";
        public static readonly string GetNameFromVolunteerRefSP = "GetNameFromVolunteerRef";
        public static readonly string MatchScanDateForVolunteerRefSP = "MatchScanDateForVolunteerRef";

        private string _imageDbName;
        private string _patientDbName;
        private string _imageDbSource;
        private string _patientDbSource;
        private string _imageDbConnection;
        private string _patientDbConnection;
        private bool   _saStatus;
        private bool   _URAAdminStatus;
        private bool _GuestStatus;

        private bool    _logonState;

        public OsDatabases()
        {
            Disconnect();
        }
           
        public void Disconnect()
        {
            _imageDbName = "";
            _patientDbName = "";
            _imageDbSource = "";
            _patientDbSource = "";
            _imageDbConnection = "";
            _patientDbConnection = "";
            _URAAdminStatus = false;
            _saStatus = false;
            _GuestStatus = false;
            _logonState = false;
        }

        public bool Connect(string userName, string password)
        {
            bool rc = false;

            InitializeFromConfigurationFile(userName, password);

            string connStr = "not set";             //used in Exception logging - indicates the connection causing the problem
            try
            {
                using (SqlConnection conImage = new SqlConnection())
                {
                    connStr = _imageDbConnection;
                    conImage.ConnectionString = connStr;
                    conImage.Open();

                    if ((_patientDbConnection.Length == 0) && (_GuestStatus == true))
                    {
                        _logonState = true;
                        rc = true;
                    }
                    else
                    {
                        using (SqlConnection conPatient = new SqlConnection())
                        {
                            connStr = _patientDbConnection;
                            conPatient.ConnectionString = connStr;
                            conPatient.Open();
                            _logonState = true;
                            rc = true;
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                throw new UniException("OSA-002", UniException.Reset.AbortOp, UniException.Category.Resource, ex, "logon failed: {0}", connStr);
            }
            return rc;
        }

        public string RunScript(string databaseName, string CmdType, string script, string connName)
        {
            string rc = "";

            if ((IsUserLoggedOn() == false) || (databaseName == null) || (CmdType == null) || (script == null) || (connName == null))
                throw new UniException("OSA-012", UniException.Reset.AbortOp, UniException.Category.System, "a parameter is null or User not logged-on running script: {0}-{1}:[{2}]", (databaseName != null) ? databaseName : "(null)", (CmdType != null) ? CmdType : "(null)", (script != null) ? script : "(null)");

            try
            {
                int start = 0;
                int end = 0;
                string commandSeparator = "GO";

                StringBuilder result = new StringBuilder(rc);
                while ((end = script.IndexOf(commandSeparator, start)) != -1)
                {
                    string subscript = script.Substring(start, end - start);

                    if (IsSaLoggedOn())
                        result.Append(ProcessSAScript(databaseName, subscript, CmdType, connName));
                    else
                        result.Append(ProcessScript(databaseName, subscript, CmdType));

                    start = end + commandSeparator.Length;
                    if (start > script.Length)
                        break;
                }
                rc = result.ToString();
                if ( start == 0 )
                    throw new InvalidOperationException("script did not contain GO");
            }
            catch (Exception ex)
            {
                throw new UniException("OSA-006", UniException.Reset.AbortOp, UniException.Category.System, ex, "unable to run script: {0},{1}:{2} [{3}]", databaseName, ex.Message, CmdType, (script.Length < 40) ? script : script.Substring(0, 40));
            }
            return rc;
        }


        private string ProcessScript(string databaseName, string subscript, string CmdType)
        {
            string rc = "";
            using (SqlConnection con = new SqlConnection())
            {
                if (databaseName == _imageDbName)
                {
                    con.ConnectionString = _imageDbConnection;
                    rc = SqlUtils.ExecuteScript(con, CmdType, subscript) + Utils.StrLFCR;
                }
                else if (databaseName == _patientDbName)
                {
                    con.ConnectionString = _patientDbConnection;
                    rc = SqlUtils.ExecuteScript(con, CmdType, subscript) + Utils.StrLFCR;
                }
                else
                    throw new InvalidOperationException("database not found in configuration file");
            }
            return rc;
        }

        private string ProcessSAScript(string databaseName, string subscript, string CmdType, string connName)
        {
            string rc = "";

            using (SqlConnection con = new SqlConnection())
            {
                if (connName == OsDatabases.ImageConn )
                {
                    con.ConnectionString = _imageDbConnection.Replace("Database=master", "Database=" + databaseName);
                    rc += SqlUtils.ExecuteScript(con, CmdType, subscript) + Utils.StrLFCR;
                }
                else if (connName == OsDatabases.PatientConn)
                {
                    con.ConnectionString = _patientDbConnection.Replace("Database=master", "Database=" + databaseName);
                    rc += SqlUtils.ExecuteScript(con, CmdType, subscript) + Utils.StrLFCR;
                }
                else
                    throw new InvalidOperationException("database not found in configuration file");
            }
            return rc;
        }

        public void DeleteImageRecord(string ImageID)
        {
            if (IsUserLoggedOn() == false)
                throw new UniException("OSA-059", UniException.Reset.AbortOp, UniException.Category.System, "User not logged-on running stored procedure");

            try
            {
                using (SqlConnection con = new SqlConnection())
                {
                    SqlCommand cmd = con.CreateCommand();

                    cmd.CommandType = CommandType.StoredProcedure;
                    cmd.CommandText = DeleteImageSP;
                    SqlUtils.CreateUniqueIDParam(cmd, "@ImageID", false, new Guid(ImageID));

                    con.ConnectionString = _imageDbConnection;
                    con.Open();

                    int result = cmd.ExecuteNonQuery();
                }
            }
            catch (Exception ex)
            {
                throw new UniException("OSA-060", UniException.Reset.AbortOp, UniException.Category.Data, ex, "error running stored procedure to delete record from image table");
            }
            return;
        }

       public void DeleteDiskLabelImages(string DiskLabel)
       {
            if (IsUserLoggedOn() == false)
                throw new UniException("OSA-065", UniException.Reset.AbortOp, UniException.Category.System, "User not logged-on running stored procedure");

            try
            {
                using (SqlConnection con = new SqlConnection())
                {
                    SqlCommand cmd = con.CreateCommand();

                    cmd.CommandType = CommandType.StoredProcedure;
                    cmd.CommandText = DeleteDiskRefImagesSP;
                    SqlUtils.CreateVarCharParam(cmd, "@DiskRef", ImageRecord.MaxDiskLabelLength, false, DiskLabel);

                    con.ConnectionString = _imageDbConnection;
                    con.Open();

                    int result = cmd.ExecuteNonQuery();
                }
            }
            catch (Exception ex)
            {
                throw new UniException("OSA-066", UniException.Reset.AbortOp, UniException.Category.Data, ex, "error running stored procedure to delete records for disk ref {0} from image table", DiskLabel);
            }
            return;
        }


        public string GetUKCTOCSQueryResults(string storedProcName, string Param1, string Param2)
        {
            string rc = "[nothing found]";

            if (IsUserLoggedOn() == false)
                throw new UniException("OSA-039", UniException.Reset.AbortOp, UniException.Category.System, "User not logged-on");
            try
            {
                using (SqlConnection con = new SqlConnection())
                {
                    SqlCommand cmd = null;
                    if ((storedProcName == OsDatabases.ListNameForVolunteerRefSP) || (storedProcName == OsDatabases.ListScanDatesForVolunteerRefSP))
                        cmd = OsDatabases.CreateInputParamVolunteerRef(con, int.Parse(Param1), storedProcName);
                    if (storedProcName == OsDatabases.ListVolunteerRefForNamesSP)
                        cmd = OsDatabases.CreateInputParamNames(con, Param1, Param2, OsDatabases.ListVolunteerRefForNamesSP);

                    con.ConnectionString = _patientDbConnection;
                    con.Open();
                    rc = SqlUtils.GetResultsFromCommandExecution(cmd);
                }
            }
            catch (Exception ex)
            {
                rc = ex.Message;
            }
            return rc;
        }

        public bool DoesScanDateExistForVolunteerRef(ImageRecord record, Program progErrorHandler)
        {
            bool rc = false;

            if ((IsUserLoggedOn() == false) || (progErrorHandler == null))
                throw new UniException("OSA-016", UniException.Reset.AbortOp, UniException.Category.System, "progErrorHandler is null or User not logged-on running stored procedure");

            if (record == null)
                progErrorHandler.Log("Error: osDatabases.DoesScanDateExistForVolunteerRef() record is null");
            else
            {
                try
                {
                    using (SqlConnection con = new SqlConnection())
                    {
                        SqlCommand cmd = con.CreateCommand();

                        cmd.CommandType = CommandType.StoredProcedure;
                        cmd.CommandText = OsDatabases.MatchScanDateForVolunteerRefSP;
                        SqlUtils.CreateIntParam(cmd, "@VolunteerRef", false, int.Parse(record.VolunteerRef));
                        SqlUtils.CreateDateTimeParam(cmd, "@BeginDate", false, record.AppointmentDate);
                        SqlUtils.CreateDateTimeParam(cmd, "@EndDate", false, record.AppointmentDate.Add(new TimeSpan(1, 0, 0, 0)));
                        SqlParameter paramOut = SqlUtils.CreateIntParam(cmd, "@result", true, -1);

                        con.ConnectionString = _patientDbConnection;
                        con.Open();

                        SqlDataReader results = null;
                        if ((results = cmd.ExecuteReader()) != null)
                        {
                            if (paramOut.Value.ToString().Length > 0)
                                rc = true;
                        }
                    }
                }
                catch (Exception ex)
                {
                    progErrorHandler.Log("Error: osDatabases.DoesScanDateExistForVolunteerRef() - exception, unable to run stored procedure {0}: {1}", OsDatabases.MatchScanDateForVolunteerRefSP, ex.Message);
                    record.UnspecifiedErrorText = Resources.StrErrorInvalidScanDate; //VolunteerRef has previously been validated, so error must be ScanDate param
                }
            }
            return rc;
        }

        public string GetImageIDForDicomRef(ImageRecord record, Program progErrorHandler)
        {
            string rc = "(null)";

            if ( (IsUserLoggedOn() == false ) || (progErrorHandler == null))
                throw new UniException("OSA-018", UniException.Reset.AbortOp, UniException.Category.System, "progErrorHandler is null or User not logged-on running stored procedure");

            if (record == null)
                progErrorHandler.Log("Error: osDatabases.GetImageIDForDicomRef() record is null");
            else
            {
                try
                {
                    using (SqlConnection con = new SqlConnection())
                    {
                        SqlCommand cmd = con.CreateCommand();

                        cmd.CommandType = CommandType.StoredProcedure;
                        cmd.CommandText = OsDatabases.GetImageIDFromDicomRefSP;     //although it uses LIKE for revision number, there should only ever be maximum one match as database holds only latest revision
                        SqlUtils.CreateVarCharParam(cmd, "@DicomRef", ImageRecord.MaxDicomRefLength, false, record.DicomRefMatchStr);
                        SqlParameter paramOut = SqlUtils.CreateUniqueIDParam(cmd, "@result", true, System.Guid.Empty);

                        con.ConnectionString = _imageDbConnection;
                        con.Open();

                        SqlDataReader results = null;
                        if ((results = cmd.ExecuteReader()) != null)
                            record.ImageID = paramOut.Value.ToString();
                        rc = record.ImageID;
                    }
                }
                catch (Exception ex)
                {
                    progErrorHandler.Log("Error: osDatabases.GetImageIDForDicomRef - exception, unable to run stored procedure {0}: {1}", OsDatabases.GetImageIDFromDicomRefSP, ex.Message);
                    record.UnspecifiedErrorText = Resources.StrMsgBoxInvalidDicomRef;
                }
            }
            return rc;
        }

        public string GetNameFromVolunteerRef(ImageRecord record, Program progErrorHandler)
        {
            string name = "";

            if ((IsUserLoggedOn() == false) || (progErrorHandler == null))
                throw new UniException("OSA-014", UniException.Reset.AbortOp, UniException.Category.System, "progErrorHandler is null or User not logged-on running stored procedure");

            if (record == null)
                progErrorHandler.Log("Error: osDatabases.GetNameFromVolunteerRef() record is null");
            else
            {
                try
                {
                    using (SqlConnection con = new SqlConnection())
                    {
                        SqlCommand cmd = con.CreateCommand();

                        cmd.CommandType = CommandType.StoredProcedure;
                        cmd.CommandText = OsDatabases.GetNameFromVolunteerRefSP;
                        SqlUtils.CreateIntParam(cmd, "@VolunteerRef", false, int.Parse(record.VolunteerRef));
                        SqlParameter paramOutSurname = SqlUtils.CreateVarCharParam(cmd, "@surname", ImageRecord.MaxSurnameLength, true, null);
                        SqlParameter paramOutFirstnames = SqlUtils.CreateVarCharParam(cmd, "@firstnames", ImageRecord.MaxFirstNamesLength, true, null);

                        con.ConnectionString = _patientDbConnection;
                        con.Open();

                        SqlDataReader results = null;
                        if ((results = cmd.ExecuteReader()) != null)
                        {
                            if (paramOutSurname.Value.ToString().Length > 0)
                            {
                                name = paramOutSurname.Value.ToString();
                                if (paramOutFirstnames.Value.ToString().Length > 0)
                                    name += "^" + paramOutFirstnames.Value.ToString().Replace(' ', '^');
                            }
                        }
                    }
                }
                catch (Exception ex)
                {
                    progErrorHandler.Log("Error: osDatabases.GetNameFromVolunteerRef() - exception: unable to run stored procedure {0}: {1}", OsDatabases.GetNameFromVolunteerRefSP, ex.Message);
                    record.UnspecifiedErrorText = Resources.StrErrorInvalidVolunteerRef;
                }
            }
            return name;
        }

        public bool StoreImageRecord(DicomFile file, Program progErrorHandler)
        {
            bool rc = false;

            if ((IsUserLoggedOn() == false) || (progErrorHandler == null))
                throw new UniException("OSA-021", UniException.Reset.AbortOp, UniException.Category.System, "progErrorHandler is null or User not logged-on saving: {0}", ((file == null) || (file.Record == null)) ? "[file or file.record is null]" : file.Record.DicomPathFilename);

            if ((file == null) || (file.Record == null))
                progErrorHandler.Log("Error: osDatabases.StoreImageRecord() file is null");
            else
            {
                file.Record.ResetUnspecifiedError();
                file.UpdateAndMaskCache(progErrorHandler);

                if ((file.Record.GetDicomFileCache() == null) || (file.Record.HeaderFieldsInitiated() == false) || (file.Record.UnspecifiedError == true))
                    progErrorHandler.Log("Error: osDatabases.StoreImageRecord() image is null or record is not properly initiated or error writing mask fragment");
                else
                {
                    try
                    {
                        using (SqlConnection con = new SqlConnection())
                        {
                            file.Record.ArchiveOperator = SqlUtils.getConnectionElement(_imageDbConnection, "User ID=");

                            SqlCommand cmd = con.CreateCommand();
                            cmd.CommandType = CommandType.StoredProcedure;
                            Guid guid = Guid.NewGuid();
                            if (file.Record.Duplicate == false)
                            {
                                cmd.CommandText = OsDatabases.InsertImageSP;
                                file.Record.ImageID = guid.ToString();

                            }
                            else
                            {
                                cmd.CommandText = OsDatabases.UpdateImageDb;
                                guid = new Guid(file.Record.ImageID);

                            }
                            SqlUtils.CreateUniqueIDParam(cmd, "@ImageID", false, guid);
                            SqlUtils.CreateDateTimeParam(cmd, "@CaptureDate", false, file.Record.AppointmentDate);
                            SqlUtils.CreateDateTimeParam(cmd, "@StoredDate", false, file.Record.ArchiveDate);
                            SqlUtils.CreateVarCharParam(cmd, "@StoredBy", ImageRecord.MaxUserNameLength, false, file.Record.ArchiveOperator);
                            SqlUtils.CreateVarCharParam(cmd, "@DiskRef", ImageRecord.MaxDiskLabelLength, false, file.Record.DiskLabel);
                            SqlUtils.CreateIntParam(cmd, "@VolunteerRef", false, int.Parse(file.Record.VolunteerRef));
                            SqlUtils.CreateVarCharParam(cmd, "@DicomRef", ImageRecord.MaxDicomRefLength, false, file.Record.DicomRef);
                            SqlUtils.CreateImageParam(cmd, "@DicomFile", file.Record.DicomFileSize, false, file.Record.GetDicomFileCache());
                            SqlUtils.CreateIntParam(cmd, "@DicomFileSize", false, file.Record.DicomFileSize);
                            SqlUtils.CreateVarCharParam(cmd, "@MaskFile", ImageRecord.MaxMaskPathFileLength, false, file.Record.MaskFileLabel);
                            SqlUtils.CreateVarBinaryParam(cmd, "@Flags", ImageRecord.FlagsLength, false, file.Record.GetFlags());

                            con.ConnectionString = _imageDbConnection;
                            con.Open();

                            SqlDataReader results = null;
                            if ((results = cmd.ExecuteReader()) == null)
                                progErrorHandler.Log("Error: osDatabases.StoreImageRecord() database error running {0} saving: {1}", cmd.CommandText, file.Record.DicomPathFilename);
                            else
                                rc = true;
                        }
                    }
                    catch (Exception ex)
                    {
                        progErrorHandler.Log("Error: osDatabases.StoreImageRecord() exception unable to {0}: {1} - {2}", (file.Record.Duplicate) ? "update" : "insert", file.Record.DicomPathFilename, ex.Message);
                    }
                }
            }
            return rc;
        }

        public void LoadImageFileDataForImageID(ImageRecord rec)
        {
            if ((IsUserLoggedOn() == false) || (rec == null))
                throw new UniException("OSA-026", UniException.Reset.AbortOp, UniException.Category.System, "Record null or User not logged-on restoring: {0}", (rec != null) ? rec.ImageID : "(null rec)" );
            rec.ClearDicomFileCache();

            try
            {
                using (SqlConnection con = new SqlConnection())
                {
                    string expectedDicomRef = rec.DicomRef;

                    SqlCommand cmd = con.CreateCommand();

                    cmd.CommandType = CommandType.StoredProcedure;
                    cmd.CommandText = OsDatabases.GetImageSP;
                    SqlUtils.CreateUniqueIDParam(cmd, "@ImageID", false, new Guid(rec.ImageID));

                    con.ConnectionString = _imageDbConnection;
                    con.Open();

                    SqlDataReader results = null;
                    if ((results = cmd.ExecuteReader()) != null)
                    {
                        if (results.Read())
                        {
                            rec.DicomRef = results.GetString(0);
                            SqlBinary image = (SqlBinary)results.GetSqlValue(1);
                            rec.SetDicomFileCache(image.Value);

                            if (expectedDicomRef != rec.DicomRef)
                                rec.ClearDicomFileCache();
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                throw new UniException("OSA-037", UniException.Reset.AbortOp, UniException.Category.System, ex, "Error running {0} to get image {1} from database", OsDatabases.GetImageSP, (rec == null) ? "[record is null]" : rec.ImageID);
            }
            if (rec.IsDicomFileCached() == false)
                throw new UniException("OSA-049", UniException.Reset.AbortOp, UniException.Category.System, "Error running {0} to get image {1} from database", OsDatabases.GetImageSP, (rec == null) ? "[record is null]" : rec.ImageID);
            return;
        }

        public void LoadImageDetailsForImageID(ImageRecord record)
        {
            bool rc = false;

            if ((IsUserLoggedOn() == false) || (record == null))
                throw new UniException("OSA-021", UniException.Reset.AbortOp, UniException.Category.System, "saving: {0}", (record == null) ? "[record is null]" : record.DicomPathFilename);

            string errorMsg = "[unknown error]";
            string ImageID = record.ImageID;
            record.Clear();

            try
            {
                using (SqlConnection con = new SqlConnection())
                {
                    SqlCommand cmd = con.CreateCommand();
                    cmd.CommandType = CommandType.StoredProcedure;
                    cmd.CommandText = OsDatabases.GetImageDetailsDb;
                    SqlUtils.CreateUniqueIDParam(cmd, "@ImageID", false, new Guid(ImageID));

                    con.ConnectionString = _imageDbConnection;
                    con.Open();

                    SqlDataReader results = null;
                    if ((results = cmd.ExecuteReader()) == null)
                        errorMsg = "ExecuteReader failed";
                    else
                    {
                        if (results.Read() == false)
                            errorMsg = "Record is empty";
                        else
                        {
                            if (OsDatabases.LoadResultsInRecord(results, record) == null)
                                errorMsg = "Unable to load results into Record";
                            else
                                rc = true;
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                errorMsg = ex.Message;
            }
            if (rc == false)
                throw new UniException("OSA-047", UniException.Reset.AbortOp, UniException.Category.Data, "Error running {0} when getting {1}: {2}", OsDatabases.GetImageDetailsDb, record.ImageID, errorMsg);

            return;
        }

        public ImageRecord[] GetAllImageDetailsForVolunteerRef(int VolunteerRef)
        {
            return GetImageDetails(VolunteerRef, null);
        }

        public ImageRecord[] GetAllImageDetailsForDiskRef(string DiskRef)
        {
            return GetImageDetails(-1, DiskRef);
        }
        
        private ImageRecord[] GetImageDetails(int VolunteerRef, string DiskRef)
        {
            if (IsUserLoggedOn() == false)
                throw new UniException("OSA-044", UniException.Reset.AbortOp, UniException.Category.Program, "User not logged-on running {0} stored procedure", OsDatabases.GetAllImageDetailsForVolunteerSP);

            List<ImageRecord> list = new List<ImageRecord>();
            try
            {
                using (SqlConnection con = new SqlConnection())
                {
                    SqlCommand cmd = null;
                    if (VolunteerRef != -1 )
                        cmd = OsDatabases.CreateInputParamVolunteerRef(con, VolunteerRef, OsDatabases.GetAllImageDetailsForVolunteerSP);
                    if (DiskRef != null)
                        cmd = OsDatabases.CreateInputParamDiskRef(con, DiskRef, OsDatabases.GetAllImageDetailsForDiskRefSP);
                    if (cmd == null)
                        throw new UniException("OSA-043", UniException.Reset.AbortOp, UniException.Category.Data, "unable to create command for stored proc: VolunteerRef={0}, DiskRef={1}", VolunteerRef, (DiskRef == null) ? "[null]" : DiskRef);

                    con.ConnectionString = _imageDbConnection;
                    con.Open();

                    SqlDataReader results = cmd.ExecuteReader();
                    while (results.Read())
                    {
                        list.Add(OsDatabases.LoadResultsInRecord(results, new ImageRecord()));
                    }
                }
            }
            catch (Exception ex)
            {
                throw new UniException("OSA-045", UniException.Reset.AbortOp, UniException.Category.Data, ex, "Error running {0}: {1}", OsDatabases.GetAllImageDetailsForDiskRefSP, ex.Message);
            }
            ImageRecord[] arr = new ImageRecord[list.Count];
            list.CopyTo(arr);
            return arr;
        }

        private static ImageRecord LoadResultsInRecord(SqlDataReader results, ImageRecord rec)
        {
            rec.ImageID = results.GetGuid(0).ToString().ToUpper();
            rec.AppointmentDate = results.GetDateTime(1);
            rec.ArchiveDate = results.GetDateTime(2);
            rec.ArchiveOperator = results.GetString(3);
            rec.DiskLabel = results.GetString(4);
            rec.VolunteerRef = results.GetInt32(5).ToString();
            rec.DicomRef = results.GetString(6);
            rec.DicomFileSize = results.GetInt32(7);
            rec.MaskFileLabel = results.GetString(8);
            byte[] flags = new byte[ImageRecord.FlagsLength];
            results.GetBytes(9, 0, flags, 0, ImageRecord.FlagsLength);
            rec.SetFlags(flags);

            return rec;
        }

        private static SqlCommand CreateInputParamVolunteerRef(SqlConnection con, int VolunteerRef, string StoredProcName)
        {
            SqlCommand cmd = con.CreateCommand();
            cmd.CommandType = CommandType.StoredProcedure;
            cmd.CommandText = StoredProcName;
            SqlUtils.CreateIntParam(cmd, "@VolunteerRef", false, VolunteerRef);

            return cmd;
        }

        private static SqlCommand CreateInputParamDiskRef(SqlConnection con, string DiskRef, string StoredProcName)
        {
            SqlCommand cmd = con.CreateCommand();
            cmd.CommandType = CommandType.StoredProcedure;
            cmd.CommandText = StoredProcName;
            SqlUtils.CreateVarCharParam(cmd, "@DiskRef", ImageRecord.MaxDiskLabelLength, false, DiskRef);

            return cmd;
        }

        private static SqlCommand CreateInputParamNames(SqlConnection con, string Surname, string FirstNames, string StoredProcName)
        {
            SqlCommand cmd = con.CreateCommand();
            cmd.CommandType = CommandType.StoredProcedure;
            cmd.CommandText = StoredProcName;
            SqlUtils.CreateVarCharParam(cmd, "@surname", ImageRecord.MaxSurnameLength, false, Surname);
            SqlUtils.CreateVarCharParam(cmd, "@firstnames", ImageRecord.MaxFirstNamesLength, false, FirstNames);

            return cmd;
        }

        private void InitializeFromConfigurationFile(string userName, string password)
        {
            bool done = false;
            try
            {
                AppSettingsSection appSettings = ExecutionTime.Configuration.AppSettings as AppSettingsSection;
                if (appSettings != null)
                {
                    KeyValueConfigurationCollection settings = appSettings.Settings;
                    string[] names = appSettings.Settings.AllKeys;
                    foreach (string name in names)
                    {
                        if (name == "sa")
                        {
                            if (userName == settings[name].Value)
                                 _saStatus = true;
                        }
                        if (name == "URA-Admin")
                        {
                            if (userName == settings[name].Value)
                                _URAAdminStatus = true;
                        }
                        if (name == "Guest")
                        {
                            if (userName == settings[name].Value)
                                _GuestStatus = true;
                        }
                    }
                }

                ConnectionStringsSection csSection = ExecutionTime.Configuration.ConnectionStrings;

                if (csSection.ConnectionStrings.Count > 1)
                {
                    ConnectionStringSettings csSettings = csSection.ConnectionStrings[1];
                    if (csSettings.Name == OsDatabases.ImageConn)
                    {
                        _imageDbConnection = csSettings.ConnectionString.Replace("User ID=JohnDoe", "User ID=" + userName);
                        _imageDbConnection = _imageDbConnection.Replace("Password=password", "Password=" + password);
                        if (_saStatus == false)
                            _imageDbName = SqlUtils.getConnectionElement(_imageDbConnection, "Database=");
                        else
                        {
                            _imageDbName = SqlUtils.getConnectionElement(_imageDbConnection, "Database=");
                            _imageDbConnection = _imageDbConnection.Replace("Database=" + _imageDbName, "Database=master");
                        }
                        _imageDbSource = SqlUtils.getConnectionElement(_imageDbConnection, "Data Source=");

                        if ((_GuestStatus == true) && (_imageDbConnection.Length > 0))
                            done = true;
                        else
                        {
                            if (csSection.ConnectionStrings.Count > 2)
                            {
                                csSettings = csSection.ConnectionStrings[2];
                                if (csSettings.Name == OsDatabases.PatientConn)
                                {
                                    _patientDbConnection = csSettings.ConnectionString.Replace("User ID=JohnDoe", "User ID=" + userName);
                                    _patientDbConnection = _patientDbConnection.Replace("Password=password", "Password=" + password);
                                    if (_saStatus == false)
                                        _patientDbName = SqlUtils.getConnectionElement(_patientDbConnection, "Database=");
                                    else
                                    {
                                        _patientDbName = SqlUtils.getConnectionElement(_patientDbConnection, "Database=");
                                        _patientDbConnection = _patientDbConnection.Replace("Database=" + _patientDbName, "Database=master");
                                    }
                                    _patientDbSource = SqlUtils.getConnectionElement(_patientDbConnection, "Data Source=");

                                    done = true;
                                }
                            }
                        }
                    }
                }
                if (done == false)
                    throw new InvalidProgramException("configuration file doesn't contain required database settings");

            }
            catch (Exception ex)
            {
                throw new UniException("OSA-001", UniException.Reset.AbortOp, UniException.Category.Resource, ex, "config file is bad: PatientDbConnection length={0} ImageDbConnection length = {1}", _patientDbConnection.Length, _imageDbConnection.Length);
            }
            return;
        }

        public bool IsUserLoggedOn()
        {
            return _logonState;
        }
        public bool IsSaLoggedOn()
        {
            return _saStatus;
        }
        public bool IsURAAdminLoggedOn()
        {
            return _URAAdminStatus;
        }
        public bool IsGuestLoggedOn()
        {
            return _GuestStatus;
        }
        public string ImageDbName
        {
            get { return _imageDbName; }
        }
        public string PatientDbName
        {
            get { return _patientDbName; }
        }
        public string ImageDbConnection
        {
            get { return _imageDbConnection; }
        }
        public string PatientDbConnection
        {
            get { return _patientDbConnection; }
        }
        public string PatientDbSource
        {
            get { return _patientDbSource; }
        }
        public string ImageDbSource
        {
            get { return _imageDbSource; }
        }
    }
}

