using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;

using fit;
using osImageManagerLib;
using WillStott.MxAppToolbox;

namespace osImageManagerLibFIT
{
    public class FileImportDialogFile : ActionFixture
    {
        private string _selectFile;
        private string _correctPatientName;
        private string _correctPatientID;
        private string _correctAppointment;

        private string _result;
        private DlgProcFileImport _dlgproc;
        private ListBox _imageListBox;
        private ListBox _correctionsListBox;
        private ProgressBar _workProgressBar;
        private Label _progressTextLabel;

        public FileImportDialogFile()
        {
            _imageListBox = new ListBox();
            _correctionsListBox = new ListBox();
            _workProgressBar = new ProgressBar();
            _progressTextLabel = new Label();
            _dlgproc = DlgProcFileImport.CreateDlgProc(DlgProcFileImport.ImportMode.IndividualFile, MockProgram.Instance.GetDatabases(), _imageListBox, _correctionsListBox, _workProgressBar, _progressTextLabel, 0, MockProgram.Instance);
            DicomMask.InitializeMaskSettings();
            DicomMask.ComputerName = "UAT";
        }

        public void SelectFile(string filename) { _selectFile = filename; }
        public void CorrectPatientName(string name) { _correctPatientName = name; }
        public void CorrectPatientID(string id) { _correctPatientID = id; }
        public void CorrectAppointment(string appointment) { _correctAppointment = appointment; }

        public int ImageListCount()
        {
            return _dlgproc.ImageListCnt;
        }

        public int CorrectionListCount()
        {
            return _dlgproc.CorrectionsListCnt;
        }

        public void ReadFileFromDataDirectory(string pathfilename)
        {
            int cnt = _dlgproc.ReadFiles(osImageManagerLib.ExecutionTime.Instance.DataDir + pathfilename, "");
            _result = string.Format("{0} files read", cnt);
        }

        public void Import()
        {
            TestSuiteSetup.DeleteMaskFragmentFiles(DicomMask.MaskPath);
            int cnt = _dlgproc.ImportFiles("UAT", new DateTime(2006, 8, 21));
            _result = string.Format("{0} files imported", cnt);
        }

        public string Progress()
        {
            return _result;
        }

        public void Correct()
        {
            CorrectFile(DialogResult.OK, false);
        }

        public void CorrectDuplicate()
        {
            CorrectFile(DialogResult.None, true);
        }

        private void CorrectFile(DialogResult result, bool duplicate)
        {
            DicomFile file = GetDicomFileFromCorrectionsList(_selectFile);
            if (file != null)
            {
                DlgProcDICOMCorrection _dlgproc = new DlgProcDICOMCorrection(file, MockProgram.Instance.GetDatabases(), MockProgram.Instance);

                string patientName = (_correctPatientName != "-") ? _correctPatientName : _dlgproc.PatientNameLabel;
                string volRef = (_correctPatientID != "-") ? _correctPatientID : _dlgproc.VolunteerRefLabel;
                DateTime appointmentDate = (_correctAppointment != "-") ? DateTime.Parse(_correctAppointment) : _dlgproc.AppointmentDateValue;

                if (_dlgproc.DoOK(patientName, volRef, appointmentDate) == result)
                {
                    if ((duplicate && _dlgproc.Duplicate) || (duplicate == false))
                        _imageListBox.Items.Add(file);
                }
            }
        }

        public DicomFile GetDicomFileFromCorrectionsList(string filename)
        {
            DicomFile rc = null;
            foreach (DicomFile file in _correctionsListBox.Items)
            {
                if (file.Record.DicomPathFilename.Contains(filename))
                {
                    rc = file;
                    break;
                }
            }
            return rc;
        }
    }
}
