using System;
using System.Text;
using System.Collections.Generic;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using WillStott.MxAppToolbox;
using osImageManagerLib;
using osImageManagerLibFIT;


namespace osImageManagerLibUnitTests
{
    /// <summary>
    /// Summary description for ImageFileCollectionTest
    /// </summary>
    [TestClass]
    public class DicomFileSetTests
    {
        public DicomFileSetTests()
        {
        }

        [ClassInitialize()]
        public static void DicomFileCollectionTestsInitialize(TestContext testContext)
        {
            TestSuiteSetup.CreateStdDirAndSampleFiles(TestSuiteSetup.ProjectRootFromUnitTestRuntime, MockProgram.Instance);
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class

        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        [TestMethod]
        public void MakeRecordFromFile()
        {
            DicomFile file = DicomFileSet.MakeRecordFromFile(ExecutionTime.Instance.MonochromeSampleDicomFile);
            Assert.IsNotNull(file);
            Assert.AreEqual(ExecutionTime.Instance.MonochromeSampleDicomFile, file.Record.DicomPathFilename);
        }
        [TestMethod]
        [ExpectedException(typeof(UniException))]
        public void MakeRecordFromFileNull()
        {
            DicomFileSet.MakeRecordFromFile(null);
        }

        [TestMethod]
        [ExpectedException(typeof(UniException))]
        public void MakeRecordFromFileEmpty()
        {
            DicomFileSet.MakeRecordFromFile("");
        }

        [TestMethod]
        [ExpectedException(typeof(UniException))]
        public void MakeRecordFromFileDir()
        {
            DicomFileSet.MakeRecordFromFile(ExecutionTime.Instance.DataDir);
        }

        [TestMethod]
        [ExpectedException(typeof(UniException))]
        public void MakeRecordFromFileNonExist()
        {
            DicomFileSet.MakeRecordFromFile(ExecutionTime.Instance.DataDir + "123.txt");
        }

        [TestMethod]
        public void MakeRecordFromDirectory()
        {
            DicomFileSet disk = new DicomFileSet();
            DicomFile[] files = disk.MakeRecordsFromDirectory(ExecutionTime.Instance.DataDir);
            Assert.IsNotNull(files);
            Assert.AreEqual(7, files.Length);
            Assert.AreEqual(ExecutionTime.Instance.ImageDir2DicomFileA, files[2].Record.DicomPathFilename);
       }

        [TestMethod]
        public void MakeRecordFromDirectoryNoDicomFiles()
        {
            DicomFileSet disk = new DicomFileSet();
            DicomFile[] files = disk.MakeRecordsFromDirectory(ExecutionTime.Instance.ScriptsDir);
            Assert.IsNotNull(files);
            Assert.AreEqual(0, files.Length);
        }

        [TestMethod]
        [ExpectedException(typeof(UniException))]
        public void MakeRecordFromDirectoryEmpty()
        {
            DicomFileSet disk = new DicomFileSet();
            DicomFile[] files = disk.MakeRecordsFromDirectory("");
        }

        [TestMethod]
        [ExpectedException(typeof(UniException))]
        public void MakeRecordFromDirectoryNoneExist()
        {
            DicomFileSet disk = new DicomFileSet();
            DicomFile[] files = disk.MakeRecordsFromDirectory("x:\\");
        }

        [TestMethod]
        [ExpectedException(typeof(UniException))]
        public void MakeRecordFromDirectoryNull()
        {
            DicomFileSet disk = new DicomFileSet();
            DicomFile[] files = disk.MakeRecordsFromDirectory(null);
        }
    }
}
