using System;
using System.Text;
using System.Collections.Generic;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.IO;

using WillStott.MxAppToolbox;
using osImageManagerLib;
using osImageManagerLibFIT;

namespace osImageManagerLibUnitTests
{
    /// <summary>
    /// Summary description for ImageMaskTest
    /// </summary>
    [TestClass]
    public class DicomMaskTests
    {
        private DicomMask mask;
        private OsDatabases db;

        public DicomMaskTests()
        {
            mask = new DicomMask();
            db = new OsDatabases();
            db.Connect(TestSuiteSetup.AdminUser, TestSuiteSetup.AdminUserPassword);
        }

        [ClassInitialize()]
        public static void DicomMaskTestsInitialize(TestContext testContext)
        {
            TestSuiteSetup.CreateStdDirAndSampleFiles(TestSuiteSetup.ProjectRootFromUnitTestRuntime, MockProgram.Instance);
            if ((DicomMask.MaskDir.Length > 0) &&(Directory.Exists(DicomMask.MaskDir) == false))
                Directory.CreateDirectory(DicomMask.MaskDir);
        }

        [TestInitialize()]
        public void TestInitialize()
        {
            DicomMask.InitializeMaskSettings();
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class

        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        [TestMethod]
        public void InitializeMaskSettingsBad()
        {
            //DicomMask.InitializeMaskSettings("xxx");
        }

        [TestMethod]
        public void InitializeMaskSettings()
        {
            //Assert.AreEqual(TestSuiteSetup.DicomMaskDefaultMaskData, DicomMask.MaskData); 
            //Assert.AreEqual(TestSuiteSetup.DicomMaskDefaultLeft, DicomMask.Left);
            //Assert.AreEqual(TestSuiteSetup.DicomMaskDefaultWidth, DicomMask.Width);
            //Assert.AreEqual(TestSuiteSetup.DicomMaskDefaultStartrow, DicomMask.StartRow);
            //Assert.AreEqual(TestSuiteSetup.DicomMaskDefaultEndrow, DicomMask.EndRow);
            //Assert.AreEqual(TestSuiteSetup.DicomMaskDefaultMaskDir, DicomMask.MaskDir);
            //Assert.AreEqual(TestSuiteSetup.DicomMaskDefaultMaskImageValue, DicomMask.MaskImageValue);
            //Assert.AreEqual(TestSuiteSetup.DicomMaskDefaultMaskPatientNameChar, DicomMask.MaskPatientNameChar);
            //Assert.AreEqual(TestSuiteSetup.DicomMaskDefaultMaskPatientNameCnt, DicomMask.MaskPatientNameCnt); 

            Assert.IsTrue(DicomMask.InitializeMaskSettings());

            Assert.AreEqual(TestSuiteSetup.DicomMaskConfigMaskData, DicomMask.MaskData);
            Assert.AreEqual(TestSuiteSetup.DicomMaskConfigLeft, DicomMask.Left);
            Assert.AreEqual(TestSuiteSetup.DicomMaskConfigWidth, DicomMask.Width);
            Assert.AreEqual(TestSuiteSetup.DicomMaskConfigStartrow, DicomMask.StartRow);
            Assert.AreEqual(TestSuiteSetup.DicomMaskConfigEndrow, DicomMask.EndRow);
            Assert.AreEqual(TestSuiteSetup.DicomMaskDefaultMaskDir, DicomMask.MaskDir);
            Assert.AreEqual(TestSuiteSetup.DicomMaskConfigMaskImageValue, DicomMask.MaskImageValue);
            Assert.AreEqual(TestSuiteSetup.DicomMaskConfigMaskPatientNameChar, DicomMask.MaskPatientNameChar);
            Assert.AreEqual(TestSuiteSetup.DicomMaskConfigMaskPatientNameCnt, DicomMask.MaskPatientNameCnt);
        }

        [TestMethod]
        [ExpectedException(typeof(InvalidOperationException))]
        public void ApplyMaskNullImageParam()
        {
            DicomMask.MaskData = true;
            mask.ApplyMask(null, 1, 1);
        }

        [TestMethod]
        [ExpectedException(typeof(InvalidOperationException))]
        public void ApplyMaskImageParamSizeZero()
        {
            DicomMask.MaskData = true;
            byte[] image = new byte[0];
            mask.ApplyMask(image, 0, 0);
        }

        [TestMethod]
        [ExpectedException(typeof(InvalidOperationException))]
        public void ApplyMaskBadParams()
        {
            DicomMask.MaskData = true;
            byte[] image = new byte[100];
            mask.ApplyMask(image, 0, 0);
        }

        [TestMethod]
        public void ApplyMaskLowerRangeParams()
        {
            DicomMask.MaskData = true;
            byte[] image = new byte[100];
            int rows = DicomMask.StartRow - 1;
            int cols = DicomMask.Left - 1;

            mask.ApplyMask(image, rows, cols);

            Assert.AreEqual(rows + 1, DicomMask.StartRow);
            Assert.AreEqual(cols + 1, DicomMask.Left);
        }

        [TestMethod]
        public void ApplyMaskUpperRangeParams()
        {
            DicomMask.MaskData = true;
            byte[] image = new byte[100];
            int rows = DicomMask.StartRow + 1;
            int cols = DicomMask.Left + DicomMask.Width + 1;

            mask.ApplyMask(image, DicomMask.StartRow + 1, DicomMask.Left + DicomMask.Width + 1);

            Assert.AreEqual(rows - 1, DicomMask.StartRow);
            Assert.AreEqual(cols - 1, DicomMask.Left + DicomMask.Width);

        }

        [TestMethod]
        public void MaskDirIsRelative()
        {
            DicomMask.MaskDir = "..\\temp";
            Assert.IsTrue(DicomMask.IsMaskDirRelative());
            Assert.AreEqual(ExecutionTime.Instance.InstallDir + "..\\temp", DicomMask.MaskPath);
        }

        [TestMethod]
        public void MaskDirIsAbsolute()
        {
            DicomMask.MaskDir = "c:\\temp";
            Assert.IsFalse(DicomMask.IsMaskDirRelative());
            DicomMask.MaskDir = "\\\\DELLFAST\\temp";
            Assert.IsFalse(DicomMask.IsMaskDirRelative());
            Assert.AreEqual("\\\\DELLFAST\\temp", DicomMask.MaskPath);
        }

        [TestMethod]
        public void ComputerNameAbsolute()
        {
            DicomMask.MaskDir = "c:\\temp";
            Assert.IsFalse(DicomMask.IsMaskDirRelative());
            DicomMask.ComputerName = "DELLFAST";
            Assert.AreEqual("", DicomMask.ComputerName);
        }

        [TestMethod]
        public void ComputerNameNull()
        {
            DicomMask.MaskDir = "..\\temp";
            Assert.IsTrue(DicomMask.IsMaskDirRelative());
            DicomMask.ComputerName = null;
            Assert.AreEqual("[null]", DicomMask.ComputerName);
        }

        [TestMethod]
        public void ComputerNameLong()
        {
            DicomMask.MaskDir = "..\\temp";
            Assert.IsTrue(DicomMask.IsMaskDirRelative());
            DicomMask.ComputerName = "0123456789xxxyyy";
            Assert.AreEqual("[456789xxxyyy]", DicomMask.ComputerName);
        }

        [TestMethod]
        public void ComputerNameRelative()
        {
            DicomMask.MaskDir = "..\\temp";
            Assert.IsTrue(DicomMask.IsMaskDirRelative());
            DicomMask.ComputerName = "DELLFAST";
            Assert.AreEqual("[DELLFAST]", DicomMask.ComputerName);
        }

        [TestMethod]
        [ExpectedException(typeof(UniException))]
        public void ReadMaskFromDiskBadDir()
        {
            DicomMask.MaskDir = "";
            mask.ReadMaskFromDisk("---");
        }

        [TestMethod]
        public void WriteToDisk()
        {
            DicomFile file = new DicomFile(ExecutionTime.Instance.MonochromeSampleDicomFile);

            Assert.IsFalse(file.ReadFromDisk(db, MockProgram.Instance));
            Assert.IsTrue(file.UpdateAndMaskCache(MockProgram.Instance));

            Assert.IsTrue(File.Exists(DicomMask.MaskDir + "\\" + file.Record.GetShortDicomRef() + DicomMask.StdExt));

            File.Delete(ExecutionTime.Instance.OutputDir + "\\TestExport.dicom");
            Assert.IsTrue(file.WriteToDisk(db, true, ExecutionTime.Instance.OutputDir + "\\TestExport.dicom"));
            Assert.IsTrue(File.Exists(ExecutionTime.Instance.OutputDir + "\\TestExport.dicom"));
        }

        [TestMethod]
        public void WriteMaskToDiskBadParam()
        {
            Assert.IsFalse(mask.WriteMaskToDisk("---", null));
        }

        [TestMethod]
        public void WriteMaskToDiskNoMaskDir()
        {
            DicomMask.MaskDir = "";
            Assert.IsTrue(mask.WriteMaskToDisk("---", MockProgram.Instance));
        }

        [TestMethod]
        public void WriteMaskToDiskMaskDirLength()
        {
            StringBuilder file = new StringBuilder();
            for (int x = 0; x < 100; x++ )
                file.Append("1234567890");
            DicomMask.MaskDir = file.ToString();
            Assert.IsFalse(mask.WriteMaskToDisk("---", MockProgram.Instance));
        }

        [TestMethod]
        [ExpectedException(typeof(UniException))]
        public void RestoreMaskEmptyMaskDir()
        {
            DicomMask.MaskDir = "";
            mask.RestoreMask(null, null);
        }

        [TestMethod]
        [ExpectedException(typeof(UniException))]
        public void RestoreMaskImageBuffLen()
        {
            DicomFile file = new DicomFile(ExecutionTime.Instance.MonochromeSampleDicomFile);
            Assert.IsFalse(file.ReadFromDisk(db, MockProgram.Instance));
            Assert.IsTrue(file.UpdateAndMaskCache(MockProgram.Instance));
            mask.RestoreMask(new byte[1], file.Record.GetShortDicomRef());
        }

        [TestMethod]
        [ExpectedException(typeof(UniException))]
        public void ReadMaskFromDiskException()
        {
            DicomFile file = new DicomFile(ExecutionTime.Instance.MonochromeSampleDicomFile);
            mask.ReadMaskFromDisk(file.Record.GetShortDicomRef());
        }


        [TestMethod]
        [ExpectedException(typeof(UniException))]
        public void ReadMaskFromDiskBadID()
        {
            DicomFile file = new DicomFile(ExecutionTime.Instance.ColourSampleDicomFile);
            Assert.IsFalse(file.ReadFromDisk(db, MockProgram.Instance));
            String pathFilename = DicomMask.GetMaskPathFilename(file.Record.GetShortDicomRef());

            Assert.IsTrue(WriteBadMask(pathFilename, "XXXX", file.Record.GetShortDicomRef(), DicomMask.MaskFileTerm, DicomMask.StartRow, DicomMask.EndRow, DicomMask.Left, DicomMask.Width, 255, 255));

            mask.ReadMaskFromDisk(file.Record.GetShortDicomRef());
        }

        [TestMethod]
        [ExpectedException(typeof(UniException))]
        public void ReadMaskFromDiskBadDicomRef()
        {
            DicomFile file = new DicomFile(ExecutionTime.Instance.ColourSampleDicomFile);
            Assert.IsFalse(file.ReadFromDisk(db, MockProgram.Instance));
            String pathFilename = DicomMask.GetMaskPathFilename(file.Record.GetShortDicomRef());

            Assert.IsTrue(WriteBadMask(pathFilename, DicomMask.MaskFileID, "1111111", DicomMask.MaskFileTerm, DicomMask.StartRow, DicomMask.EndRow, DicomMask.Left, DicomMask.Width, 255, 255));

            mask.ReadMaskFromDisk(file.Record.GetShortDicomRef());
        }

        [TestMethod]
        [ExpectedException(typeof(UniException))]
        public void ReadMaskFromDiskBadByteCnt()
        {
            DicomFile file = new DicomFile(ExecutionTime.Instance.ColourSampleDicomFile);
            Assert.IsFalse(file.ReadFromDisk(db, MockProgram.Instance));
            String pathFilename = DicomMask.GetMaskPathFilename(file.Record.GetShortDicomRef());

            Assert.IsTrue(WriteBadMask(pathFilename, DicomMask.MaskFileID, file.Record.GetShortDicomRef(), DicomMask.MaskFileTerm, 0, DicomMask.EndRow, DicomMask.Left, DicomMask.Width, 255, 255));

            mask.ReadMaskFromDisk(file.Record.GetShortDicomRef());
        }

        [TestMethod]
        [ExpectedException(typeof(UniException))]
        public void ReadMaskFromDiskBadImageRows()
        {
            DicomFile file = new DicomFile(ExecutionTime.Instance.ColourSampleDicomFile);
            Assert.IsFalse(file.ReadFromDisk(db, MockProgram.Instance));
            String pathFilename = DicomMask.GetMaskPathFilename(file.Record.GetShortDicomRef());

            Assert.IsTrue(WriteBadMask(pathFilename, DicomMask.MaskFileID, file.Record.GetShortDicomRef(), DicomMask.MaskFileTerm, DicomMask.StartRow, DicomMask.EndRow, DicomMask.Left, DicomMask.Width, 0, 255));

            mask.ReadMaskFromDisk(file.Record.GetShortDicomRef());
        }

        [TestMethod]
        [ExpectedException(typeof(UniException))]
        public void ReadMaskFromDiskBadImageCols()
        {
            DicomFile file = new DicomFile(ExecutionTime.Instance.ColourSampleDicomFile);
            Assert.IsFalse(file.ReadFromDisk(db, MockProgram.Instance));
            String pathFilename = DicomMask.GetMaskPathFilename(file.Record.GetShortDicomRef());

            Assert.IsTrue(WriteBadMask(pathFilename, DicomMask.MaskFileID, file.Record.GetShortDicomRef(), DicomMask.MaskFileTerm, DicomMask.StartRow, DicomMask.EndRow, DicomMask.Left, DicomMask.Width, 255, 0));

            mask.ReadMaskFromDisk(file.Record.GetShortDicomRef());
        }

        [TestMethod]
        [ExpectedException(typeof(UniException))]
        public void ReadMaskFromDiskBadImageTerm()
        {
            DicomFile file = new DicomFile(ExecutionTime.Instance.ColourSampleDicomFile);
            Assert.IsFalse(file.ReadFromDisk(db, MockProgram.Instance));
            String pathFilename = DicomMask.GetMaskPathFilename(file.Record.GetShortDicomRef());

            Assert.IsTrue(WriteBadMask(pathFilename, DicomMask.MaskFileID, file.Record.GetShortDicomRef(), "xxx", DicomMask.StartRow, DicomMask.EndRow, DicomMask.Left, DicomMask.Width, 255, 255));

            mask.ReadMaskFromDisk(file.Record.GetShortDicomRef());
        }

        private bool WriteBadMask(string pathFilename, string MaskFileID, string ShortDicomRef, string MaskFileTerm, int startRow, int endRow, int left, int width, int imageRows, int imageCols)
        {
            bool rc = false;

            StreamWriter writer = null;
            BinaryWriter binwriter = null;

            try
            {
                byte[] maskBuffer = new byte[8000];
                writer = new StreamWriter(pathFilename);
                binwriter = new BinaryWriter(writer.BaseStream);

                char[] id = MaskFileID.ToCharArray();
                binwriter.Write(id);

                binwriter.Write(ShortDicomRef);
                binwriter.Write(imageRows);
                binwriter.Write(imageCols);
                binwriter.Write(startRow);
                binwriter.Write(endRow);
                binwriter.Write(left);
                binwriter.Write(width);
                binwriter.Write(2);
                binwriter.Write(maskBuffer.Length);
                binwriter.Write(maskBuffer);

                binwriter.Write(MaskFileTerm);

                binwriter.Flush();
                rc = true;
            }
            catch (Exception)
            {
                throw;
            }
            finally
            {
                if (writer != null)
                    writer.Close();
                if (binwriter != null)
                    binwriter.Close();
            }
            return rc;
        }
    }
}
