using System;
using System.Text;
using System.Collections.Generic;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Windows.Forms;
using WillStott.MxAppToolbox;
using osImageManagerLib;
using osImageManagerLibFIT;

namespace osImageManagerLibUnitTests
{
    /// <summary>
    /// Summary description for DlgProcDICOMCorrectionTest
    /// </summary>
    [TestClass]
    public class DlgProcDICOMCorrectionTests
    {
        private Examination exam;
        private OsDatabases db;

        public DlgProcDICOMCorrectionTests()
        {
            exam = new Examination();
            DicomFileSet disk = new DicomFileSet();
            DicomFile[] files = disk.MakeRecordsFromDirectory(ExecutionTime.Instance.DataDir);
            foreach (DicomFile file in files)
                exam.Add(file);
            db = new OsDatabases();
            db.Connect(TestSuiteSetup.AdminUser, TestSuiteSetup.AdminUserPassword);
        }

        [ClassInitialize()]
        public static void DlgProcDICOMCorrectionTestInitialize(TestContext testContext)
        {
            Assert.IsTrue(TestSuiteSetup.CreateStdDirAndSampleFiles(TestSuiteSetup.ProjectRootFromUnitTestRuntime, MockProgram.Instance));
        }

         #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class

        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        [TestMethod]
        [ExpectedException(typeof(UniException))]
        public void CtorNullRecord()
        {
            DlgProcDICOMCorrection dlgproc = new DlgProcDICOMCorrection(null, db, MockProgram.Instance);
        }
        [TestMethod]
        [ExpectedException(typeof(UniException))]
        public void CtorNullDatabase()
        {
            DlgProcDICOMCorrection dlgproc = new DlgProcDICOMCorrection(new DicomFile(), null, MockProgram.Instance);
        }
        [TestMethod]
        [ExpectedException(typeof(UniException))]
        public void CtorNullLog()
        {
            DlgProcDICOMCorrection dlgproc = new DlgProcDICOMCorrection(new DicomFile(), db, null);
        }

        [TestMethod]
        public void CtorExamList()
        {
            DlgProcDICOMCorrection dlgproc = new DlgProcDICOMCorrection(exam, db, MockProgram.Instance);
            Assert.AreEqual(6, dlgproc.GetDicomFiles().Length);
        }

        [TestMethod]
        public void GetDoOKMsgButtons()
        {
            DlgProcDICOMCorrection dlgproc = new DlgProcDICOMCorrection(new DicomFile(), db, MockProgram.Instance);
            Assert.AreEqual("", dlgproc.DoOKMessage);
            Assert.AreEqual(MessageBoxButtons.OK, dlgproc.DoOKMessageButtons);
        }

        [TestMethod]
        public void DicomRefLabelValid()
        {
            DicomFile[] files = exam.GetList();
            Assert.IsTrue(files.Length > 0);
            DlgProcDICOMCorrection dlgproc = new DlgProcDICOMCorrection(files[0], db, MockProgram.Instance);
            Assert.IsFalse(files[0].ReadFromDisk(db, MockProgram.Instance));

            files[0].Record.DicomRef = "1.2.826.999999.1.9900.1.20050531130000000.9.9.9";
            Assert.AreEqual(files[0].Record.DicomRef, dlgproc.DicomRefLabel);
        }

        [TestMethod]
        public void Duplicate()
        {
            DicomFile[] files = exam.GetList();
            Assert.IsTrue(files.Length > 0);
            DlgProcDICOMCorrection dlgproc = new DlgProcDICOMCorrection(files[0], db, MockProgram.Instance);
            files[0].Record.Duplicate = true;
            Assert.IsTrue(dlgproc.Duplicate);
        }
        [TestMethod]
        public void DicomRefLabelInvalid()
        {
            DicomFile[] files = exam.GetList();
            Assert.IsTrue(files.Length > 0);
            DlgProcDICOMCorrection dlgproc = new DlgProcDICOMCorrection(files[0], db, MockProgram.Instance);
            Assert.IsFalse(files[0].ReadFromDisk(db, MockProgram.Instance));

            files[0].Record.DicomRef = "0";
            Assert.AreEqual(" - INVALID", dlgproc.DicomRefLabel);
        }

        [TestMethod]
        public void DoOK()
        {
            DicomFile[] files = exam.GetList();
            Assert.IsTrue(files.Length > 0);
            Assert.IsFalse(files[0].ReadFromDisk(db, MockProgram.Instance));

            files[0].Record.DicomRef = "1.2.826.999999.1.9900.1.20050531130000000.9.9.9";
            DlgProcDICOMCorrection dlgproc = new DlgProcDICOMCorrection(files[0], db, MockProgram.Instance);

            Assert.AreEqual(DialogResult.OK, dlgproc.DoOK(TestSuiteSetup.ValidPatientName, TestSuiteSetup.ValidVolunteerRef, TestSuiteSetup.ValidAppointmentDateForVolunteerRef));
            Assert.AreEqual(MessageBoxButtons.OK, dlgproc.DoOKMessageButtons);
        }

        [TestMethod]
        public void DoOKDuplicate()
        {
            DicomFile[] files = exam.GetList();
            Assert.IsTrue(files.Length > 0);
            Assert.IsFalse(files[0].ReadFromDisk(db, MockProgram.Instance));

            DlgProcDICOMCorrection dlgproc = new DlgProcDICOMCorrection(files[0], db, MockProgram.Instance);

            Assert.AreEqual(DialogResult.None, dlgproc.DoOK(TestSuiteSetup.ValidPatientName, TestSuiteSetup.ValidVolunteerRef, TestSuiteSetup.ValidAppointmentDateForVolunteerRef));
            Assert.AreEqual(MessageBoxButtons.YesNo, dlgproc.DoOKMessageButtons);
            Assert.IsTrue(dlgproc.DoOKMessage.Contains("0020,00D0"));
        }

        [TestMethod]
        public void DoOKOrphan()
        {
            DicomFile[] files = exam.GetList();
            Assert.IsTrue(files.Length > 0);
            Assert.IsFalse(files[0].ReadFromDisk(db, MockProgram.Instance));

            DlgProcDICOMCorrection dlgproc = new DlgProcDICOMCorrection(files[0], db, MockProgram.Instance);

            Assert.AreEqual(DialogResult.None, dlgproc.DoOK(TestSuiteSetup.ValidPatientName, TestSuiteSetup.ValidVolunteerRef, new DateTime(2007, 1, 21)));
            Assert.AreEqual(MessageBoxButtons.YesNo, dlgproc.DoOKMessageButtons);
            Assert.IsTrue(dlgproc.DoOKMessage.Contains("0008,0020"));
        }
        [TestMethod]
        public void DoOKNameMismatch()
        {
            DicomFile[] files = exam.GetList();
            Assert.IsTrue(files.Length > 0);
            Assert.IsFalse(files[0].ReadFromDisk(db, MockProgram.Instance));

            DlgProcDICOMCorrection dlgproc = new DlgProcDICOMCorrection(files[0], db, MockProgram.Instance);

            Assert.AreEqual(DialogResult.None, dlgproc.DoOK("xxx", TestSuiteSetup.ValidVolunteerRef, TestSuiteSetup.ValidAppointmentDateForVolunteerRef));
            Assert.AreEqual(MessageBoxButtons.OK, dlgproc.DoOKMessageButtons);
            Assert.IsTrue(dlgproc.DoOKMessage.Contains("0010,0010"));
        }

        [TestMethod]
        public void DoOKInvalidDicomRef()
        {
            DicomFile[] files = exam.GetList();
            Assert.IsTrue(files.Length > 0);
            Assert.IsFalse(files[0].ReadFromDisk(db, MockProgram.Instance));
            files[0].Record.DicomRef = "0";

            DlgProcDICOMCorrection dlgproc = new DlgProcDICOMCorrection(files[0], db, MockProgram.Instance);

            Assert.AreEqual(DialogResult.None, dlgproc.DoOK(TestSuiteSetup.ValidPatientName, TestSuiteSetup.ValidVolunteerRef, TestSuiteSetup.ValidAppointmentDateForVolunteerRef));
            Assert.AreEqual(MessageBoxButtons.OK, dlgproc.DoOKMessageButtons);
        }

        [TestMethod]
        public void DoOKExam()
        {
            DlgProcDICOMCorrection dlgproc = new DlgProcDICOMCorrection(exam, db, MockProgram.Instance);
            DicomFile[] files = dlgproc.GetDicomFiles();
            Assert.AreEqual(7, files.Length);
            foreach (DicomFile file in files)
                file.ReadFromDisk(db, MockProgram.Instance);

            Assert.AreEqual(DialogResult.None, dlgproc.DoOK(TestSuiteSetup.ValidPatientName, TestSuiteSetup.ValidVolunteerRef, TestSuiteSetup.ValidAppointmentDateForVolunteerRef));
            Assert.AreEqual(MessageBoxButtons.YesNo, dlgproc.DoOKMessageButtons);
        }

        [TestMethod]
        public void SetDetailsForAllFilesInExam()
        {
            DlgProcDICOMCorrection dlgproc = new DlgProcDICOMCorrection(exam, db, MockProgram.Instance);
            DicomFile[] files = dlgproc.GetDicomFiles();
            Assert.AreEqual(7, files.Length);
            foreach (DicomFile file in files)
            {
                Assert.AreNotEqual(TestSuiteSetup.ValidPatientName, file.Record.PatientName);
                Assert.AreNotEqual(TestSuiteSetup.ValidVolunteerRef, file.Record.VolunteerRef);
                Assert.AreNotEqual(TestSuiteSetup.ValidAppointmentDateForVolunteerRef, file.Record.AppointmentDate);
                file.Record.Orphan = true;
                file.Record.NameMismatch = true;
                file.Record.Duplicate = true;
                file.Record.UnspecifiedErrorText = "error";
            }

            dlgproc.SelectedDicomFile.Record.PatientName = TestSuiteSetup.ValidPatientName;
            dlgproc.SelectedDicomFile.Record.VolunteerRef = TestSuiteSetup.ValidVolunteerRef;
            dlgproc.SelectedDicomFile.Record.AppointmentDate = TestSuiteSetup.ValidAppointmentDateForVolunteerRef;
            dlgproc.SelectedDicomFile.Record.ResetValidationFlags();
            dlgproc.SelectedDicomFile.Record.ResetUnspecifiedError();
            dlgproc.SelectedDicomFile.Record.SetValidationFlags(db, MockProgram.Instance);

            Assert.IsFalse(dlgproc.SetDetailsForAllFilesInExam(null));
            Assert.IsTrue(dlgproc.SetDetailsForAllFilesInExam(dlgproc.SelectedDicomFile));


            foreach (DicomFile file in files)
            {
                Assert.AreEqual(TestSuiteSetup.ValidPatientName, file.Record.PatientName);
                Assert.AreEqual(TestSuiteSetup.ValidVolunteerRef, file.Record.VolunteerRef);
                Assert.AreEqual(TestSuiteSetup.ValidAppointmentDateForVolunteerRef, file.Record.AppointmentDate);
                Assert.IsFalse(file.Record.Orphan);
                Assert.IsFalse(file.Record.NameMismatch);
                Assert.IsFalse(file.Record.Duplicate);
                Assert.IsFalse(file.Record.UnspecifiedError);
            }
        }

        [TestMethod]
        public void SaveRestoreValues()
        {
            DicomFile file = new DicomFile();
            file.Record.PatientName = TestSuiteSetup.ValidPatientName;
            file.Record.VolunteerRef = TestSuiteSetup.ValidVolunteerRef;
            file.Record.AppointmentDate = TestSuiteSetup.ValidAppointmentDateForVolunteerRef;

            DlgProcDICOMCorrection dlgproc = new DlgProcDICOMCorrection(file, db, MockProgram.Instance);

            dlgproc.SaveRecordValues();

            file.Record.PatientName = "xxx";
            file.Record.VolunteerRef = "xxx";
            file.Record.AppointmentDate = DateTime.MinValue;
            file.Record.DicomRef = "1.1.1.1.1.1";
            file.Record.Orphan = true;
            file.Record.NameMismatch = true;
            file.Record.Duplicate = false;
            file.Record.UnspecifiedErrorText = "hello";

            dlgproc.RestoreRecordValues();

            Assert.AreEqual(TestSuiteSetup.ValidPatientName, file.Record.PatientName);
            Assert.AreEqual(TestSuiteSetup.ValidVolunteerRef, file.Record.VolunteerRef);
            Assert.AreEqual(TestSuiteSetup.ValidAppointmentDateForVolunteerRef, file.Record.AppointmentDate);
            Assert.IsFalse(file.Record.Orphan);
            Assert.IsFalse(file.Record.NameMismatch);
            Assert.IsFalse(file.Record.Duplicate);
            Assert.IsTrue(file.Record.UnspecifiedError);
        }

        [TestMethod]
        public void GetDialogValues()
        {

            DicomFile file = new DicomFile(ExecutionTime.Instance.MonochromeSampleDicomFile);
            file.Record.PatientName = TestSuiteSetup.ValidPatientName;
            file.Record.VolunteerRef = TestSuiteSetup.ValidVolunteerRef;
            file.Record.AppointmentDate = TestSuiteSetup.ValidAppointmentDateForVolunteerRef;
            file.Record.UnspecifiedErrorText = "hello";
            file.Record.DicomRef = "1111";

            DlgProcDICOMCorrection dlgproc = new DlgProcDICOMCorrection(file, db, MockProgram.Instance);

            Assert.AreEqual(TestSuiteSetup.ValidPatientName, dlgproc.PatientNameLabel);
            Assert.AreEqual(TestSuiteSetup.ValidVolunteerRef, dlgproc.VolunteerRefLabel);
            Assert.AreEqual(TestSuiteSetup.ValidAppointmentDateForVolunteerRef, dlgproc.AppointmentDateValue);
            Assert.AreEqual(" - INVALID", dlgproc.DicomRefLabel);
            Assert.AreEqual("X = Unspecified error: hello", dlgproc.ErrorText);
            Assert.AreEqual(db, dlgproc.Databases);
            Assert.AreEqual(ExecutionTime.Instance.MonochromeSampleDicomFile, dlgproc.ImagePathFilename);

        }

        [TestMethod]
        public void GetSurnames()
        {
            Assert.AreEqual( TestSuiteSetup.ValidPatientSurname, DlgProcDICOMCorrection.GetSurname(TestSuiteSetup.ValidPatientName, true));
            Assert.AreEqual("", DlgProcDICOMCorrection.GetSurname("", true));
            Assert.AreEqual("", DlgProcDICOMCorrection.GetSurname("", false));
            Assert.AreEqual("Stott", DlgProcDICOMCorrection.GetSurname("Stott",true));
            Assert.AreEqual("Stott", DlgProcDICOMCorrection.GetSurname("Stott", false));
        }

        [TestMethod]
        public void GetFirstNames()
        {
            Assert.AreEqual(TestSuiteSetup.ValidPatientFirstNames, DlgProcDICOMCorrection.GetFirstNames(TestSuiteSetup.ValidPatientName, true));
            Assert.AreEqual("%", DlgProcDICOMCorrection.GetFirstNames("Stott", true));
            Assert.AreEqual("%", DlgProcDICOMCorrection.GetFirstNames("Stott", false));
            Assert.AreEqual("Will", DlgProcDICOMCorrection.GetFirstNames("Stott^Will", true));
            Assert.AreEqual("Will", DlgProcDICOMCorrection.GetFirstNames("Will^Stott", false));
        }
    }
}
