using System;
using System.Text;
using System.Collections.Generic;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Windows.Forms;

using WillStott.MxAppToolbox;
using osImageManagerLib;
using osImageManagerLibFIT;

namespace osImageManagerLibUnitTests
{
    /// <summary>
    /// Summary description for DlgProcFileImportTests
    /// </summary>
    [TestClass]
    public class DlgProcFileImportTreeTest
    {
        DlgProcFileImport _dlgproc;
        OsDatabases _databases;
        ListBox _imageListBox;
        ListBox _correctionsListBox;
        ProgressBar _workProgressBar;
        Label _progressTextLabel;

        public DlgProcFileImportTreeTest()
        {
            _imageListBox = new ListBox();
            _correctionsListBox = new ListBox();
            _workProgressBar = new ProgressBar();
            _progressTextLabel = new Label();
            _databases = new OsDatabases();

            _dlgproc = DlgProcFileImport.CreateDlgProc(DlgProcFileImport.ImportMode.AllFilesInTree, _databases, _imageListBox, _correctionsListBox, _workProgressBar, _progressTextLabel, 0, MockProgram.Instance);
            Assert.AreEqual(0, _dlgproc.CorrectionsListCnt);
            Assert.AreEqual(0, _dlgproc.ImageListCnt);

        }

        [TestInitialize()]
        public void MyTestInitialize() 
        { 
            if ( _databases.IsURAAdminLoggedOn() == false)
                _databases.Connect(TestSuiteSetup.AdminUser, TestSuiteSetup.AdminUserPassword);
        }

        [ClassInitialize()]
        public static void ClassInitialize(TestContext testContext)
        {
            TestSuiteSetup.CreateStdDirAndSampleFiles(TestSuiteSetup.ProjectRootFromUnitTestRuntime, MockProgram.Instance);
            DicomMask.InitializeMaskSettings();
            DicomMask.MaskDir = "";
        }

        [ClassCleanup()]
        public static void ClassCleanup() 
        {
            DicomMask.InitializeMaskSettings();

        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class

        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion


        [TestMethod]
        public void Factory()
        {
            DlgProcFileImport dlgproc = null;
            dlgproc = DlgProcFileImport.CreateDlgProc(DlgProcFileImport.ImportMode.AllFilesInTree, null, null, null, null, null, 1, MockProgram.Instance);
            Assert.AreEqual(Resources.StrImportModeTree, dlgproc.DialogTitle);
            dlgproc = DlgProcFileImport.CreateDlgProc(DlgProcFileImport.ImportMode.ExaminationCollections, null, null, null, null, null, 0, MockProgram.Instance);
            Assert.AreEqual(Resources.StrImportModeExam, dlgproc.DialogTitle);
            dlgproc = DlgProcFileImport.CreateDlgProc(DlgProcFileImport.ImportMode.IndividualFile, null, null, null, null, null, 0, MockProgram.Instance);
            Assert.AreEqual(Resources.StrImportModeFile, dlgproc.DialogTitle);
        }

        [TestMethod]
        public void GetDialogTitleTree()
        {
            Assert.AreEqual(Resources.StrImportModeTree, _dlgproc.DialogTitle);
        }

        [TestMethod]
        public void ImportModeToInt32()
        {
            Assert.AreEqual(2, DlgProcFileImport.ImportModeToInt32(DlgProcFileImport.ImportMode.None));
            Assert.AreEqual(2, DlgProcFileImport.ImportModeToInt32(DlgProcFileImport.ImportMode.AllFilesInTree));
            Assert.AreEqual(1, DlgProcFileImport.ImportModeToInt32(DlgProcFileImport.ImportMode.IndividualFile));
            Assert.AreEqual(3, DlgProcFileImport.ImportModeToInt32(DlgProcFileImport.ImportMode.ExaminationCollections));
        }

        [TestMethod]
        public void Int32ToImportMode()
        {
            Assert.AreEqual(DlgProcFileImport.ImportMode.AllFilesInTree, DlgProcFileImport.Int32ToImportMode(2));
            Assert.AreEqual(DlgProcFileImport.ImportMode.AllFilesInTree, DlgProcFileImport.Int32ToImportMode(2 ));
            Assert.AreEqual(DlgProcFileImport.ImportMode.IndividualFile, DlgProcFileImport.Int32ToImportMode(1 ));
            Assert.AreEqual(DlgProcFileImport.ImportMode.ExaminationCollections, DlgProcFileImport.Int32ToImportMode(3));
        }

        [TestMethod]
        public void CtorTreeNullImageList()
        {
            DlgProcFileImport dlg = DlgProcFileImport.CreateDlgProc(DlgProcFileImport.ImportMode.AllFilesInTree, _databases, null, _correctionsListBox, _workProgressBar, _progressTextLabel, 0, MockProgram.Instance);
            Assert.AreEqual(-1, dlg.ImageListCnt);    
        }

        [TestMethod]
        public void CtorTreeNullCorrectionList()
        {
            DlgProcFileImport dlg = DlgProcFileImport.CreateDlgProc(DlgProcFileImport.ImportMode.AllFilesInTree, _databases, _imageListBox, null, _workProgressBar, _progressTextLabel, 0, MockProgram.Instance);
            Assert.AreEqual(-1, dlg.CorrectionsListCnt);
        }

        [TestMethod()]
        public void DiskRefTreeIsValid()
        {
            Assert.IsFalse(_dlgproc.DiskRefIsValid(null));
            Assert.IsFalse(_dlgproc.DiskRefIsValid(""));
            Assert.IsFalse (_dlgproc.DiskRefIsValid("UAT"));
            Assert.IsTrue(_dlgproc.DiskRefIsValid("xxx"));
        }


        [TestMethod()]
        public void GetFilenamesForSelectedItemTreeNull()
        {
            Assert.IsNull(_dlgproc.GetFilenamesForSelectedItem(null));
        }

        [TestMethod()]
        public void GetFilenamesForSelectedItemTree()
        {
            DicomFile file = new DicomFile();
            file.Record.DicomPathFilename = "c:\temp.txt";
            String[] list = _dlgproc.GetFilenamesForSelectedItem(file);
            Assert.AreEqual(1, list.Length);
            Assert.AreEqual("c:\temp.txt", list[0]);

        }

        [TestMethod()]
        public void GetAcceptButtonStateForSelectionTreeNull()
        {
           Assert.IsFalse(_dlgproc.GetAcceptButtonStateForSelection(null));
        }

        [TestMethod()]
        public void GetAcceptButtonStateForSelectionTreeNameMismatch()
        {
           DicomFile file = new DicomFile();
           file.Record.NameMismatch = true;
           Assert.IsFalse(_dlgproc.GetAcceptButtonStateForSelection(file));
        }

        [TestMethod()]
        public void GetAcceptButtonStateForSelectionTreeUnspecError()
        {
           DicomFile file = new DicomFile();
           file.Record.UnspecifiedErrorText = "error";
           Assert.IsFalse(_dlgproc.GetAcceptButtonStateForSelection(file));
        }
        
        [TestMethod()]
        public void GetAcceptButtonStateForSelectionTreeOrphan()
        {
           DicomFile file = new DicomFile();
           file.Record.Orphan = true;
           Assert.IsTrue(_dlgproc.GetAcceptButtonStateForSelection(file));
        }
    
        [TestMethod]
        public void ReadFilesTree()
        {
            _dlgproc.ReadFiles(ExecutionTime.Instance.DataDir, null);

            Assert.AreEqual(5, _dlgproc.CorrectionsListCnt);
            Assert.AreEqual(1, _dlgproc.ImageListCnt);
        }

        [TestMethod()]
        public void ReadFilesTreeNoneFound()
        {
            _dlgproc.ReadFiles(ExecutionTime.Instance.ScriptsDir, null);
            Assert.AreEqual(0, _dlgproc.ImageListCnt);
            Assert.AreEqual(0, _dlgproc.CorrectionsListCnt);
        }

        [TestMethod()]
        [ExpectedException(typeof(UniException))]
        public void ReadFilesTreeEmptyLocation()
        {
            _dlgproc.ReadFiles("", null);
            Assert.AreEqual(0, _dlgproc.ImageListCnt);
            Assert.AreEqual(0, _dlgproc.CorrectionsListCnt);
        }

        [TestMethod()]
        [ExpectedException(typeof(UniException))]
        public void ReadFilesTreeNullLocation()
        {
            _dlgproc.ReadFiles(null, null);
            Assert.AreEqual(0, _dlgproc.ImageListCnt);
            Assert.AreEqual(0, _dlgproc.CorrectionsListCnt);
        }

        [TestMethod()]
        public void ImportFilesTree()
        {
            _dlgproc.ReadFiles(ExecutionTime.Instance.DataDir, null);
            Assert.AreEqual(1, _dlgproc.ImageListCnt);

            Assert.AreEqual(1, _dlgproc.ImportFiles("UAT", new DateTime(2006, 8, 21)));

            _databases.DeleteDiskLabelImages("UAT");
            ImageRecord rec = new ImageRecord();
            rec.DicomRef = TestSuiteSetup.TestImageDicomRef;
            if (_databases.GetImageIDForDicomRef(rec, MockProgram.Instance).Length == 0)
                TestSuiteSetup.CreateMonochromeImageRecord(MockProgram.Instance);
        }

        [TestMethod()]
        public void ImportFilesTreeNone()
        {
            Assert.AreEqual(0, _dlgproc.ImageListCnt);
            Assert.AreEqual(0, _dlgproc.ImportFiles("UAT", new DateTime(2006, 8, 21)));
        }

        [TestMethod()]
        [ExpectedException(typeof(UniException))]
        public void ImportFilesTreeBad()
        {
            _dlgproc.ReadFiles(ExecutionTime.Instance.DataDir, null);

            Assert.AreEqual(1, _dlgproc.ImageListCnt);
            DicomFile file = _imageListBox.Items[0] as DicomFile;
            file.Record.VolunteerRef = "";

            _dlgproc.ImportFiles("UAT", new DateTime(2006, 8, 21));
        }

        [TestMethod]
        public void AcceptClickTreeNullParams()
        {
            Assert.IsFalse(_dlgproc.acceptClick(null, _imageListBox));
            Assert.IsFalse(_dlgproc.acceptClick(_correctionsListBox, null));
            Assert.IsFalse(_dlgproc.acceptClick(null, null));
        }

        [TestMethod]
        public void AcceptClickTreeEmpty()
        {
            _correctionsListBox.Items.Clear();
            _imageListBox.Items.Clear();
            Assert.IsFalse(_dlgproc.acceptClick(_correctionsListBox, _imageListBox));
            Assert.AreEqual(0, _imageListBox.Items.Count);
            Assert.AreEqual(0, _correctionsListBox.Items.Count);
        }

        [TestMethod]
        public void AcceptClickTreeOneCorrectionItem()
        {
            _imageListBox.Items.Clear();
            _correctionsListBox.Items.Clear();
            _correctionsListBox.Items.Add(new DicomFile());
            _correctionsListBox.SelectedIndex = 0;

            Assert.AreEqual(1, _correctionsListBox.Items.Count);

            Assert.IsTrue(_dlgproc.acceptClick(_correctionsListBox, _imageListBox));

            Assert.AreEqual(1, _imageListBox.Items.Count);
            Assert.AreEqual(0, _correctionsListBox.Items.Count);
        }

        [TestMethod]
        public void AcceptClickTreeTwoCorrectionItems()
        {
            _imageListBox.Items.Clear();
            _correctionsListBox.Items.Clear();
            _correctionsListBox.Items.Add(new DicomFile());
            _correctionsListBox.Items.Add(new DicomFile());
            _correctionsListBox.SelectedIndex = 1;

            Assert.AreEqual(2, _correctionsListBox.Items.Count);

            Assert.IsTrue(_dlgproc.acceptClick(_correctionsListBox, _imageListBox));

            Assert.AreEqual(1, _imageListBox.Items.Count);
            Assert.AreEqual(1, _correctionsListBox.Items.Count);
        }

        [TestMethod]
        public void AcceptClickTreeThreeCorrectionItems()
        {
            _imageListBox.Items.Clear();
            _correctionsListBox.Items.Clear();
            _correctionsListBox.Items.Add(new DicomFile());
            _correctionsListBox.Items.Add(new DicomFile());
            _correctionsListBox.Items.Add(new DicomFile());
            _correctionsListBox.SelectedIndex = 0;

            Assert.AreEqual(3, _correctionsListBox.Items.Count);

            Assert.IsTrue(_dlgproc.acceptClick(_correctionsListBox, _imageListBox));

            Assert.AreEqual(1, _imageListBox.Items.Count);
            Assert.AreEqual(2, _correctionsListBox.Items.Count);
        }

    }
}
