using System;
using System.Text;
using System.Collections.Generic;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using osImageManagerLib;

namespace osImageManagerLibUnitTests
{
    /// <summary>
    /// Summary description for FavoritesTests
    /// </summary>
    [TestClass]
    public class FavoritesTests
    {
        private Favorites favorites;
        private Favorite armLeft;
        private Favorite armRight;
        private Favorite legRight;

        public FavoritesTests()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        [TestInitialize()]
        public void BeforeTest()
        {
            favorites = new Favorites();
            armLeft = new Favorite("X-Ray of left Humerus", new Uri("db://0000102"));
            armRight = new Favorite("X-Ray of right Humerus", new Uri("db://0000103"));
            legRight = new Favorite("X-Ray of right Fibia", new Uri("db://0000105"));

        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion
//1. exercise 14-2,14-3
        [TestMethod]
        public void EmptyFavoritesCountShouldBeZero()
        {
           Assert.AreEqual(0, favorites.Count);
        }
//2. exercise 14-4
        [TestMethod]
        public void AddFavoriteCountIsOne()
        {
            favorites.Add(armLeft);
            Assert.AreEqual(1, favorites.Count);
        }
//3. Add tests DIY after Exercise 14-4 
        [TestMethod]
        public void AddRemoveFavoriteCountIsZero()
        {
            favorites.Add(armLeft);
            favorites.Remove(armLeft);
            Assert.AreEqual(0, favorites.Count);
        }
//4.
        [TestMethod]
        public void AddTwoFavoritesCountIsTwo()
        {
            favorites.Add(armLeft);
            favorites.Add(armRight);
            Assert.AreEqual(2, favorites.Count);
        }
//5.
        [TestMethod]
        public void AddTwoFavoritesRemoveFavoriteCountIsOne()
        {
            favorites.Add(armLeft);
            favorites.Add(armRight);
            favorites.Remove(armLeft);
            Assert.AreEqual(1, favorites.Count);
        }
//6. exercise 15-1
        [TestMethod]
        public void AddTwoFavoritesClearCountIsZero()
        {
            favorites.Add(armLeft);
            favorites.Add(armRight);
            favorites.Clear();
            Assert.AreEqual(0, favorites.Count);
        }
//7. exercise 15-3
        [TestMethod]
        public void AddRemoveReturnsTrue()
        {
            favorites.Add(armLeft);
            Assert.IsTrue(favorites.Remove(armLeft));
        }
//8. exercise 15-4
        [TestMethod]
        public void AddFavoriteContainsReturnsTrue()
        {
            favorites.Add(armLeft);
            Assert.IsTrue(favorites.Contains(armLeft));
        }
//9. exercise 15-6
        [TestMethod]
        public void RemoveFromEmptyCollectionReturnsFalse()
        {
            Assert.IsFalse(favorites.Remove(armLeft));
        }
// add tests DIY after exercise 15-6
        [TestMethod]
        public void AddRemoveFavoriteContainsFavoriteReturnsFalse() //added after exercise 15-3
        {
            favorites.Add(armLeft);
            favorites.Remove(armLeft);
            Assert.IsFalse(favorites.Contains(armLeft));
        }
//10.
        [TestMethod]
        public void ContainsFavoriteNotInCollectionReturnsFalse()
        {
            Assert.IsFalse(favorites.Contains(armLeft));
        }
        [TestMethod]
        public void Add3FavoritesCopyTo() //added after exercise 15-3
        {
            favorites.Add(armLeft);
            favorites.Add(armRight);
            favorites.Add(legRight);
            Favorite[] favs = new Favorite[3];
            favorites.CopyTo(favs, 0);
            Assert.AreEqual("X-Ray of left Humerus", favs[0].Label);
            Assert.AreEqual("X-Ray of right Humerus", favs[1].Label);
            Assert.AreEqual("X-Ray of right Fibia", favs[2].Label);

        }
//11.
        [TestMethod]
        public void AddThreeFavoritesEnumerateThree()
        {
            favorites.Add(armLeft);
            favorites.Add(armRight);
            favorites.Add(legRight);
            IEnumerator<Favorite> iter = favorites.GetEnumerator();
            iter.MoveNext();
            Assert.AreEqual(armLeft, iter.Current);
            iter.MoveNext();
            Assert.AreEqual(armRight, iter.Current);
            iter.MoveNext();
            Assert.AreEqual(legRight, iter.Current);
        }
        [TestMethod]
        public void IsReadOnlyShouldBeFalse() //added after exercise 15-3
        {
            Assert.IsFalse(favorites.IsReadOnly);
        }
//12. exercise 15-7
        [TestMethod]
        public void AddTwoFavoritesFindLabelIsTrue()
        {
            favorites.Add(armLeft);
            favorites.Add(armRight);
            Assert.AreEqual(armLeft.Uri, favorites[armLeft.Label]);
            Assert.AreEqual(armRight.Uri, favorites[armRight.Label]);
        }
//13. exercise 15-8
        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException))]
        public void AddFavoriteNullLabel()
        {
            Favorite favorite = new Favorite(null, armLeft.Uri);
            favorites.Add(favorite);
        }
//14. DIY after Exercise 15-8
        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException))]
        public void AddFavoriteNullURL()
        {
            Favorite favorite = new Favorite(armLeft.Label, null);
            favorites.Add(favorite);
        }
//15. DIY after Exercise 15-8
        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void AddTwoFavoritesWithSameLabel()
        {
            favorites.Add(armLeft);
            favorites.Add(armLeft);
        }
        //19. Exercise 16-1
        [TestMethod]
        public void FavoritesIndexerLabelNotContainedInCollection()
        {
            Assert.IsNull(favorites["unknown"]);
        }
        //20. Exercise 16-2
        [TestMethod]
        public void AddThreeAndFindOne()
        {
            favorites.Add(armLeft);
            favorites.Add(armRight);
            favorites.Add(legRight);
            Assert.AreEqual(armRight.Uri, favorites[armRight.Label]);

        }
        //21. Exercise 17-1
        [TestMethod]
        public void FindFavorite()
        {
            favorites.Add(armLeft);
            favorites.Add(armRight);
            Assert.AreEqual(armLeft, favorites.Find(armLeft.Label));
            Assert.IsNull(favorites.Find(legRight.Label));
        }
    }
}
