using System;
using System.Text;
using System.Collections.Generic;
using System.Data.SqlClient;
using System.Data.SqlTypes;
using System.Data;
using System.IO;

using Microsoft.VisualStudio.TestTools.UnitTesting;
using WillStott.MxAppToolbox;
using osImageManagerLib;
using osImageManagerLibFIT;

namespace osImageManagerLibUnitTests
{
    /// <summary>
    /// Summary description for osImageDbTest
    /// </summary>
    [TestClass]
    public class OsDatabasesTests
    {
        private OsDatabases db = new OsDatabases();
        private static String maskdir;

        public OsDatabasesTests()
        {
            db = new OsDatabases(); 
        }

        [TestInitialize()]
        public void TestInitialize() 
        { 
            db.Connect(TestSuiteSetup.AdminUser, TestSuiteSetup.AdminUserPassword);
        }

        [TestCleanup()]
        public void TestCleanup() 
        {
            if(db.IsUserLoggedOn() == false)
                db.Connect(TestSuiteSetup.AdminUser, TestSuiteSetup.AdminUserPassword);
            ImageRecord rec = new ImageRecord();
            rec.DicomRef = TestSuiteSetup.TestImageDicomRef;
            if (db.GetImageIDForDicomRef(rec, MockProgram.Instance).Length == 0)
                TestSuiteSetup.CreateMonochromeImageRecord(MockProgram.Instance);
            db.Disconnect();
        }

        [ClassInitialize()]
        public static void OsDatabasesTestInitialize(TestContext testContext)
        {
            Assert.IsTrue(TestSuiteSetup.CreateStdDirAndSampleFiles(TestSuiteSetup.ProjectRootFromUnitTestRuntime, MockProgram.Instance));
            OsDatabasesTests.maskdir = DicomMask.MaskDir;
            DicomMask.MaskDir = "";
        }

        [ClassCleanup()]
        public static void OsDatabasesTestCleanup() 
        {
            DicomMask.MaskDir = OsDatabasesTests.maskdir;
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        [TestMethod]
        public void Connect()
        {
            OsDatabases database = new OsDatabases();
            Assert.IsFalse(database.IsUserLoggedOn());
            database.Connect(TestSuiteSetup.AdminUser, TestSuiteSetup.AdminUserPassword);
            Assert.IsTrue(database.IsUserLoggedOn());
            Assert.AreEqual(TestSuiteSetup.ImageDb, database.ImageDbName);
            Assert.AreEqual(TestSuiteSetup.ImageDataSource, database.ImageDbSource);
            Assert.AreEqual(TestSuiteSetup.connectImageStr, database.ImageDbConnection);
            Assert.AreEqual(TestSuiteSetup.PatientDb, database.PatientDbName);
            Assert.AreEqual(TestSuiteSetup.PatientDataSource, database.PatientDbSource);
            Assert.AreEqual(TestSuiteSetup.connectPatientStr, database.PatientDbConnection);
        }

        [TestMethod]
        [ExpectedException(typeof(UniException))]
        public void BadPassword()
        {
            Assert.IsTrue(db.IsUserLoggedOn());
            db.Disconnect();
            Assert.IsFalse(db.IsUserLoggedOn());
            db.Connect(TestSuiteSetup.AdminUser, "xxx");
        }

        [TestMethod]
        public void Disconnect()
        {
            db.Disconnect();
            Assert.AreEqual("", db.ImageDbName);
            Assert.AreEqual("", db.PatientDbName);
        }

        [TestMethod]
        public void ConnectSA()
        {
            Assert.IsFalse(db.IsSaLoggedOn());
            db.Connect("sa", "password");
            Assert.IsTrue(db.IsSaLoggedOn());
        }

        [TestMethod]
        public void ConnectGuest()
        {
            Assert.IsFalse(db.IsGuestLoggedOn());
            db.Connect("URA-Guest", "password");
            Assert.IsTrue(db.IsGuestLoggedOn());
        }

        [TestMethod]
        public void GetAllImageDetailsForDiskRef()
        {
            ImageRecord[] recs = db.GetAllImageDetailsForDiskRef("UAT");
            Assert.AreEqual(1, recs.Length);
        }

        [TestMethod]
        [ExpectedException(typeof(UniException))]
        public void GetAllImageDetailsForBadDiskRef()
        {
            ImageRecord[] recs = db.GetAllImageDetailsForDiskRef(null);
        }

        [TestMethod]
        [ExpectedException(typeof(UniException))]
        public void GetAllImageDetailsForBadVolunteerRef()
        {
            ImageRecord[] recs = db.GetAllImageDetailsForVolunteerRef(-1);
        }

        [TestMethod]
        [ExpectedException(typeof(UniException))]
        public void GetAllImageDetailsForDiskRefNotLoggedOn()
        {
            db.Disconnect();
            Assert.IsFalse(db.IsUserLoggedOn());
            ImageRecord[] recs = db.GetAllImageDetailsForDiskRef("UAT");
        }

        [TestMethod]
        public void GetUKCTOCSQueryResultsForName()
        {
            String result = db.GetUKCTOCSQueryResults(OsDatabases.ListVolunteerRefForNamesSP, "Stott", "William Philip Quentin");
            Assert.IsTrue(result.IndexOf("Stott") != -1);
        }

        [TestMethod]
        public void GetUKCTOCSQueryResultsForRef()
        {
            String result = db.GetUKCTOCSQueryResults(OsDatabases.ListNameForVolunteerRefSP, "99000002", null);
            Assert.IsTrue(result.IndexOf("Stott") != -1);
        }

        [TestMethod]
        public void GetUKCTOCSQueryScanDatesResultsForRef()
        {
            String result = db.GetUKCTOCSQueryResults(OsDatabases.ListScanDatesForVolunteerRefSP, "99000002", null);
            Assert.IsTrue(result.IndexOf("1/21/2005") != -1);
        }

        [TestMethod]
        public void GetUKCTOCSQueryResultsForBadSP()
        {
            String result = db.GetUKCTOCSQueryResults("xxx", "99000002", null);
            Assert.AreEqual("", result);
        }

        [TestMethod]
        [ExpectedException(typeof(UniException))]
        public void GetUKCTOCSQueryResultsNotLogon()
        {
            db.Disconnect();
            Assert.IsFalse(db.IsUserLoggedOn());
            db.GetUKCTOCSQueryResults(OsDatabases.ListVolunteerRefForNamesSP, "Stott", "William Philip Quentin");
        }

        [TestMethod]
        public void RunGetNameFromVolunteerRef()
        {
            ImageRecord rec = new ImageRecord();
            rec.VolunteerRef = TestSuiteSetup.ValidVolunteerRef;
            Assert.AreEqual(TestSuiteSetup.ValidPatientName, db.GetNameFromVolunteerRef(rec, MockProgram.Instance));
        }

        [TestMethod]
        [ExpectedException(typeof(UniException))]
        public void RunGetNameFromVolunteerRefNoLogon()
        {
            ImageRecord rec = new ImageRecord();
            rec.VolunteerRef = TestSuiteSetup.ValidVolunteerRef;

            db.Disconnect();
            Assert.IsFalse(db.IsUserLoggedOn());
            db.GetNameFromVolunteerRef(rec, MockProgram.Instance);
        }

        [TestMethod]
        public void RunGetNameFromVolunteerRefBadParam1()
        {
            ImageRecord rec = new ImageRecord();
            rec.VolunteerRef = TestSuiteSetup.ValidVolunteerRef;
            Assert.AreEqual("",db.GetNameFromVolunteerRef(null, MockProgram.Instance));
        }

        [TestMethod]
        public void RunGetNameFromBadVolunteerRef()
        {
            ImageRecord rec = new ImageRecord();
            rec.VolunteerRef = "xxx";
            rec.ResetUnspecifiedError();
            Assert.IsFalse(rec.UnspecifiedError);

            Assert.AreEqual("", db.GetNameFromVolunteerRef(rec, MockProgram.Instance));
            Assert.IsTrue(rec.UnspecifiedError);
        }

        [TestMethod]
        [ExpectedException(typeof(UniException))]
        public void RunGetNameFromVolunteerRefBadParam2()
        {
            ImageRecord rec = new ImageRecord();
            rec.VolunteerRef = TestSuiteSetup.ValidVolunteerRef;
            Assert.AreEqual(TestSuiteSetup.ValidPatientName, db.GetNameFromVolunteerRef(rec, null));
        }

        [TestMethod]
        public void RunGetNameFromVolunteerRefNoMatch()
        {
            ImageRecord rec = new ImageRecord();
            rec.VolunteerRef = "99999992";
            Assert.AreEqual("", db.GetNameFromVolunteerRef(rec, MockProgram.Instance));
        }

        [TestMethod]
        public void MatchScanDateForVolunteerRef()
        {
            ImageRecord rec = new ImageRecord();
            rec.AppointmentDate = new DateTime(2005, 1, 21, 0, 0, 0);
            rec.VolunteerRef = TestSuiteSetup.ValidVolunteerRef;
            Assert.IsTrue(db.DoesScanDateExistForVolunteerRef(rec, MockProgram.Instance));
            rec.AppointmentDate = new DateTime(2005, 1, 23, 0, 0, 1);
            Assert.IsFalse(db.DoesScanDateExistForVolunteerRef(rec, MockProgram.Instance));
            rec.AppointmentDate = new DateTime(2005, 1, 24, 0, 0, 0);
            Assert.IsFalse(db.DoesScanDateExistForVolunteerRef(rec, MockProgram.Instance));
        }


        [TestMethod]
        [ExpectedException(typeof(UniException))]
        public void MatchScanDateForVolunteerRefNotLogOn()
        {
            ImageRecord rec = new ImageRecord();
            rec.AppointmentDate = new DateTime(2005, 1, 21, 0, 0, 0);
            rec.VolunteerRef = TestSuiteSetup.ValidVolunteerRef;

            db.Disconnect();
            Assert.IsFalse(db.IsUserLoggedOn());
            db.DoesScanDateExistForVolunteerRef(rec, MockProgram.Instance);
        }

        [TestMethod]
        public void MatchScanDateForInvalidVolunteerRef()
        {
            ImageRecord rec = new ImageRecord();
            rec.AppointmentDate = new DateTime(2005, 1, 21, 0, 0, 0);
            rec.VolunteerRef = "xxx";
            rec.ResetUnspecifiedError();

            Assert.IsFalse(rec.UnspecifiedError);
            Assert.IsFalse(db.DoesScanDateExistForVolunteerRef(rec, MockProgram.Instance));
            Assert.IsTrue(rec.UnspecifiedError);
        }

        [TestMethod]
        public void MatchScanDateForVolunteerRefBadParam1()
        {
            Assert.IsFalse(db.DoesScanDateExistForVolunteerRef(null, MockProgram.Instance));
        }

        [TestMethod]
        [ExpectedException(typeof(UniException))]
        public void MatchScanDateForVolunteerRefBadParam2()
        {
            ImageRecord rec = new ImageRecord();
            rec.AppointmentDate = new DateTime(2005, 1, 21, 0, 0, 0);
            rec.VolunteerRef = TestSuiteSetup.ValidVolunteerRef;
            Assert.IsTrue(db.DoesScanDateExistForVolunteerRef(rec, null));
        }

        [TestMethod]
        public void GetImageIDFromDicomRef()
        {
            ImageRecord rec = new ImageRecord();
            rec.DicomRef = TestSuiteSetup.TestImageDicomRef; 
            Assert.IsTrue(db.GetImageIDForDicomRef(rec, MockProgram.Instance).Length > 0);
        }

        [TestMethod]
        public void GetImageIDFromDicomRefNotFound()
        {
            ImageRecord rec = new ImageRecord();
            rec.DicomRef = "-";
            Assert.IsTrue(db.GetImageIDForDicomRef(rec, MockProgram.Instance).Length == 0);
        }

        [TestMethod]
        [ExpectedException(typeof(UniException))]
        public void GetImageIDFromDicomRefNotLogon()
        {
            ImageRecord rec = new ImageRecord();
            rec.DicomRef = "-";

            db.Disconnect();
            Assert.IsFalse(db.IsUserLoggedOn());

            db.GetImageIDForDicomRef(rec, MockProgram.Instance);
        }

        [TestMethod]
        [ExpectedException(typeof(UniException))]
        public void GetImageIDFromDicomRefBadParam2()
        {
            ImageRecord rec = new ImageRecord();
            rec.DicomRef = "-";

            db.GetImageIDForDicomRef(rec, null);
        }

        [TestMethod]
        public void GetImageIDFromDicomRefBadParam1()
        {
            Assert.AreEqual("(null)", db.GetImageIDForDicomRef(null, MockProgram.Instance));
        }


        [TestMethod]
        public void GetAllImageDetailsForVolunteerRef()
        {
            ImageRecord[] recs = db.GetAllImageDetailsForVolunteerRef(99000002);
            Assert.IsTrue(recs.Length > 0);

        }

        [TestMethod]
        public void LoadImageDetailsForImageID()
        {           //must have inserted record into database - OsDatabasesTestInitialize
            ImageRecord rec = new ImageRecord();
            rec.DicomRef = TestSuiteSetup.TestImageDicomRef;
            rec.ImageID = db.GetImageIDForDicomRef(rec, MockProgram.Instance);
            Assert.IsTrue(rec.ImageID.Length > 0);
            db.LoadImageDetailsForImageID(rec); //throws exception if fails

            Assert.IsTrue(ImageRecord.MatchDicomRefAnyRevision(TestSuiteSetup.TestImageDicomRef, rec.DicomRef));
        }

        [TestMethod]
        [ExpectedException(typeof(UniException))]
        public void LoadImageDetailsForImageIDNoLogon()
        {
            ImageRecord rec = new ImageRecord();
            rec.DicomRef = TestSuiteSetup.TestImageDicomRef;
            rec.ImageID = db.GetImageIDForDicomRef(rec, MockProgram.Instance);

            db.Disconnect();
            Assert.IsFalse(db.IsUserLoggedOn());

            db.LoadImageDetailsForImageID(rec); //throws exception if fails
        }

        [TestMethod]
        [ExpectedException(typeof(UniException))]
        public void LoadImageDetailsForImageIDBadParam()
        {
            db.LoadImageDetailsForImageID(null); //throws exception if fails
        }

        [TestMethod]
        [ExpectedException(typeof(UniException))]
        public void LoadImageDetailsForImageIDBadRecord()
        {
            db.LoadImageDetailsForImageID(new ImageRecord()); //throws exception if fails
        }

        [TestMethod]
        [ExpectedException(typeof(UniException))]
        public void LoadImageFileDataForImageID()
        {           //must have inserted record into database - OsDatabasesTestInitialize
            ImageRecord rec = new ImageRecord();
            rec.DicomRef = TestSuiteSetup.TestImageDicomRef; ;
            rec.ImageID = db.GetImageIDForDicomRef(rec, MockProgram.Instance);
            rec.DicomRef = "";
            db.LoadImageFileDataForImageID(rec); //throws exception if fails
        }

        [TestMethod]
        [ExpectedException(typeof(UniException))]
        public void LoadImageFileDataForImageIDNoLogon()
        {
            ImageRecord rec = new ImageRecord();
            rec.DicomRef = TestSuiteSetup.TestImageDicomRef;
            rec.ImageID = db.GetImageIDForDicomRef(rec, MockProgram.Instance);

            db.Disconnect();
            Assert.IsFalse(db.IsUserLoggedOn());

            db.LoadImageFileDataForImageID(rec); //throws exception if fails
        }

        [TestMethod]
        [ExpectedException(typeof(UniException))]
        public void LoadImageFileDataForImageIDBadParam()
        {
            db.LoadImageFileDataForImageID(null); //throws exception if fails
        }

        [TestMethod]
        [ExpectedException(typeof(UniException))]
        public void LoadImageDFileDataForImageIDBadRecord()
        {
            db.LoadImageFileDataForImageID(new ImageRecord()); //throws exception if fails
        }

        [TestMethod]
        public void StoreImageRecordDuplicate()
        {
            DeleteMaskDirFiles();

            DicomFile file = DicomFileSet.MakeRecordFromFile(ExecutionTime.Instance.MonochromeSampleDicomFile);
            file.Record.ArchiveDate = DateTime.Now;
            file.Record.DiskLabel = "UAT";
            file.ReadFromDisk(db, MockProgram.Instance);
             Assert.IsTrue(db.StoreImageRecord(file, MockProgram.Instance));
            Assert.IsTrue(file.Record.Duplicate);
        }

        [TestMethod]
        [ExpectedException(typeof(UniException))]
        public void StoreImageRecordNoLogon()
        {
            db.Disconnect();
            Assert.IsFalse(db.IsUserLoggedOn());

            DicomFile file = DicomFileSet.MakeRecordFromFile(ExecutionTime.Instance.MonochromeSampleDicomFile);
            file.Record.ArchiveDate = DateTime.Now;
            file.Record.DiskLabel = "UAT";
            file.ReadFromDisk(db, MockProgram.Instance);
            db.StoreImageRecord(file, MockProgram.Instance);
        }

        [TestMethod]
        public void StoreImageRecordBadParam1()
        {
            DicomFile file = DicomFileSet.MakeRecordFromFile(ExecutionTime.Instance.MonochromeSampleDicomFile);
            file.Record.ArchiveDate = DateTime.Now;
            file.Record.DiskLabel = "UAT";
            file.ReadFromDisk(db, MockProgram.Instance);
            Assert.IsFalse(db.StoreImageRecord(null, MockProgram.Instance));
        }

        [TestMethod]
        [ExpectedException(typeof(UniException))]
        public void StoreImageRecordBadParam2()
        {
            DicomFile file = DicomFileSet.MakeRecordFromFile(ExecutionTime.Instance.MonochromeSampleDicomFile);
            file.Record.ArchiveDate = DateTime.Now;
            file.Record.DiskLabel = "UAT";
            file.ReadFromDisk(db, MockProgram.Instance);
            db.StoreImageRecord(file, null);
        }

        [TestMethod]
        public void RestoreImage()
        {           //must have inserted record into database - OsDatabasesTestInitialize
            ImageRecord rec = new ImageRecord();
            rec.DicomRef = TestSuiteSetup.TestImageDicomRef;
            rec.ImageID = db.GetImageIDForDicomRef(rec, MockProgram.Instance);
            Assert.IsTrue(rec.ImageID.Length > 0);
            db.LoadImageDetailsForImageID(rec); //throws exception if fails

            Assert.IsTrue(ImageRecord.MatchDicomRefAnyRevision(TestSuiteSetup.TestImageDicomRef, rec.DicomRef));

            DicomFile file = new DicomFile(rec);
            file.WriteToDisk(db, false, ExecutionTime.Instance.MonochromeSampleDicomFile);
        }

        [TestMethod]
        public void DeleteDiskLabelImages()
        {
            Assert.IsTrue(db.GetAllImageDetailsForDiskRef("UAT").Length == 1);
            db.DeleteDiskLabelImages("UAT");
            Assert.IsTrue(db.GetAllImageDetailsForDiskRef("UAT").Length == 0);
        }
    
        [TestMethod]
        [ExpectedException(typeof(UniException))]
        public void DeleteDiskLabelImagesNoLogOn()
        {
            db.Disconnect();
            Assert.IsFalse(db.IsUserLoggedOn());
            db.DeleteDiskLabelImages("UAT");
        }

        [TestMethod]
        [ExpectedException(typeof(UniException))]
        public void DeleteDiskLabelImagesNoAdminLogOn()
        {
            db.Disconnect();
            db.Connect(TestSuiteSetup.OpUser, TestSuiteSetup.OpUserPassword);
            Assert.IsTrue(db.IsUserLoggedOn());
            db.DeleteDiskLabelImages("UAT");
        }

        [TestMethod]
        public void DeleteImageRecord()
        {
            ImageRecord rec = new ImageRecord();
            rec.DicomRef = TestSuiteSetup.TestImageDicomRef;
            String ImageID = db.GetImageIDForDicomRef(rec, MockProgram.Instance);
            Assert.IsTrue(ImageID.Length > 0);

            db.DeleteImageRecord(ImageID);

            Assert.IsTrue(db.GetImageIDForDicomRef(rec, MockProgram.Instance).Length == 0);
        }

        [TestMethod]
        [ExpectedException(typeof(UniException))]
        public void DeleteImageRecordNotExist()
        {
            db.DeleteImageRecord("xxx");
        }
        [TestMethod]
        [ExpectedException(typeof(UniException))]
        public void DeleteImageRecordAdminNotLoggedOn()
        {
            ImageRecord rec = new ImageRecord();
            rec.DicomRef = TestSuiteSetup.TestImageDicomRef;
            String ImageID = db.GetImageIDForDicomRef(rec, MockProgram.Instance);
            Assert.IsTrue(ImageID.Length > 0);

            db.Disconnect();
            db.Disconnect();
            db.Connect(TestSuiteSetup.OpUser, TestSuiteSetup.OpUserPassword);
            Assert.IsTrue(db.IsUserLoggedOn());

            db.DeleteImageRecord(ImageID);
        }

        [TestMethod]
        [ExpectedException(typeof(UniException))]
        public void DeleteImageRecordNotLoggedOn()
        {
            ImageRecord rec = new ImageRecord();
            rec.DicomRef = TestSuiteSetup.TestImageDicomRef;
            String ImageID = db.GetImageIDForDicomRef(rec, MockProgram.Instance);
            Assert.IsTrue(ImageID.Length > 0);

            db.Disconnect();
            Assert.IsFalse(db.IsUserLoggedOn());

            db.DeleteImageRecord(ImageID);
        }

        [TestMethod]
        public void FailValidateImageData()
        {
            ImageRecord rec = new ImageRecord();
            rec.PatientName = TestSuiteSetup.ValidPatientName;
            rec.VolunteerRef = TestSuiteSetup.ValidVolunteerRef;
            rec.AppointmentDate = TestSuiteSetup.ValidAppointmentDateForVolunteerRef;
            Assert.IsTrue(rec.SetValidationFlags(db, MockProgram.Instance));
            Assert.IsFalse(rec.IsAnyValidationFlagSet());
        }

        [TestMethod]
        public void RunScriptReader()
        {
            string rc = db.RunScript(TestSuiteSetup.PatientDb, "Reader", TestSuiteSetup.SamplePatientDbSQL, OsDatabases.PatientConn);
            Assert.IsTrue(rc.Length > 300);
        }
        [TestMethod]
        public void RunScriptScalar()
        {
            string rc = db.RunScript(TestSuiteSetup.PatientDb, "Scalar", TestSuiteSetup.SamplePatientDbSQL, OsDatabases.PatientConn);
            Assert.IsTrue(int.Parse(rc) > 0);
        }
        [TestMethod]
        public void RunScriptNonQuery()
        {
            string rc = db.RunScript(TestSuiteSetup.PatientDb, "NonQuery", TestSuiteSetup.SamplePatientDbSQL, OsDatabases.PatientConn);
            Assert.AreEqual("-1" + Utils.StrLFCR, rc);
        }
        
        [TestMethod]
        public void RunScriptImage()
        {    //must have inserted record into database
            string rc = db.RunScript(TestSuiteSetup.ImageDb, "Reader", TestSuiteSetup.SampleImageDbSQL, OsDatabases.ImageConn);
            Assert.IsTrue(rc.Length > 365);
        }

        [TestMethod]
        [ExpectedException(typeof(UniException))]
        public void RunScriptLogoff()
        {
            db.Disconnect();
            db.RunScript(TestSuiteSetup.ImageDb, "Reader", TestSuiteSetup.SampleImageDbSQL, OsDatabases.ImageConn);
        }

        [TestMethod]
        [ExpectedException(typeof(UniException))]
        public void RunScriptBadParam1()
        {
            db.RunScript(null, "Reader", TestSuiteSetup.SampleImageDbSQL, OsDatabases.ImageConn);
        }

        [TestMethod]
        [ExpectedException(typeof(UniException))]
        public void RunScriptBadParam2()
        {
            db.RunScript(TestSuiteSetup.ImageDb, null, TestSuiteSetup.SampleImageDbSQL, OsDatabases.ImageConn);
        }
        [TestMethod]
        [ExpectedException(typeof(UniException))]
        public void RunScriptBadParam3()
        {
            db.RunScript(TestSuiteSetup.ImageDb, "Reader", null, OsDatabases.ImageConn);
        }
        [TestMethod]
        [ExpectedException(typeof(UniException))]
        public void RunScriptBadParam4()
        {
            db.RunScript(TestSuiteSetup.ImageDb, "Reader", TestSuiteSetup.SampleImageDbSQL, null);
        }

        [TestMethod]
        [ExpectedException(typeof(UniException))]
        public void RunBadScript()
        {
            db.RunScript(TestSuiteSetup.ImageDb, "Reader", TestSuiteSetup.BadImageDbSQL, OsDatabases.PatientConn);
        }

        [TestMethod]
        [ExpectedException(typeof(UniException))]
        public void BadCmdTypeRunScript()
        {
            string rc = db.RunScript(TestSuiteSetup.PatientDb, "XXX", TestSuiteSetup.SamplePatientDbSQL, OsDatabases.PatientConn);
        }
        [TestMethod]
        [ExpectedException(typeof(UniException))]
        public void BadDatabaseNameRunScript()
        {
            string rc = db.RunScript("XXX", "Reader", TestSuiteSetup.SamplePatientDbSQL, OsDatabases.PatientConn);
        }
        [TestMethod]
        [ExpectedException(typeof(UniException))]
        public void BadLogonRunScript()
        {
            db.Disconnect();
            Assert.IsFalse(db.IsUserLoggedOn());
            string rc = db.RunScript(TestSuiteSetup.PatientDb, "Reader", TestSuiteSetup.SamplePatientDbSQL, OsDatabases.PatientConn);
        }

        [TestMethod]
        public void RunScriptNonQuerySAPatient()
        {
            Assert.IsFalse(db.IsSaLoggedOn());
            db.Connect("sa", "password");
            Assert.IsTrue(db.IsSaLoggedOn());

            string rc = db.RunScript(TestSuiteSetup.PatientDb, "NonQuery", TestSuiteSetup.SamplePatientDbSQL, OsDatabases.PatientConn);
            Assert.AreEqual("-1" + Utils.StrLFCR, rc);
        }
        [TestMethod]
        public void RunScriptNonQuerySAImage()
        {
            Assert.IsFalse(db.IsSaLoggedOn());
            db.Connect("sa", "password");
            Assert.IsTrue(db.IsSaLoggedOn());

            string rc = db.RunScript(TestSuiteSetup.PatientDb, "NonQuery", TestSuiteSetup.SamplePatientDbSQL, OsDatabases.ImageConn);
            Assert.AreEqual("-1" + Utils.StrLFCR, rc);
        }
        [TestMethod]
        [ExpectedException(typeof(UniException))]
        public void RunScriptNonQuerySAUnknown()
        {
            Assert.IsFalse(db.IsSaLoggedOn());
            db.Connect("sa", "password");
            Assert.IsTrue(db.IsSaLoggedOn());

            string rc = db.RunScript(TestSuiteSetup.PatientDb, "NonQuery", TestSuiteSetup.SamplePatientDbSQL, "xxx");
            Assert.AreEqual("-1" + Utils.StrLFCR, rc);
        }

        private void DeleteMaskDirFiles()
        {
            if (Directory.Exists(DicomMask.MaskDir))
            {
                string[] list = Directory.GetFiles(DicomMask.MaskDir, "*.ura");
                foreach (String filename in list)
                    File.Delete(filename);
            }
        }

    }
}
