using System;
using System.Text;
using System.Collections.Generic;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Configuration;
using WillStott.MxAppToolbox;
using osImageManagerLib;
using osImageManagerLibFIT;

namespace osImageManagerLibUnitTests
{
    /// <summary>
    /// Summary description for SqlScriptsSectionTests
    /// </summary>
    [TestClass]
    public class SqlScriptsTests
    {
        public SqlScriptsTests()
        {
        }

        [ClassInitialize()]
        public static void SqlScriptsTestsInitialize(TestContext testContext)
        {
            TestSuiteSetup.CreateStdDirAndSampleFiles(TestSuiteSetup.ProjectRootFromUnitTestRuntime, MockProgram.Instance);
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        [TestMethod]
        public void ScriptSection()
        {
            SqlScriptsSection section = new SqlScriptsSection();
            section.FilePath = "C:\\Bills";
            Assert.AreEqual("C:\\Bills", section.FilePath);
        }

        [TestMethod]
        public void Script()
        {
            SqlScripts scripts = new SqlScripts();
            scripts.Load();
            Assert.AreEqual(ExecutionTime.Instance.ScriptsDir, ExecutionTime.Instance.InstallDir + scripts.Directory + "\\");
        }

        [TestMethod]
        public void ScriptElement()
        {
            SqlScriptElement element = new SqlScriptElement();

            element.Name = "Production";
            element.FileName = "xxx.sql";
            element.DbName = "ImageDb";
            element.ScriptType = "Prod";
            element.CmdType = "Reader";
            element.ShowResults = "yes";
            element.Comment = "PRODSERVER";
            element.RunBy = "Admin";
            element.ConnName = "ImageDb";

            Assert.AreEqual("Production", element.Name);
            Assert.AreEqual("xxx.sql", element.FileName);
            Assert.AreEqual("ImageDb", element.DbName);
            Assert.AreEqual("Prod", element.ScriptType);
            Assert.AreEqual("Reader", element.CmdType);
            Assert.AreEqual("yes", element.ShowResults);
            Assert.AreEqual("PRODSERVER", element.Comment);
            Assert.AreEqual("Admin", element.RunBy);
            Assert.AreEqual("ImageDb", element.ConnName);
            Assert.IsTrue(element.ToString().Contains("ImageDb"));

        }

        [TestMethod]
        public void ScriptInfo()
        {
            SqlScripts scripts = new SqlScripts();
            Assert.AreEqual(ExecutionTime.Instance.ScriptsDir, ExecutionTime.Instance.InstallDir + scripts.Directory + "\\");

            SqlScriptElement[] arr = scripts.GetList();
            Assert.AreEqual(TestSuiteSetup.ConfigDbScripts0Name, arr[0].Name);
            Assert.AreEqual(TestSuiteSetup.ConfigDbScripts0Filename, arr[0].FileName);
            Assert.AreEqual(TestSuiteSetup.ConfigDbScripts0dbName, arr[0].DbName);
            Assert.AreEqual(TestSuiteSetup.ConfigDbScripts0scriptType, arr[0].ScriptType);
            Assert.AreEqual(TestSuiteSetup.ConfigDbScripts0cmdType, arr[0].CmdType);
            Assert.AreEqual(TestSuiteSetup.ConfigDbScripts0ShowResults, arr[0].ShowResults);
            Assert.AreEqual(TestSuiteSetup.ConfigDbScripts0Comment, arr[0].Comment);

        }

        [TestMethod]
        public void Confirm()
        {
            SqlScriptElement element = new SqlScriptElement();
            Assert.IsFalse(element.DoConfirm());
            element.Comment = "lll";
            Assert.IsTrue(element.DoConfirm());
        }
        [TestMethod]
        public void ConfirmMsg()
        {
            SqlScriptElement element = new SqlScriptElement();
            element.Comment = "test";
            Assert.AreEqual("Script: \n\ntest\n\nDo you want to run this script?", element.ConfirmMsg());
        }
        [TestMethod]
        public void GetScript()
        {
          SqlScriptElement element = new SqlScriptElement();
          element.FileName = TestSuiteSetup.SampleDbScript;
          Assert.AreEqual(TestSuiteSetup.SampleDbScriptHeader, element.GetScript(ExecutionTime.Instance.ScriptsDir).Substring(0, TestSuiteSetup.SampleDbScriptHeader.Length));
        }
        [TestMethod]
        [ExpectedException(typeof(UniException))]
        public void BadScript()
        {
            SqlScriptElement element = new SqlScriptElement();
            element.GetScript("xxx");
        }

    }
}
