using System;
using System.Text;
using System.Collections.Generic;
using System.Data.SqlClient;

using Microsoft.VisualStudio.TestTools.UnitTesting;
using WillStott.MxAppToolbox;
using osImageManagerLib;
using osImageManagerLibFIT;

namespace osImageManagerLibUnitTests
{
    /// <summary>
    /// Summary description for SqlUtilsTests
    /// </summary>
    [TestClass]
    public class SqlUtilsTests
    {
        public SqlUtilsTests()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        [ClassInitialize()]
        public static void SqlUtilsTestsInitialize(TestContext testContext)
        {
            TestSuiteSetup.CreateStdDirAndSampleFiles(TestSuiteSetup.ProjectRootFromUnitTestRuntime, MockProgram.Instance);
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        [TestMethod]
        public void getConnectionElement()
        {
            Assert.AreEqual(TestSuiteSetup.PatientDb, SqlUtils.getConnectionElement("Data Source=XXX;Database=ukctocs;Integrated Security=false;", "Database="));
            Assert.AreEqual("", SqlUtils.getConnectionElement("Data Source=XXX;Database=URATestIntegrated Security=false", "Database="));
            Assert.AreEqual("", SqlUtils.getConnectionElement("Data Source=XXX;Databse=URATest;Integrated Security=false;", "Database="));
        }

        [TestMethod]
        public void getConnectionElementBad()
        {
            Assert.AreEqual("", SqlUtils.getConnectionElement(null, "Database="));
            Assert.AreEqual("", SqlUtils.getConnectionElement(null, "Database="));
        }

        [TestMethod]
        public void CreateVarCharParamBad()
        {
            Assert.IsNull(SqlUtils.CreateVarCharParam(null, "xxx", 0, false, ""));
            Assert.IsNull(SqlUtils.CreateVarCharParam(new SqlCommand(), null, 0, false, ""));
        }
        [TestMethod]
        public void CreateIntParamBad()
        {
            Assert.IsNull(SqlUtils.CreateIntParam(new SqlCommand(), null, false, 0));
            Assert.IsNull(SqlUtils.CreateIntParam(null, "xxx", false, 0));
        }
        [TestMethod]
        public void CreateDateTimeParamBad()
        {
            Assert.IsNull(SqlUtils.CreateDateTimeParam(new SqlCommand(), null, false, new System.DateTime()));
            Assert.IsNull(SqlUtils.CreateDateTimeParam(null, "xxx", false, new System.DateTime()));
        }
        [TestMethod]
        public void CreateDateTimeDirection()
        {
            SqlParameter param = SqlUtils.CreateDateTimeParam(new SqlCommand(), "xxx", true, new System.DateTime());
            Assert.AreEqual(System.Data.ParameterDirection.Output, param.Direction);
        }

        [TestMethod]
        public void CreateUniqueIDParamBad()
        {
            Assert.IsNull(SqlUtils.CreateUniqueIDParam(new SqlCommand(), null, false, new System.Guid()));
            Assert.IsNull(SqlUtils.CreateUniqueIDParam(null, "xxx", false, new System.Guid()));
        }

        [TestMethod]
        public void CreateVarBinaryParamBad()
        {
            Assert.IsNull(SqlUtils.CreateVarBinaryParam(new SqlCommand(), null, 0, false, null));
            Assert.IsNull(SqlUtils.CreateVarBinaryParam(null, "xxx", 0, false, null));
        }

        [TestMethod]
        public void CreateVarBinaryParam()
        {
            SqlParameter param = SqlUtils.CreateVarBinaryParam(new SqlCommand(), "xxx", 0, true, null);
            Assert.AreEqual(System.Data.ParameterDirection.Output, param.Direction);
            param = SqlUtils.CreateVarBinaryParam(new SqlCommand(), "xxx", 0, false, null);
            Assert.AreEqual(System.Data.ParameterDirection.Input, param.Direction);
        }

        [TestMethod]
        public void CreateImageParamBad()
        {
            Assert.IsNull(SqlUtils.CreateImageParam(new SqlCommand(), null, 0, false, null));
            Assert.IsNull(SqlUtils.CreateImageParam(null, "xxx", 0, false, null));
        }

        [TestMethod]
        public void CreateImageParam()
        {
            SqlParameter param = SqlUtils.CreateImageParam(new SqlCommand(), "xxx", 0, true, null);
            Assert.AreEqual(System.Data.ParameterDirection.Output, param.Direction);
            param = SqlUtils.CreateImageParam(new SqlCommand(), "xxx", 0, false, null);
            Assert.AreEqual(System.Data.ParameterDirection.Input, param.Direction);
        }

        [TestMethod]
        public void ExecuteScriptParamBad()
        {
            Assert.AreEqual("", SqlUtils.ExecuteScript(new SqlConnection(), null, ""));
            Assert.AreEqual("", SqlUtils.ExecuteScript(null, "xxx", ""));
        }

        [TestMethod]
        public void GetResultsFromCommandExecutionParamBad()
        {
            Assert.AreEqual("", SqlUtils.GetResultsFromCommandExecution(null));
        } 
    }
}
